@@ -1,6 +1,6 @@
 // cygnal.cpp:  GNU streaming Flash media server, for Gnash.
 // 
-// Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+// Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -1085,7 +1085,7 @@ event_handler(Network::thread_params_t *args)
 			  hand->removeClient(args->netfd);
 			  done = true;
 		      } else {
-			  log_network("Not Done with HTTP connection for fd #%d, it's a persistant connection.", i);
+			  log_network("Not Done with HTTP connection for fd #%d, it's a persistent connection.", i);
 			  
 		      }
 		      continue;
