@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,7 +17,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include <map>
@@ -26,7 +27,8 @@
 #include "dejagnu.h"
 #include "fn_call.h"
 #include "as_object.h"
-#include "builtin_function.h" // need builtin_function
+#include "as_function.h"
+#include "Global_as.h"
 
 using namespace std;
 
@@ -37,44 +39,39 @@ as_value dejagnu_pass(const fn_call& fn);
 as_value dejagnu_fail(const fn_call& fn);
 as_value dejagnu_totals(const fn_call& fn);
 
-LogFile& dbglogfile = LogFile::getDefaultInstance();
-
-class dejagnu_as_object : public as_object
+class DejaGnu : public Relay
 {
 public:
-    DejaGnu obj;
+    DejaGnu();
+    ~DejaGnu();
+    const char *pass (const char *msg);
+    const char *fail (const char *msg);
+    const char *xpass (const char *msg);
+    const char *xfail (const char *msg);
+    void totals ();
+private:
+    int passed;
+    int failed;
+    int xpassed;
+    int xfailed;
 };
 
 static void
-attachInterface(as_object *obj)
+attachInterface(as_object& obj)
 {
-//    GNASH_REPORT_FUNCTION;
-
-    obj->init_member("pass", new builtin_function(dejagnu_pass));
-    obj->init_member("fail", new builtin_function(dejagnu_fail));
-    obj->init_member("totals", new builtin_function(dejagnu_totals));
-}
-
-static as_object*
-getInterface()
-{
-//    GNASH_REPORT_FUNCTION;
-    static boost::intrusive_ptr<as_object> o;
-    if (o == NULL) {
-	o = new as_object();
-    }
-    return o.get();
+    Global_as& gl = getGlobal(obj);
+    
+    obj.init_member("pass", gl.createFunction(dejagnu_pass));
+    obj.init_member("fail", gl.createFunction(dejagnu_fail));
+    obj.init_member("totals", gl.createFunction(dejagnu_totals));
 }
 
 static as_value
-dejagnu_ctor(const fn_call& /* fn */)
+dejagnu_ctor(const fn_call& fn)
 {
-//    GNASH_REPORT_FUNCTION;
-    dejagnu_as_object* obj = new dejagnu_as_object();
-
-    attachInterface(obj);
-    return as_value(obj); // will keep alive
-//    printf ("Hello World from %s !!!\n", __PRETTY_FUNCTION__);
+    as_object* obj = ensure<ValidThis>(fn);
+    obj->setRelay(new DejaGnu());
+    return as_value(); 
 }
 
 
@@ -92,20 +89,16 @@ DejaGnu::~DejaGnu()
 const char *
 DejaGnu::pass (const char *msg)
 {
-//    GNASH_REPORT_FUNCTION;
-
     passed++;
-    dbglogfile << "PASSED: " << msg << endl;
+    log_debug("PASSED: %s\n", msg);
     return NULL;
 }
 
 const char *
 DejaGnu::fail (const char *msg)
 {
-//    GNASH_REPORT_FUNCTION;
-
     failed++;
-    dbglogfile << "FAILED: " << msg << endl;
+    log_debug("FAILED: %s\n", msg);
     return NULL;
 }
 
@@ -113,59 +106,52 @@ as_value
 dejagnu_pass(const fn_call& fn)
 {
 //    GNASH_REPORT_FUNCTION;
-    boost::intrusive_ptr<dejagnu_as_object> ptr = ensureType<dejagnu_as_object>(fn.this_ptr);
+    DejaGnu* ptr = ensure<ThisIsNative<DejaGnu> >(fn);
     
     if (fn.nargs > 0) {
 	string text = fn.arg(0).to_string();
-	return as_value(ptr->obj.pass(text.c_str()));
+	return as_value(ptr->pass(text.c_str()));
     }
+
+    return as_value();
 }
 
 as_value
 dejagnu_fail(const fn_call& fn)
 {
 //    GNASH_REPORT_FUNCTION;
-    boost::intrusive_ptr<dejagnu_as_object> ptr = ensureType<dejagnu_as_object>(fn.this_ptr);
+    DejaGnu* ptr = ensure<ThisIsNative<DejaGnu> >(fn);
     
     if (fn.nargs > 0) {
-	string text = fn.arg(0).to_string();
-	return as_value(ptr->obj.fail(text.c_str()));
+        string text = fn.arg(0).to_string();
+        return as_value(ptr->fail(text.c_str()));
     }
+
+    return as_value();
 }
 
 as_value
 dejagnu_totals(const fn_call& fn)
 {
-//    GNASH_REPORT_FUNCTION;
-    boost::intrusive_ptr<dejagnu_as_object> ptr = ensureType<dejagnu_as_object>(fn.this_ptr);
+    DejaGnu* ptr = ensure<ThisIsNative<DejaGnu> >(fn);
     
-    ptr->obj.totals();
+    ptr->totals();
     return as_value(true);
 }
 
     
-std::auto_ptr<as_object>
-init_dejagnu_instance()
-{
-    return std::auto_ptr<as_object>(new dejagnu_as_object());
-}
-
 extern "C" {
-    void
-    dejagnu_class_init(as_object &obj)
-    {
-//	GNASH_REPORT_FUNCTION;
-	// This is going to be the global "class"/"function"
-	static boost::intrusive_ptr<builtin_function> cl;
-	if (cl == NULL) {
-	    cl = new builtin_function(&dejagnu_ctor, getInterface());
-// 	    // replicate all interface to class, to be able to access
-// 	    // all methods as static functions
- 	    attachInterface(cl.get());
-	}
+void
+dejagnu_class_init(as_object &obj)
+{
+    Global_as& gl = getGlobal(obj);
+    as_object* proto = createObject(gl);
+    attachInterface(*proto);
+
+    as_object* cl = gl.createClass(&dejagnu_ctor, proto);
 	
-	obj.init_member("DejaGnu", cl.get());
-    }
+	obj.init_member("DejaGnu", cl);
+}
 } // end of extern C
 
 
