@@ -1,6 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -36,10 +36,9 @@
 #include "fn_call.h"
 #include "as_object.h"
 #include "as_function.h"
-#include "builtin_function.h" // need builtin_function
 #include "debugger.h"
 #include "gtkext.h"
-//#include "Player.h"
+#include "as_function.h"
 
 using namespace std;
 
@@ -48,11 +47,6 @@ namespace gnash
 
 #define dbglogfile cerr		// FIXME: use log_*()
 
-#ifdef USE_DEBUGGER
-static Debugger& debugger = Debugger::getDefaultInstance();
-#endif
-//Player& player = Player::getDefaultInstance();
-
 // prototypes for the callbacks required by Gnash
 as_value gtkext_window_new(const fn_call& fn);
 as_value gtkext_signal_connect(const fn_call& fn);
@@ -105,8 +99,8 @@ generic_callback(GtkWidget * /*widget*/, gpointer data)
 	cerr << "event is: \"" << event << "\"" << endl;
     }
     
-    as_value	val;
-    as_environment env(VM::get());
+    as_value val;
+    as_environment env(getVM(*as_func));
 
     fn_call::Args args;
     args += handler, event, handler;
