@@ -1,6 +1,7 @@
 // Player.cpp:  Top level SWF player, for gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,21 +22,25 @@
 #include "gnashconfig.h"
 #endif
 
-#ifndef DEFAULT_GUI
-# define DEFAULT_GUI "NULL"
-#endif
+#include "Player.h"
+
+#include <iostream>
+#include <sstream>
+#include <boost/lexical_cast.hpp>
+#include <boost/variant/static_visitor.hpp>
+#include <boost/any.hpp>
+#include <utility>
+#include <memory>
+#include <vector>
 
 #include "gui.h"
 #include "NullGui.h"
-
 #include "MovieFactory.h"
 #include "movie_definition.h"
 #include "sound_handler.h" // for set_sound_handler and create_sound_handler_*
 #include "MovieClip.h" // for setting FlashVars
 #include "movie_root.h" 
-#include "Player.h"
 #include "StreamProvider.h"
-
 #include "swf/TagLoadersTable.h"
 #include "swf/DefaultTagLoaders.h"
 #include "NamingPolicy.h"
@@ -46,28 +51,134 @@
 #include "noseek_fd_adapter.h"
 #include "VM.h"
 #include "SystemClock.h"
-
-#ifdef USE_FFMPEG
-# include "MediaHandlerFfmpeg.h"
-#elif defined(USE_GST)
-# include "MediaHandlerGst.h"
-#endif
-
+#include "ExternalInterface.h"
+#include "ScreenShotter.h"
 #include "GnashSystemIOHeaders.h" // for write() 
 #include "log.h"
-#include <iostream>
-#include <sstream>
-#include <iomanip>
+#include "HostInterface.h"
 
 using namespace gnash;
 
 namespace {
-gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
+    gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 }
 
-/// @todo Shouldn't Player be in 'gnash' namespace ?
+namespace {
+
+class MessageHandler : public boost::static_visitor<boost::any>
+{
+public:
+    explicit MessageHandler(Gui& g) : _gui(g) {}
+
+    boost::any operator()(const HostMessage& e) {
+
+        switch (e.event()) {
+
+            case HostMessage::NOTIFY_ERROR:
+                _gui.error(boost::any_cast<std::string>(e.arg()));
+                return boost::blank();
+
+            case HostMessage::QUERY:
+                return _gui.yesno(boost::any_cast<std::string>(e.arg()));
+
+            case HostMessage::SHOW_MOUSE:
+            {
+                // Must return a bool, true if the mouse was visible before.
+                return _gui.showMouse(boost::any_cast<bool>(e.arg()));   
+            }
+
+            case HostMessage::SET_DISPLAYSTATE:
+            {
+                const movie_root::DisplayState s =
+                    boost::any_cast<movie_root::DisplayState>(e.arg());
+                if (s == movie_root::DISPLAYSTATE_FULLSCREEN) {
+                    _gui.setFullscreen();
+                }
+                else if (s == movie_root::DISPLAYSTATE_NORMAL) {
+                    _gui.unsetFullscreen();
+                }
+                return boost::blank();
+            }
+
+            case HostMessage::UPDATE_STAGE:
+                _gui.updateStageMatrix();
+                return boost::blank();
+
+            case HostMessage::SHOW_MENU:
+                _gui.showMenu(boost::any_cast<bool>(e.arg()));
+                return boost::blank();
+
+            case HostMessage::SET_CLIPBOARD:
+                _gui.setClipboard(boost::any_cast<std::string>(e.arg()));
+                return boost::blank();
+
+            case HostMessage::RESIZE_STAGE:
+            {
+                if (_gui.isPlugin()) {
+                    log_debug("Player doing nothing on Stage.resize as we're a plugin");
+                    return boost::blank();
+                }
+
+                typedef std::pair<int, int> Dimensions;
+                const Dimensions i = boost::any_cast<Dimensions>(e.arg());
+                _gui.resizeWindow(i.first, i.second);
+                return boost::blank();
+            }
+            case HostMessage::EXTERNALINTERFACE_ISPLAYING:
+                return (_gui.isStopped()) ? false : true;
+
+            case HostMessage::EXTERNALINTERFACE_PAN:
+                log_unimpl("GUI ExternalInterface.Pan event");
+                return boost::blank();
+
+            case HostMessage::EXTERNALINTERFACE_PLAY:
+                _gui.play();
+                return boost::blank();
+
+            case HostMessage::EXTERNALINTERFACE_REWIND:
+                _gui.restart();
+                return boost::blank();
+
+            case HostMessage::EXTERNALINTERFACE_SETZOOMRECT:
+                log_unimpl("GUI ExternalInterface.SetZoomRect event");
+                return boost::blank();
+
+            case HostMessage::EXTERNALINTERFACE_STOPPLAY:
+                _gui.pause();
+                return boost::blank();
+
+            case HostMessage::EXTERNALINTERFACE_ZOOM:
+                log_unimpl("GUI ExternalInterface.Zoom event");
+                return boost::blank();
+
+            case HostMessage::SCREEN_RESOLUTION:
+                return _gui.screenResolution();
+
+            case HostMessage::SCREEN_DPI:
+                return _gui.getScreenDPI();
+
+            case HostMessage::SCREEN_COLOR:
+                return _gui.getScreenColor();
+
+            case HostMessage::PIXEL_ASPECT_RATIO:
+                return _gui.getPixelAspectRatio();
+
+            case HostMessage::PLAYER_TYPE:
+                return std::string(_gui.isPlugin() ? "PlugIn" : "StandAlone");
+        }
+        log_error(_("Unhandled callback %s with arguments %s"), +e.event());
+        return boost::blank();
+    }
+
+    boost::any operator()(const CustomMessage& /*e*/) {
+        return boost::blank();
+    }
+private:
+    Gui& _gui;
+};
+
+}
 
-/*static private*/
 void
 Player::setFlashVars(const std::string& varstr)
 {
@@ -90,6 +201,8 @@ Player::Player()
     _delay(0),
     _width(0),
     _height(0),
+    _xPosition(-1),
+    _yPosition(-1),
     _windowID(0),
     _doLoop(true),
     _doRender(true),
@@ -101,10 +214,11 @@ Player::Player()
     _fpsDebugTime(0.0),
 #endif
     _hostfd(-1),
+    _controlfd(-1),
     _startFullscreen(false),
-    _hideMenu(false)
+    _hideMenu(false),
+    _screenshotQuality(100)
 {
-    init();
 }
 
 float
@@ -115,14 +229,6 @@ Player::setScale(float newscale)
     return oldscale;
 }
 
-void
-Player::init()
-{
-    /// Initialize gnash core library
-    gnashInit();
-
-}
-
 void
 Player::init_logfile()
 {
@@ -163,91 +269,52 @@ Player::init_logfile()
 
 }
 
-unsigned int
-Player::silentStream(void* /*udata*/, boost::int16_t* stream, unsigned int len, bool& atEOF)
-{
-    std::fill(stream, stream+len, 0);
-    atEOF=false;
-    return len;
-}
-
 void
 Player::init_sound()
 {
 
     if (_doSound) {
+        try {
 #ifdef SOUND_SDL
-        try {
-            _soundHandler.reset(sound::create_sound_handler_sdl(_audioDump));
-        } catch (SoundException& ex) {
-            log_error(_("Could not create sound handler: %s."
-                " Will continue w/out sound."), ex.what());
-        }
-        if (! _audioDump.empty()) {
-            // add a silent stream to the audio pool so that our output file
-            // is homogenous;  we actually want silent wave data when no sounds
-            // are playing on the stage
-            _soundHandler->attach_aux_streamer(silentStream, (void*) this);
-        }
+            _soundHandler.reset(sound::create_sound_handler_sdl(
+                        _mediaHandler.get()));
 #elif defined(SOUND_AHI)
-        try {
-            _soundHandler.reset(sound::create_sound_handler_aos4(_audioDump));
-        } catch (SoundException& ex) {
+            _soundHandler.reset(sound::create_sound_handler_aos4(
+                        _mediaHandler.get()));
+#elif defined(SOUND_MKIT)
+            _soundHandler.reset(sound::create_sound_handler_mkit(
+                        _mediaHandler.get()));
+#else
+            log_error(_("Sound requested but no sound support compiled in"));
+            return;
+#endif
+
+        } catch (const SoundException& ex) {
             log_error(_("Could not create sound handler: %s."
                 " Will continue w/out sound."), ex.what());
         }
-        if (! _audioDump.empty()) {
-            // add a silent stream to the audio pool so that our output file
-            // is homogenous;  we actually want silent wave data when no sounds
-            // are playing on the stage
-            _soundHandler->attach_aux_streamer(silentStream, (void*) this);
-        }
-#elif defined(SOUND_GST)
-        _soundHandler.reset(media::create_sound_handler_gst());
-#else
-        log_error(_("Sound requested but no sound support compiled in"));
-        return;
-#endif
     }
 }
 
-void
-Player::init_media()
-{
-#ifdef USE_FFMPEG
-        _mediaHandler.reset( new gnash::media::ffmpeg::MediaHandlerFfmpeg() );
-#elif defined(USE_GST)
-        _mediaHandler.reset( new gnash::media::gst::MediaHandlerGst() );
-#else
-        log_error(_("No media support compiled in"));
-        return;
-#endif
-        
-        gnash::media::MediaHandler::set(_mediaHandler);
-}
-
-
 void
 Player::init_gui()
 {
-    if ( _doRender )
-    {
-        _gui = getGui(); 
-    }
-    else
-    {
+    if (_doRender) {
+        _gui = getGui();
+    } else {
         _gui.reset(new NullGui(_doLoop, *_runResources));
     }
 
+    _gui->setAudioDump(_audioDump);
     _gui->setMaxAdvances(_maxAdvances);
 
 #ifdef GNASH_FPS_DEBUG
-    if ( _fpsDebugTime )
-    {
-        log_debug(_("Activating FPS debugging every %g seconds"), _fpsDebugTime);
+    if (_fpsDebugTime) {
+        log_debug(_("Activating FPS debugging every %g seconds"),
+                _fpsDebugTime);
         _gui->setFpsTimerInterval(_fpsDebugTime);
     }
-#endif // def GNASH_FPS_DEBUG
+#endif
 }
 
 boost::intrusive_ptr<movie_definition>
@@ -262,8 +329,7 @@ Player::load_movie()
     RcInitFile& rcfile = RcInitFile::getDefaultInstance();
     URL vurl(_url);
 
-    if ( vurl.protocol() == "file" )
-    {
+    if (vurl.protocol() == "file") {
         const std::string& path = vurl.path();
         size_t lastSlash = path.find_last_of('/');
         std::string dir = path.substr(0, lastSlash+1);
@@ -272,17 +338,14 @@ Player::load_movie()
     }
 
     try {
-        if ( _infile == "-" )
-        {
+        if (_infile == "-") {
             std::auto_ptr<IOChannel> in (
-                    noseek_fd_adapter::make_stream(fileno(stdin)));
+                noseek_fd_adapter::make_stream(fileno(stdin)));
             md = MovieFactory::makeMovie(in, _url, *_runResources, false);
         }
-        else
-        {
+        else {
             URL url(_infile);
-            if ( url.protocol() == "file" )
-            {
+            if ( url.protocol() == "file" ) {
                 std::string path = url.path();
                 // We'll need to allow load of the file, no matter virtual url
                 // specified...
@@ -299,13 +362,13 @@ Player::load_movie()
             md = MovieFactory::makeMovie(url, *_runResources, _url.c_str(),
                     false);
         }
-    } catch (const GnashException& er) {
+    }
+    catch (const GnashException& er) {
         std::cerr << er.what() << std::endl;
         md = NULL;
     }
 
-    if ( ! md )
-    {
+    if (!md) {
         fprintf(stderr, "Could not load movie '%s'\n", _infile.c_str());
         return NULL;
     }
@@ -313,18 +376,15 @@ Player::load_movie()
     return md;
 }
 
-/* \brief Run, used to open a new flash file. Using previous initialization */
-int
+/// \brief Run, used to open a new flash file. Using previous initialization
+void
 Player::run(int argc, char* argv[], const std::string& infile,
         const std::string& url)
 {
-    
     // Call this at run() time, so the caller has
     // a cache of setting some parameter before calling us...
     // (example: setDoSound(), setWindowId() etc.. ) 
     init_logfile();
-    init_media();
-    init_sound();
    
     // gnash.cpp should check that a filename is supplied.
     assert (!infile.empty());
@@ -332,113 +392,112 @@ Player::run(int argc, char* argv[], const std::string& infile,
     _infile = infile;
 
     // Work out base url
-    if ( _baseurl.empty() )
-    {
-        if (! url.empty() ) _baseurl = url;
-        else if ( infile == "-" ) _baseurl = URL("./").str();
+    if (_baseurl.empty()) {
+        if (!url.empty()) _baseurl = url;
+        else if (infile == "-") _baseurl = URL("./").str();
         else _baseurl = infile;
     }
 
     // Set _root._url (either explicit of from infile)
-    if (! url.empty() ) {
+    if (!url.empty()) {
         _url = url;
-    }  else {
+    } else {
         _url = infile;
     }
 
-    // These flags are here so we can construct
-    // the correct URL for base url later.
-    // If the URL class was not immutable we could do something smarter...
-    bool hasOverriddenBaseUrl=false;
-    std::string overriddenBaseUrl;
-
-    URL baseURL = hasOverriddenBaseUrl ? URL(overriddenBaseUrl, URL(_baseurl))
-                                       : URL(_baseurl);
-
+    // Parse player parameters. These are not passed to the SWF, but rather
+    // control stage properties etc.
+    // NOTE: it is intentional to force a trailing slash to "base" argument
+    //       as it was tested that the "base" argument is always considered
+    //       a directory!
+    Params::const_iterator it = _params.find("base");
+    const URL baseURL = (it == _params.end()) ? _baseurl :
+                                               URL(it->second+"/", _baseurl);
     /// The RunResources should be populated before parsing.
-    _runResources.reset(new RunResources(baseURL.str()));
-    _runResources->setSoundHandler(_soundHandler);
+    _runResources.reset(new RunResources());
 
     boost::shared_ptr<SWF::TagLoadersTable> loaders(new SWF::TagLoadersTable());
     addDefaultLoaders(*loaders);
     _runResources->setTagLoaders(loaders);
 
     std::auto_ptr<NamingPolicy> np(new IncrementalRename(_baseurl));
-    boost::shared_ptr<StreamProvider> sp(new StreamProvider(np));
+
+    /// The StreamProvider uses the actual URL of the loaded movie.
+    boost::shared_ptr<StreamProvider> sp(new StreamProvider(_url, baseURL, np));
 
     _runResources->setStreamProvider(sp);
+
+    // Set the Hardware video decoding resources. none, vaapi, omap
+    _runResources->setHWAccelBackend(_hwaccel);
+    // Set the Renderer resource, opengl, agg, or cairo
+    _runResources->setRenderBackend(_renderer);
+
+    _mediaHandler.reset(media::MediaFactory::instance().get(_media));
+
+    if (!_mediaHandler.get()) {
+        boost::format fmt =
+            boost::format(_("Non-existent media handler %1% specified"))
+            % _media;
+        throw GnashException(fmt.str());
+    }
+
+    _runResources->setMediaHandler(_mediaHandler);
+    
+    init_sound();
+    _runResources->setSoundHandler(_soundHandler);
     
     init_gui();
 
     // Initialize gui (we need argc/argv for this)
     // note that this will also initialize the renderer
     // which is *required* during movie loading
-    if ( ! _gui->init(argc, &argv) )
-    {
-        std::cerr << "Could not initialize gui." << std::endl;
-        return EXIT_FAILURE;
+    if (!_gui->init(argc, &argv)) {
+        throw GnashException("Could not initialize GUI");
     }
 
     // Parse querystring (before FlashVars, see
     // testsuite/misc-ming.all/FlashVarsTest*)
     setFlashVars(URL(_url).querystring());
-    
-    // Parse parameters
-    StringNoCaseEqual noCaseCompare;
-    for ( std::map<std::string,std::string>::const_iterator it=params.begin(),
-        itEnd=params.end(); it != itEnd; ++it)
-    {
-        if ( noCaseCompare(it->first, "flashvars") )
-        {
-            setFlashVars(it->second);
-            continue;
-        }
 
-        if ( noCaseCompare(it->first, "base") )
-        {
-            hasOverriddenBaseUrl=true;
-            overriddenBaseUrl=it->second;
-            continue;
-        }
+    // Add FlashVars.
+    Params::const_iterator fv = _params.find("flashvars");
+    if (fv != _params.end()) {
+        setFlashVars(fv->second);
     }
 
-
     // Load the actual movie.
     _movieDef = load_movie();
-    if ( ! _movieDef )
-    {
-        return EXIT_FAILURE;
+    if (!_movieDef) {
+        throw GnashException("Could not load movie!");
     }
 
-
     // Get info about the width & height of the movie.
-    int movie_width = static_cast<int>(_movieDef->get_width_pixels());
-    int movie_height = static_cast<int>(_movieDef->get_height_pixels());
+    const size_t movie_width = _movieDef->get_width_pixels();
+    const size_t movie_height = _movieDef->get_height_pixels();
 
     if (! _width) {
-      _width = static_cast<size_t>(movie_width * _scale);
+        _width = static_cast<size_t>(movie_width * _scale);
     }
     if (! _height) {
-      _height = static_cast<size_t>(movie_height * _scale);
+        _height = static_cast<size_t>(movie_height * _scale);
     }
 
-    if ( ! _width || ! _height )
-    {
+    if (!_width || !_height) {
         log_debug(_("Input movie has collapsed dimensions "
                     "%d/%d. Setting to 1/1 and going on."),
                      _width, _height);
-        if ( ! _width ) _width = 1;
-        if ( ! _height ) _height = 1;
+        if (!_width) _width = 1;
+        if (!_height) _height = 1;
     }
 
     // Register movie definition before creating the window
     _gui->setMovieDefinition(_movieDef.get());
 
     // Now that we know about movie size, create gui window.
-    _gui->createWindow(_url.c_str(), _width, _height);
+    _gui->createWindow(_url.c_str(), _width, _height, _xPosition, _yPosition);
 
     movie_root root(*_movieDef, _gui->getClock(), *_runResources);
-
+    
     _callbacksHandler.reset(new CallbacksHandler(*_gui, *this)); 
     
     // Register Player to receive events from the core (Mouse, Stage,
@@ -448,11 +507,20 @@ Player::run(int argc, char* argv[], const std::string& infile,
     // Register Player to receive FsCommand events from the core.
     root.registerFSCommandCallback(_callbacksHandler.get());
 
+    log_debug("Player Host FD #%d, Player Control FD #%d", 
+                     _hostfd, _controlfd);
+    
     // Set host requests fd (if any)
-    if ( _hostfd != -1 ) root.setHostFD(_hostfd);
+    if ( _hostfd != -1 ) {
+        root.setHostFD(_hostfd);
+    }
+    
+    if (_controlfd != -1) {
+        root.setControlFD(_controlfd);        
+    }
 
     _gui->setStage(&root);
-
+    
     // When startStopped is true, stop here after the stage has been 
     // registered, but before the movie has started. Initial loading
     // and VM initialization have been done by this stage, but not
@@ -460,8 +528,8 @@ Player::run(int argc, char* argv[], const std::string& infile,
     // the Gui accesses movie_root to get the sound_handler, but also
     // because the gui window should be properly set up by this point.
     RcInitFile& rcfile = RcInitFile::getDefaultInstance();
-    if ( rcfile.startStopped() )
-    {
+
+    if (rcfile.startStopped()) {
         _gui->stop();
     }
 
@@ -476,9 +544,8 @@ Player::run(int argc, char* argv[], const std::string& infile,
     _movieDef->completeLoad();
 
     if (! _delay) {
-      //float movie_fps = _movieDef->get_frame_rate();
-      //_delay = static_cast<unsigned int>(1000 / movie_fps) ; // milliseconds per frame
-      _delay = 10; // 10ms per heart beat
+        // 10ms per heart beat
+        _delay = 10; 
     }
     _gui->setInterval(_delay);
 
@@ -494,105 +561,116 @@ Player::run(int argc, char* argv[], const std::string& infile,
         _gui->hideMenu();
     }
     
+    // Now handle stage alignment and scale mode. This should be done after
+    // the GUI is created, after its stage member is set, and after the
+    // interface callbacks are registered.
+    it = _params.find("salign");
+    if (it != _params.end()) {
+        log_debug("Setting align");
+        const short align = stringToStageAlign(it->second);
+        root.setStageAlignment(align);
+    }
+
+    it = _params.find("allowscriptaccess");
+    if (it != _params.end()) {
+        std::string access = it->second;
+        StringNoCaseEqual noCaseCompare;
+        const std::string& str = it->second;
+                
+        movie_root::AllowScriptAccessMode mode = 
+            movie_root::SCRIPT_ACCESS_SAME_DOMAIN;
+        
+        if (noCaseCompare(str, "never")) {
+            mode = movie_root::SCRIPT_ACCESS_NEVER;
+        } 
+        else if (noCaseCompare(str, "sameDomain")) {
+            mode = movie_root::SCRIPT_ACCESS_SAME_DOMAIN;
+        } 
+        else if (noCaseCompare(str, "always")) {
+            mode = movie_root::SCRIPT_ACCESS_ALWAYS;
+        }
+        log_debug("Setting allowscriptaccess to %s", mode);
+        root.setAllowScriptAccess(mode);
+    }
+
+    it = _params.find("scale");
+    if (it != _params.end()) {                
+        StringNoCaseEqual noCaseCompare;
+        const std::string& str = it->second;
+        movie_root::ScaleMode mode = movie_root::SCALEMODE_SHOWALL;
+        
+        if (noCaseCompare(str, "noScale")) {
+            mode = movie_root::SCALEMODE_NOSCALE;
+        } 
+        else if (noCaseCompare(str, "exactFit")) {
+            mode = movie_root::SCALEMODE_EXACTFIT;
+        }
+        else if (noCaseCompare(str, "noBorder")) {
+            mode = movie_root::SCALEMODE_NOBORDER;
+        }
+
+        log_debug("Setting scale mode");
+            root.setStageScaleMode(mode);
+    }
+
+    // Set up screenshots. 
+    if (!_screenshots.empty()) {
+        std::istringstream is(_screenshots);
+        std::string arg;
+        bool last = false;
+        ScreenShotter::FrameList v;
+
+        while (std::getline(is, arg, ',')) {
+            if (arg == "last") last = true;
+            else try {
+                const size_t frame = boost::lexical_cast<size_t>(arg);
+                v.push_back(frame);
+            }
+            catch (const boost::bad_lexical_cast&) {}
+        }
+
+        // Use default if filename is empty.
+        if (_screenshotFile.empty()) {
+            URL url(_runResources->streamProvider().baseURL());
+            std::string::size_type p = url.path().rfind('/');
+            const std::string& name = (p == std::string::npos) ? url.path() :
+                url.path().substr(p + 1);
+            _screenshotFile = "screenshot-" + name + "-%f";
+        }
+        if (!last && v.empty()) return;
+        
+        std::auto_ptr<ScreenShotter> ss(new ScreenShotter(_screenshotFile, _screenshotQuality));
+        if (last) ss->lastFrame();
+        ss->setFrames(v);
+        _gui->setScreenShotter(ss);
+    }
+
     _gui->run();
 
     log_debug("Main loop ended, cleaning up");
 
     // Clean up as much as possible, so valgrind will help find actual leaks.
-    gnash::clear();
+    MovieFactory::clear();
 
-    return EXIT_SUCCESS;
 }
 
-
 void
-Player::CallbacksHandler::error(const std::string& msg)
+Player::CallbacksHandler::exit()
 {
-    _gui.error(msg);
+    _gui.quit();
 }
 
-bool
-Player::CallbacksHandler::yesNo(const std::string& query)
+boost::any
+Player::CallbacksHandler::call(const HostInterface::Message& e)
 {
-    return _gui.yesno(query);
-}
-
-std::string
-Player::CallbacksHandler::call(const std::string& event, const std::string& arg)
-{
-    StringNoCaseEqual noCaseCompare;
-	
-    if (event == "Mouse.hide") {
-        return _gui.showMouse(false) ? "true" : "false";
+    MessageHandler v(_gui);
+    try {
+        return boost::apply_visitor(v, e);
     }
-
-    if (event == "Mouse.show")
-    {
-        return _gui.showMouse(true) ? "true" : "false";
+    catch (const boost::bad_any_cast&) {
+        log_error(_("Got unexpected argument type for message %1%"), e);
+        return boost::blank();
     }
-    
-    if (event == "Stage.displayState")
-    {
-        if (arg == "fullScreen") _gui.setFullscreen();
-        else if (arg == "normal") _gui.unsetFullscreen();
-        return "";
-    }
-
-    if (event == "Stage.scaleMode" || event == "Stage.align" )
-    {
-        _gui.updateStageMatrix();
-        return "";
-    }
-
-    if (event == "Stage.showMenu")
-    {
-        if (noCaseCompare(arg, "true")) _gui.showMenu(true);
-        else if (noCaseCompare(arg, "false")) _gui.showMenu(false);
-        return "";
-    }
-
-    if (event == "System.capabilities.screenResolutionX")
-    {
-        std::ostringstream ss;
-        ss << _gui.getScreenResX();
-        return ss.str();
-    }
-
-    if (event == "System.capabilities.screenResolutionY")
-    {
-        std::ostringstream ss;
-        ss << _gui.getScreenResY();
-        return ss.str();
-    }
-
-    if (event == "System.capabilities.pixelAspectRatio")
-    {
-        std::ostringstream ss;
-        // Whether the pp actively limits the precision or simply
-        // gets a slightly different result isn't clear.
-        ss << std::setprecision(7) << _gui.getPixelAspectRatio();
-        return ss.str();
-    }
-
-    if (event == "System.capabilities.screenDPI")
-    {
-        std::ostringstream ss;
-        ss << _gui.getScreenDPI();
-        return ss.str();
-    }
-
-    if (event == "System.capabilities.screenColor")
-    {
-        return _gui.getScreenColor();
-    }
-
-    if (event == "System.capabilities.playerType")
-    {
-        return _gui.isPlugin() ? "PlugIn" : "StandAlone";
-    }
-
-    log_error(_("Unhandled callback %s with arguments %s"), event, arg);
-    return "";
 }
 
 void
@@ -605,11 +683,13 @@ Player::CallbacksHandler::notify(const std::string& command,
 
     // it's _hostfd, but we're a static method...
     const int hostfd = _player.getHostFD();
-    if (hostfd != -1)
-    {
+    if (hostfd != -1) {
         //log_debug("user-provided host requests fd is %d", hostfd);
         std::stringstream request;
-        request << "INVOKE " << command << ":" << args << std::endl;
+	std::vector<as_value> fnargs;
+	fnargs.push_back(as_value(command));
+	fnargs.push_back(as_value(args));
+	request << ExternalInterface::makeInvoke("fsCommand", fnargs);
 
         std::string requestString = request.str();
         const char* cmd = requestString.c_str();
@@ -618,13 +698,11 @@ Player::CallbacksHandler::notify(const std::string& command,
         // NOTE: we assuming the hostfd is set in blocking mode here..
         //log_debug("Attempt to write INVOKE requests fd %d", hostfd);
         int ret = write(hostfd, cmd, len);
-        if ( ret == -1 )
-        {
+        if ( ret == -1 ) {
             log_error("Could not write to user-provided host "
                       "requests fd %d: %s", hostfd, strerror(errno));
         }
-        if ( static_cast<size_t>(ret) < len )
-        {
+        if ( static_cast<size_t>(ret) < len ) {
             log_error("Could only write %d bytes over %d required to "
                       "user-provided host requests fd %d",
                       ret, len, hostfd);
@@ -638,8 +716,7 @@ Player::CallbacksHandler::notify(const std::string& command,
 
     /// Fscommands can be ignored using an rcfile setting. As a 
     /// plugin they are always ignored.
-    if (_gui.isPlugin())
-    {
+    if (_gui.isPlugin()) {
         // We log the request to the fd above
         log_debug(_("Running as plugin: skipping internal "
                     "handling of FsCommand %s%s."));
@@ -658,23 +735,20 @@ Player::CallbacksHandler::notify(const std::string& command,
     // quit, fullscreen, showmenu, exec, allowscale, and trapallkeys.
     
     // FSCommand quit
-    if (noCaseCompare(command, "quit"))
-    {
+    if (noCaseCompare(command, "quit")) {
         _gui.quit();
         return;
     }
 
     // FSCommand fullscreen
-    if (noCaseCompare(command, "fullscreen"))
-    {
+    if (noCaseCompare(command, "fullscreen")) {
         if (noCaseCompare(args, "true")) _gui.setFullscreen();
         else if (noCaseCompare(args, "false")) _gui.unsetFullscreen();
         return;
     }
        
     // FSCommand showmenu
-    if (noCaseCompare(command, "showmenu"))
-    {
+    if (noCaseCompare(command, "showmenu")) {
         if (noCaseCompare(args, "true")) _gui.showMenu(true);
         else if (noCaseCompare(args, "false")) _gui.showMenu(false);
         return;
@@ -684,28 +758,24 @@ Player::CallbacksHandler::notify(const std::string& command,
     // Note: the pp insists that the file to execute should be in 
     // a subdirectory 'fscommand' of the 'projector' executable's
     // location. In SWF5 there were no restrictions.
-    if (noCaseCompare(command, "exec"))
-    {
+    if (noCaseCompare(command, "exec")) {
         log_unimpl(_("FsCommand exec called with argument %s"), args);
         return;
     }
 
     // FSCommand allowscale
-    if (noCaseCompare(command, "allowscale"))
-    {
+    if (noCaseCompare(command, "allowscale")) {
         //log_debug("allowscale: %s", args);
         if (noCaseCompare(args, "true")) _gui.allowScale(true);
-        else
-        {
-                if (strtol(args.c_str(), NULL, 0)) _gui.allowScale(true);
-                else _gui.allowScale(false);
+        else {
+            if (strtol(args.c_str(), NULL, 0)) _gui.allowScale(true);
+            else _gui.allowScale(false);
         }
         return;
     }
 
     // FSCommand trapallkeys
-    if (noCaseCompare(command, "trapallkeys"))
-    {
+    if (noCaseCompare(command, "trapallkeys")) {
         log_unimpl(_("FsCommand trapallkeys called with argument %s"), args);
         return;
     }
@@ -758,6 +828,10 @@ Player::getGui()
     return createAOS4Gui(_windowID, _scale, _doLoop, *_runResources);
 #endif
 
+#ifdef GUI_HAIKU
+    return createHaikuGui(_windowID, _scale, _doLoop, *_runResources);
+#endif
+
 #ifdef GUI_DUMP
     return createDumpGui(_windowID, _scale, _doLoop, *_runResources);
 #endif
@@ -767,8 +841,7 @@ Player::getGui()
 
 Player::~Player()
 {
-    if (_movieDef.get())
-    {
+    if (_movieDef.get()) {
         log_debug("~Player - _movieDef refcount: %d (1 will be dropped "
                 "now)", _movieDef->get_ref_count());
     }
