@@ -1,7 +1,7 @@
 // gtk.cpp: Gnome ToolKit graphical user interface, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,66 +22,41 @@
 #include "gnashconfig.h"
 #endif
 
-#include "log.h"
+#include "gtksup.h"
+#include "revno.h"
 
+#include <iostream>
+#include <string>
+#include <utility>
+#include <gtk/gtk.h>
+#include <gdk/gdk.h>
+#include <gdk/gdkkeysyms.h>
+#ifdef HAVE_VA_VA_H
+# include <va/va.h>
+# include "vaapi_utils.h"
+#endif
+#ifdef HAVE_VA_VA_X11_H
+# include <va/va_x11.h>
+#endif
+#ifdef HAVE_X11
+# include <X11/keysym.h>
+# include <gdk/gdkx.h>
+# include <X11/Xlib.h>
+# ifdef HAVE_XV
+#  include <X11/extensions/Xv.h>
+#  include <X11/extensions/Xvlib.h>
+# endif
+#endif
+
+#include "log.h"
 #include "gui.h"
 #include "rc.h"
-#include "gtksup.h"
 #include "sound_handler.h"
 #include "Renderer.h"
 #include "RunResources.h"
 #include "VM.h"
-#ifdef USE_LIRC
-#include "lirc.h"
-#endif
-#include "gnash.h" // Quality
-
-#include <iostream>
-
-#ifdef HAVE_VA_VA_H
-#include <va/va.h>
-#include "vaapi_utils.h"
-#endif
-#ifdef HAVE_VA_VA_X11_H
-#include <va/va_x11.h>
-#endif
-
-#ifdef HAVE_X11
-#include <X11/keysym.h>
-#include <gdk/gdkx.h>
-#include <X11/Xlib.h>
-#include <X11/extensions/Xv.h>
-#include <X11/extensions/Xvlib.h>
-#endif
-
-#include <gtk/gtk.h>
-#include <gdk/gdk.h>
-#include <gdk/gdkkeysyms.h>
-#include <string>
-
-#ifdef BUILD_CANVAS
+#include "GnashEnums.h"
 #include "gtk_canvas.h"
-#endif
-
-#ifdef HAVE_FFMPEG_AVCODEC_H
-extern "C" {
-# include "ffmpeg/avcodec.h" // Only for the version number
-}
-#endif
-
-#ifdef HAVE_LIBAVCODEC_AVCODEC_H
-extern "C" {
-# include "libavcodec/avcodec.h" // Only for the version number
-}
-#endif
-
-#ifdef HAVE_GST_GST_H
-# include "gst/gstversion.h" // Only for the version number
-#endif
-
-#ifdef GUI_HILDON
-# include <hildon/hildon.h>
-#endif
 
 #ifdef HAVE_VA_VA_H
 extern VAStatus va_getDriverName(VADisplay dpy, char **driver_name);
@@ -136,6 +111,8 @@ namespace {
                                        gpointer data);
     gboolean motionNotifyEvent(GtkWidget *widget, GdkEventMotion *event,
                                         gpointer data);
+    gboolean visibilityNotifyEvent(GtkWidget *widget, GdkEventVisibility *event,
+                                   gpointer data);
     gint popupHandler(GtkWidget *widget, GdkEvent *event);    
 
     gint popupHandlerAlt(GtkWidget *widget, GdkEvent *event);    
@@ -166,13 +143,11 @@ GtkGui::~GtkGui()
 GtkGui::GtkGui(unsigned long xid, float scale, bool loop, RunResources& r)
     :
     Gui(xid, scale, loop, r)
-#ifdef GUI_HILDON
-    ,_hildon_program(0)
-#endif
     ,_window(0)
     ,_resumeButton(0)
     ,_overlay(0)
     ,_canvas(0)
+    ,_visible(true)
     ,_popup_menu(0)
     ,_popup_menu_alt(0)
     ,_menubar(0)
@@ -184,16 +159,13 @@ GtkGui::GtkGui(unsigned long xid, float scale, bool loop, RunResources& r)
 bool
 GtkGui::init(int argc, char **argv[])
 {
+#ifdef HAVE_X11
     if (!XInitThreads()) {
         log_debug("Failed to initialize X threading support\n");
         return false;
     }
-
-    gtk_init(&argc, argv);
-
-#ifdef GUI_HILDON
-    _hildon_program = hildon_program_get_instance();
 #endif
+    gtk_init(&argc, argv);
 
     addPixmapDirectory (PKGDATADIR);
 
@@ -205,12 +177,7 @@ GtkGui::init(int argc, char **argv[])
 #endif
         log_debug (_("Created XEmbedded window"));
     } else {
-#ifdef GUI_HILDON
-        _window = hildon_window_new();
-        hildon_program_add_window(_hildon_program, HILDON_WINDOW(_window));
-#else
         _window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-#endif
         log_debug (_("Created top level window"));
     }
     
@@ -256,18 +223,11 @@ GtkGui::init(int argc, char **argv[])
         }
     }
 
-#ifdef BUILD_CANVAS
     _canvas = gnash_canvas_new();
     gnash_canvas_setup(GNASH_CANVAS(_canvas), hwaccel, renderer, argc, argv);
     // Increase reference count to prevent its destruction (which could happen
     // later if we remove it from its container).
     g_object_ref(G_OBJECT(_canvas));
-#else
-    _drawingArea = gtk_drawing_area_new();
-    // Increase reference count to prevent its destruction (which could happen
-    // later if we remove it from its container).
-    g_object_ref(G_OBJECT(_drawingArea));
-#endif
 
     _resumeButton = gtk_button_new();
     gtk_container_add(GTK_CONTAINER(_resumeButton),
@@ -288,41 +248,21 @@ GtkGui::init(int argc, char **argv[])
     gtk_widget_show(_vbox);
     gtk_container_add(GTK_CONTAINER(_window), _vbox);
 
-#if defined(USE_MENUS) && !defined(GUI_HILDON)
+#if defined(USE_MENUS) 
     if ( ! _xid ) {
         createMenuBar();
     }
 #endif
 
-#ifdef BUILD_CANVAS
     gtk_box_pack_start(GTK_BOX(_vbox), _canvas, TRUE, TRUE, 0);
-#else
-    gtk_box_pack_start(GTK_BOX(_vbox), _drawingArea, TRUE, TRUE, 0);
-#endif
 
     setupEvents();
 
     gtk_widget_realize(_window);
-#ifdef BUILD_CANVAS
     gtk_widget_show(_canvas);
-#else
-    gtk_widget_show(_drawingArea);
-#endif
     gtk_widget_show(_window);
     
-#ifdef USE_LIRC
-    lirc = new Lirc();
-    if (!lirc->init("/dev/lircd")) {
-        log_debug(_("LIRC daemon not running"));
-    }
-#endif
-
-#ifdef BUILD_CANVAS
     _renderer = gnash_canvas_get_renderer(GNASH_CANVAS(_canvas));
-#else
-    _renderer.reset(_glue->createRenderHandler());
-    if (!_renderer.get()) return false;
-#endif
     _runResources.setRenderer(_renderer);
 
     // The first time stop() was called, stopHook() might not have had a chance
@@ -400,6 +340,14 @@ GtkGui::error(const std::string& msg)
     gtk_box_pack_start(GTK_BOX(content), label, false, false, 0);
     gtk_widget_show_all(popup);
 } 
+    
+void
+GtkGui::setClipboard(const std::string& copy)
+{
+    GtkClipboard* cb = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+    gtk_clipboard_clear(cb);
+    gtk_clipboard_set_text(cb, copy.c_str(), copy.size());
+}
 
 void
 GtkGui::setFullscreen()
@@ -570,33 +518,29 @@ GtkGui::showMenu(bool show)
 }
 
 double
-GtkGui::getPixelAspectRatio()
+GtkGui::getPixelAspectRatio() const
 {
     GdkScreen* screen = gdk_screen_get_default();
 
+    const std::pair<int, int> res = screenResolution();
+
     // Screen size / number of pixels = pixel size.
     // The physical size of the screen may be reported wrongly by gdk (from X),
     // but it's the best we have. This method agrees with the pp in my case.
     double pixelAspectRatio =
-        (gdk_screen_get_height_mm(screen) / static_cast<double>(getScreenResY())) / 
-        (gdk_screen_get_width_mm(screen) / static_cast<double>(getScreenResX()));
+        (gdk_screen_get_height_mm(screen) / static_cast<double>(res.first)) / 
+        (gdk_screen_get_width_mm(screen) / static_cast<double>(res.second));
     return pixelAspectRatio;
 }
 
-int
-GtkGui::getScreenResX()
+std::pair<int, int>
+GtkGui::screenResolution() const
 {
-    return gdk_screen_width();
-}
-
-int
-GtkGui::getScreenResY()
-{
-    return gdk_screen_height(); 
+    return std::make_pair(gdk_screen_width(), gdk_screen_height());
 }
 
 double
-GtkGui::getScreenDPI()
+GtkGui::getScreenDPI() const
 {
 #if GTK_CHECK_VERSION(2,10,0)
     GdkScreen* screen = gdk_screen_get_default();
@@ -626,6 +570,7 @@ GtkGui::setupEvents()
     setupWindowEvents();
 
     gtk_widget_add_events(_canvas, GDK_EXPOSURE_MASK
+                        | GDK_VISIBILITY_NOTIFY_MASK
                         | GDK_BUTTON_PRESS_MASK
                         | GDK_BUTTON_RELEASE_MASK
                         | GDK_KEY_RELEASE_MASK
@@ -648,6 +593,8 @@ GtkGui::setupEvents()
                    G_CALLBACK(motionNotifyEvent), this);
     g_signal_connect(_canvas, "scroll_event",
                    G_CALLBACK(mouseWheelEvent), this);
+    g_signal_connect(_canvas, "visibility-notify-event",
+                   G_CALLBACK(visibilityNotifyEvent), this);
   
     g_signal_connect_after(_canvas, "realize",
                          G_CALLBACK (realizeEvent), NULL);
@@ -718,13 +665,7 @@ GtkGui::createMenuBar()
 {
     _menubar = gtk_menu_bar_new();
     gtk_widget_show(_menubar);
-#ifdef GUI_HILDON
-//     _hildon_toolbar = create_hildon_toolbar(_hildon_program);
-//     hildon_window_add_toolbar(HILDON_WINDOW(_window),
-//                               GTK_TOOLBAR(_hildon_toolbar));
-#else
     gtk_box_pack_start(GTK_BOX(_vbox), _menubar, FALSE, FALSE, 0);
-#endif
 
     createFileMenu(_menubar);
     createEditMenu(_menubar);
@@ -774,12 +715,6 @@ GtkGui::createMenu()
     gtk_container_add(GTK_CONTAINER(_popup_menu), quit);
     g_signal_connect(quit, "activate", G_CALLBACK(menuQuit), this);
 
-#ifdef GUI_HILDON
-     hildon_window_set_menu(HILDON_WINDOW(_window),
-                               GTK_MENU(_popup_menu));
-     gtk_widget_show_all(GTK_WIDGET(_popup_menu));   
-#endif
-
     return true;
 }
 
@@ -806,12 +741,6 @@ GtkGui::createMenuAlt()
     gtk_container_add(GTK_CONTAINER(_popup_menu_alt), quit);
     g_signal_connect(quit, "activate", G_CALLBACK(menuQuit), this);
 
-#ifdef GUI_HILDON
-     hildon_window_set_menu(HILDON_WINDOW(_window),
-                               GTK_MENU(_popup_menu_alt));
-     gtk_widget_show_all(GTK_WIDGET(_popup_menu_alt));   
-#endif
-
     return true;
 }
 
@@ -864,10 +793,10 @@ GtkGui::createWindow(const char *title, int width, int height,
 
 // This creates a GtkTree model for displaying movie info.
 GtkTreeModel*
-GtkGui::makeTreeModel (std::auto_ptr<InfoTree> treepointer)
+GtkGui::makeTreeModel(std::auto_ptr<movie_root::InfoTree> treepointer)
 {
 
-    InfoTree& info = *treepointer;
+    const movie_root::InfoTree& info = *treepointer;
 
     enum
     {
@@ -877,8 +806,7 @@ GtkGui::makeTreeModel (std::auto_ptr<InfoTree> treepointer)
     };
     
     GtkTreeStore *model = gtk_tree_store_new (NUM_COLUMNS,
-                         G_TYPE_STRING,
-                         G_TYPE_STRING);
+                         G_TYPE_STRING, G_TYPE_STRING);
     
     GtkTreeIter iter;
     GtkTreeIter child_iter;
@@ -888,9 +816,10 @@ GtkGui::makeTreeModel (std::auto_ptr<InfoTree> treepointer)
     int depth = 0;    
 
     assert(info.depth(info.begin()) == 0); // seems assumed in the code below
-    for (InfoTree::iterator i=info.begin(), e=info.end(); i!=e; ++i)
-    {
-        StringPair& p = *i;
+    for (movie_root::InfoTree::iterator i = info.begin(), e = info.end();
+            i != e; ++i) {
+
+        const movie_root::InfoTree::value_type& p = *i;
 
         std::ostringstream os;
         os << info.depth(i);  
@@ -904,23 +833,23 @@ GtkGui::makeTreeModel (std::auto_ptr<InfoTree> treepointer)
         }
 
         if (newdepth < depth) {
-            int gap = depth-newdepth;
+            int gap = depth - newdepth;
             depth = newdepth;
-            while(gap--)
-            {
-                gtk_tree_model_iter_parent (GTK_TREE_MODEL(model), &parent_iter, &iter);  
+            while (gap--) {
+                gtk_tree_model_iter_parent(GTK_TREE_MODEL(model),
+                        &parent_iter, &iter);  
                 iter = parent_iter;
-        }
+            }
         }
 
         //Read in data from present node
-        if (depth == 0) gtk_tree_store_append (model, &child_iter, NULL);
-        else gtk_tree_store_append (model, &child_iter, &iter);
+        if (depth == 0) gtk_tree_store_append(model, &child_iter, NULL);
+        else gtk_tree_store_append(model, &child_iter, &iter);
 
-        gtk_tree_store_set (model, &child_iter,
-                           STRING1_COLUMN, p.first.c_str(),   // "Variable"
-                           STRING2_COLUMN, p.second.c_str(),  // "Value"
-                           -1);
+        gtk_tree_store_set(model, &child_iter,
+                          STRING1_COLUMN, p.first.c_str(),   // "Variable"
+                          STRING2_COLUMN, p.second.c_str(),  // "Value"
+                          -1);
 
     }
 
@@ -956,7 +885,7 @@ GtkGui::createWindow(int width, int height)
 void
 GtkGui::beforeRendering()
 {
-    gnash_canvas_before_rendering(GNASH_CANVAS(_canvas));
+    gnash_canvas_before_rendering(GNASH_CANVAS(_canvas), getStage());
 }
 
 void
@@ -1059,9 +988,6 @@ private:
         GtkWidget *mediaDir;
         GtkWidget *saveStreamingMediaToggle;
         GtkWidget *saveLoadedMediaToggle;
-#ifdef USE_DEBUGGER
-        GtkWidget *DebuggerToggle;
-#endif
 
         PrefWidgets()
             :
@@ -1089,9 +1015,6 @@ private:
             mediaDir(0),
             saveStreamingMediaToggle(0),
             saveLoadedMediaToggle(0)
-#ifdef USE_DEBUGGER
-            ,DebuggerToggle(0)
-#endif
         {}
 
     };
@@ -1289,7 +1212,7 @@ PreferencesDialog::handlePrefs(GtkWidget* dialog, gint response, gpointer data)
         gtk_widget_destroy(dialog);
     }
 
-    if (prefs) delete prefs;
+    delete prefs;
 }
 
 void
@@ -1444,18 +1367,6 @@ PreferencesDialog::addLoggingTab()
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_prefs->ASCodingErrorToggle),
                 _rcfile.showASCodingErrors());
 
-#ifdef USE_DEBUGGER
-
-    _prefs->DebuggerToggle = 
-        gtk_check_button_new_with_mnemonic(_("Enable _debugger"));
-    gtk_box_pack_start(GTK_BOX(loggingvbox), _prefs->DebuggerToggle, FALSE,
-            FALSE, 0);
-    // Align button state with _rcfile
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_prefs->DebuggerToggle),
-            _rcfile.useDebugger());
-
-#endif
-
 }
 
 void
@@ -1766,7 +1677,7 @@ GtkGui::showPropertiesDialog()
 
 #ifdef USE_SWFTREE
 
-    std::auto_ptr<InfoTree> infoptr = getMovieInfo();
+    std::auto_ptr<movie_root::InfoTree> infoptr = getMovieInfo();
 
     GtkWidget *scrollwindow1 = gtk_scrolled_window_new(0, 0);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwindow1),
@@ -1799,34 +1710,28 @@ GtkGui::showPropertiesDialog()
     
     gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW(treeview), TRUE);
 
-    gint coloffset;
-    GtkCellRenderer *renderer;
-    GtkTreeViewColumn *column;
-
     // Add columns:
     
     // 'Variable' column:
-    renderer = gtk_cell_renderer_text_new();
-    coloffset = gtk_tree_view_insert_column_with_attributes(
+    GtkCellRenderer* renderer = gtk_cell_renderer_text_new();
+    gtk_tree_view_insert_column_with_attributes(
             GTK_TREE_VIEW(treeview),
             -1, _("Variable"),
             renderer, "text",
             STRING1_COLUMN,
             NULL);
-    column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), coloffset - 1);
 
     // 'Value' column:
     // Set to be 'editable' so that the data can be selected and
     // copied; it can't actually be edited, though.
     renderer = gtk_cell_renderer_text_new ();
     g_object_set (renderer, "xalign", 0.0, "editable", TRUE, NULL);
-    coloffset = gtk_tree_view_insert_column_with_attributes(
+    gtk_tree_view_insert_column_with_attributes(
             GTK_TREE_VIEW(treeview),
             -1, _("Value"),
             renderer, "text",
             STRING2_COLUMN,
             NULL);
-    column = gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), coloffset - 1);
 
     //Add tree to scrollwindow.
     gtk_container_add(GTK_CONTAINER(scrollwindow1), treeview);
@@ -1875,12 +1780,26 @@ GtkGui::showAboutDialog()
         NULL
     };
 
-    std::string comments = _("Gnash is the GNU SWF Player based on GameSWF.");
+	const std::string license = 
+        _("This program is free software; you can redistribute it and/or modify\n"
+         "it under the terms of the GNU General Public License as published by\n"
+         "the Free Software Foundation; either version 3 of the License, or\n"
+         "(at your option) any later version.\n\n"
+         "This program is distributed in the hope that it will be useful,\n"
+         "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+         "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+         "GNU General Public License for more details.\n"
+         "You should have received a copy of the GNU General Public License\n"
+         "along with this program; if not, write to the Free Software\n"
+         "Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301\n"
+         "USA or visit http://www.gnu.org/licenses/.");
 
     media::MediaHandler* m = _runResources.mediaHandler();
 
+    std::string comments =
+        _("Gnash is the GNU SWF Player based on GameSWF.");
     comments.append(_("\nRenderer: "));
-    comments.append(RENDERER_CONFIG);
+    comments.append(_renderer->description());
     comments.append(_("\nHardware Acceleration: "));
     comments.append(HWACCEL_CONFIG);
     comments.append(_("\nGUI: "));
@@ -1888,41 +1807,36 @@ GtkGui::showAboutDialog()
     comments.append(_("\nMedia: "));
     comments.append(m ? m->description() : "no media handler");
 
-    gtk_about_dialog_set_url_hook(NULL, NULL, NULL);
-    GdkPixbuf *logo_pixbuf = createPixbuf("GnashG.png");
+    GdkPixbuf* logo_pixbuf = createPixbuf("GnashG.png");
 
-#if 1
     // gtk-2.8.20 (Debian 4.0) doesn't work fine with 
     // the gtk_show_about_dialog() call [ omits info ].
     // See bug #24426.
-
     GtkWidget* aboutWidget = gtk_about_dialog_new();
+    addGnashIcon(GTK_WINDOW(aboutWidget)); 
     GtkAboutDialog* about = GTK_ABOUT_DIALOG(aboutWidget);
 
     gtk_about_dialog_set_name (about, "Gnash");
-    gtk_about_dialog_set_version(about, VERSION);
+    std::string version = VERSION;
+    version.append("\n");
+    version.append("(");
+    version.append(BRANCH_NICK);
+    version.append("-");
+    version.append(BRANCH_REVNO);
+    version.append("-");
+    version.append(COMMIT_ID);
+    version.append(")");
+
+    gtk_about_dialog_set_version(about, version.c_str());
     gtk_about_dialog_set_copyright(about, "Copyright (C) 2005, 2006, 2007, "
-            "2008, 2009, 2010 The Free Software Foundation");
+            "2008, 2009, 2010, 2011 The Free Software Foundation");
     gtk_about_dialog_set_comments (about, comments.c_str());
     gtk_about_dialog_set_authors(about, authors);
     gtk_about_dialog_set_documenters(about, documentors);
     gtk_about_dialog_set_artists(about, artists);
     gtk_about_dialog_set_translator_credits(about, _("translator-credits"));
     gtk_about_dialog_set_logo(about, logo_pixbuf);
-    gtk_about_dialog_set_license(about, 
-        "This program is free software; you can redistribute it and/or modify\n"
-        "it under the terms of the GNU General Public License as published by\n"
-        "the Free Software Foundation; either version 3 of the License, or\n"
-        "(at your option) any later version.\n\n"
-        "This program is distributed in the hope that it will be useful,\n"
-        "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
-        "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
-        "GNU General Public License for more details.\n"
-        "You should have received a copy of the GNU General Public License\n"
-        "along with this program; if not, write to the Free Software\n"
-        "Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 "
-        "USA"
-    );
+    gtk_about_dialog_set_license(about, license.c_str());
     gtk_about_dialog_set_website(about, "http://www.gnu.org/software/gnash/");
 
     // Destroy the dialogue box when 'close' is clicked.
@@ -1931,39 +1845,7 @@ GtkGui::showAboutDialog()
 
     gtk_widget_show (aboutWidget);
 
-#else
-
-
-    gtk_show_about_dialog (
-        NULL,
-        "program-name", _("Gnash"), 
-        "version", VERSION,
-        "copyright", "Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 "
-                     "The Free Software Foundation",
-        "comments", comments.c_str(),
-        "authors", authors,
-        "documenters", documentors,
-        "artists", artists,
-        "translator-credits", _("translator-credits"),
-        "logo", logo_pixbuf,
-        "license", 
-        "This program is free software; you can redistribute it and/or modify\n"
-        "it under the terms of the GNU General Public License as published by\n"
-        "the Free Software Foundation; either version 3 of the License, or\n"
-        "(at your option) any later version.\n\n"
-        "This program is distributed in the hope that it will be useful,\n"
-        "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
-        "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
-        "GNU General Public License for more details.\n"
-        "You should have received a copy of the GNU General Public License\n"
-        "along with this program; if not, write to the Free Software\n"
-        "Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301"
-        "  USA",
-        "website", "http://www.gnu.org/software/gnash/",
-        NULL);
-#endif
-    if (logo_pixbuf)
-        gdk_pixbuf_unref(logo_pixbuf);
+    if (logo_pixbuf) gdk_pixbuf_unref(logo_pixbuf);
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -2249,7 +2131,10 @@ GtkGui::playHook()
 bool 
 GtkGui::checkX11Extension(const std::string& ext)
 {
-    
+#ifdef HAVE_X11
+#if GTK_CHECK_VERSION(2,22,0)
+	#define GDK_DISPLAY() (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()))
+#endif 
     int n = 0;
     char **extlist = XListExtensions(GDK_DISPLAY(), &n);
 
@@ -2260,6 +2145,7 @@ GtkGui::checkX11Extension(const std::string& ext)
             }
         }
     }
+#endif /* HAVE_X11 */ 
     // do not free, Xlib can depend on contents being unaltered
     return false;
 }
@@ -2407,7 +2293,7 @@ void
 addPixmapDirectory(const gchar* directory)
 {
     pixmaps_directories = 
-        g_list_prepend(pixmaps_directories, g_strdup (directory));
+        g_list_prepend(pixmaps_directories, g_strdup(directory));
 }
 
 
@@ -2420,7 +2306,7 @@ findPixmapFile(const gchar* filename)
     /* We step through each of the pixmaps directory to find it. */
     elem = pixmaps_directories;
     while (elem) {
-        gchar *pathname = g_strdup_printf ("%s%s%s", (gchar*)elem->data,
+        gchar *pathname = g_strdup_printf("%s%s%s", (gchar*)elem->data,
                 G_DIR_SEPARATOR_S, filename);
         if (g_file_test (pathname, G_FILE_TEST_EXISTS))
             return pathname;
@@ -2560,6 +2446,25 @@ motionNotifyEvent(GtkWidget *const /*widget*/, GdkEventMotion *const event,
     return true;
 }
 
+gboolean
+visibilityNotifyEvent(GtkWidget *const /*widget*/, GdkEventVisibility  *const event,
+                  const gpointer data)
+{
+    GtkGui *obj = static_cast<GtkGui *>(data);
+
+    switch (event->state) {
+        case GDK_VISIBILITY_FULLY_OBSCURED:
+            obj->setVisible(false);
+            break;
+        case GDK_VISIBILITY_PARTIAL:
+        case GDK_VISIBILITY_UNOBSCURED:
+            obj->setVisible(true);
+            break;
+    }
+        
+    return false; // propagate the event to other listeners, if any.
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////
 ///                                                                         ///
