@@ -1,7 +1,7 @@
 // GC.cpp: Garbage Collector, for Gnash
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,39 +18,38 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #include "GC.h"
-#include "utility.h" // for typeName()
-
-#ifdef GNASH_GC_DEBUG
-#include "log.h"
-#endif
 
 #include <cstdlib>
 
+#include "utility.h" // for typeName()
+#include "GnashAlgorithm.h"
+
+#ifdef GNASH_GC_DEBUG
+# include "log.h"
+#endif
+
+
 namespace gnash {
 
-GC* GC::_singleton = NULL;
-unsigned int GC::maxNewCollectablesCount = 64;
-
-GC&
-GC::init(GcRoot& root)
+GC::GC(GcRoot& root)
+    :
+    // might raise the default ...
+    _maxNewCollectablesCount(64),
+    _resListSize(0),
+    _root(root),
+    _lastResCount(0)
+#ifdef GNASH_GC_DEBUG 
+    , _collectorRuns(0)
+#endif
 {
-    assert(!_singleton);
-    _singleton = new GC(root);
-    char *gcgap = std::getenv("GNASH_GC_TRIGGER_THRESHOLD");
-    if ( gcgap )
-    {
-        unsigned int gap = strtoul(gcgap, NULL, 0);
-        _singleton->maxNewCollectablesCount = gap;
+#ifdef GNASH_GC_DEBUG 
+    log_debug(_("GC %p created"), (void*)this);
+#endif
+    char* gcgap = std::getenv("GNASH_GC_TRIGGER_THRESHOLD");
+    if (gcgap) {
+        const size_t gap = std::strtoul(gcgap, NULL, 0);
+        _maxNewCollectablesCount = gap;
     }
-    return *_singleton;
-}
-
-void
-GC::cleanup()
-{
-    assert(_singleton);
-    delete _singleton;
-    _singleton = NULL;
 }
 
 GC::~GC()
@@ -58,9 +57,8 @@ GC::~GC()
 #ifdef GNASH_GC_DEBUG 
     log_debug(_("GC deleted, deleting all managed resources - collector run %d times"), _collectorRuns);
 #endif
-
-    for (ResList::iterator i=_resList.begin(), e=_resList.end(); i!=e; ++i)
-    {
+    for (ResList::const_iterator i = _resList.begin(), e = _resList.end();
+            i != e; ++i) {
         delete *i;
     }
 }
@@ -68,17 +66,17 @@ GC::~GC()
 size_t
 GC::cleanUnreachable()
 {
-    size_t deleted = 0;
 
 #if (GNASH_GC_DEBUG > 1)
     log_debug(_("GC: sweep scan started"));
 #endif
 
-    for (ResList::iterator i=_resList.begin(), e=_resList.end(); i!=e; )
-    {
+    size_t deleted = 0;
+
+    for (ResList::iterator i = _resList.begin(), e = _resList.end(); i != e;) {
         const GcResource* res = *i;
-        if ( ! res->isReachable() )
-        {
+        if (!res->isReachable()) {
+
 #if GNASH_GC_DEBUG > 1
             log_debug(_("GC: recycling object %p (%s)"), res, typeName(*res));
 #endif
@@ -86,8 +84,7 @@ GC::cleanUnreachable()
             delete res;
             i = _resList.erase(i); // _resListSize updated at end of loop
         }
-        else
-        {
+        else {
             res->clearReachable();
             ++i;
         }
@@ -100,7 +97,6 @@ GC::cleanUnreachable()
             deleted, _resListSize);
 #endif
 
-
     return deleted;
 }
 
@@ -116,14 +112,12 @@ GC::runCycle()
 #endif
 
 #ifdef GNASH_GC_DEBUG 
-    log_debug(_("GC: collection cycle started - %d/%d new resources allocated since last run (from %d to %d)"), _resListSize-_lastResCount, maxNewCollectablesCount, _lastResCount, _resListSize);
+    log_debug(_("GC: collection cycle started - %d/%d new resources "
+            "allocated since last run (from %d to %d)"),
+            _resListSize - _lastResCount, _maxNewCollectablesCount,
+            _lastResCount, _resListSize);
 #endif // GNASH_GC_DEBUG
 
-#ifndef NDEBUG
-    boost::thread self;
-    assert(self == mainThread);
-#endif
-
     // Mark all resources as reachable
     markReachable();
 
@@ -132,18 +126,14 @@ GC::runCycle()
 
     _lastResCount = _resListSize;
 
-    //assert(_lastResCount == _resList.size()); // O(n)...
-
 }
 
 void
 GC::countCollectables(CollectablesCount& count) const
 {
-    for (ResList::const_iterator i=_resList.begin(), e=_resList.end(); i!=e; ++i)
-    {
-        const GcResource* res = *i;
-        std::string type = typeName(*res);
-        count[type]++;
+    for (ResList::const_iterator i = _resList.begin(), e = _resList.end();
+            i!=e; ++i) {
+        ++count[typeName(**i)];
     }
 }
 
