@@ -1,7 +1,7 @@
 // GnashSleep.h -- How do you sleep?
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 #if defined(_WIN32) || defined(WIN32)
 # include <windows.h>
 #else
-# include <unistd.h>
+# include <time.h>
 #endif
 
 namespace gnash {
@@ -37,7 +37,9 @@ inline void gnashSleep(size_t useconds)
 #if defined(_WIN32) || defined(WIN32)
     Sleep(useconds / 1000);
 #else
-    usleep(useconds);
+    const size_t m = 1000000;
+    const struct timespec t = { useconds / m, (useconds % m) * 1000 };
+    ::nanosleep(&t, 0);
 #endif
 }
 
