@@ -1,7 +1,7 @@
 // GnashSystemFDHeaders.h: Compatibility FD header for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,16 +22,40 @@
 // Include this file for file descriptor utilities:
 //
 // select()
+// ioctl() / ioctlsocket (use ioctlSocket())
 
 #ifndef GNASH_SYSTEM_FD_HEADERS
 #define GNASH_SYSTEM_FD_HEADERS
 
+#include "gnashconfig.h"
+
 #ifdef HAVE_WINSOCK2_H
 # include <winsock2.h>
+namespace {
+
+inline int ioctlSocket(int fd, int request, int* arg) {
+    unsigned long p = *arg;
+    const int ret = ::ioctlsocket(fd, request, &p);
+    *arg = p;
+    return ret;
+}
+
+}
+
 #else
+
 # include <sys/types.h>
 # include <sys/stat.h>
 # include <unistd.h>
+# include <sys/ioctl.h>
+
+namespace {
+
+inline int ioctlSocket(int fd, int request, int* arg) {
+    return ::ioctl(fd, request, arg);
+}
+
+}
 #endif
 
 #endif
