@@ -1,6 +1,6 @@
 // GnashVaapiTexture.cpp: GnashImage class used for VA/GLX rendering
 // 
-// Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+// Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,8 +24,10 @@
 
 namespace gnash {
 
-GnashVaapiTexture::GnashVaapiTexture(unsigned int width, unsigned int height, ImageType type)
-    : GnashTexture(width, height, type)
+GnashVaapiTexture::GnashVaapiTexture(unsigned int width, unsigned int height,
+        image::ImageType type)
+    :
+    GnashTexture(width, height, type)
 {
     _flags |= GNASH_TEXTURE_VAAPI;
 
