@@ -1,5 +1,5 @@
 //
-//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,19 +15,22 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+#include "RTMP.h"
+
 #include <cstdlib>
 #include <cstring>
 #include <cassert>
 #include <cstdio>
-#include <unistd.h>
-
 #include <boost/lexical_cast.hpp>
-
 // Replace!!
-#include <sys/times.h>
-#include <netinet/in.h>
+#ifndef _WIN32
+# include <sys/times.h>
+#else
+// TODO: use uptime properly on win32.
+# include <ctime>
+#endif
 
-#include "RTMP.h"
+#include "GnashSystemNetHeaders.h"
 #include "log.h"
 #include "AMF.h"
 #include "GnashAlgorithm.h"
@@ -41,9 +44,6 @@ namespace {
 
     bool sendBytesReceived(RTMP* r);
 
-    // Not sure we ever want to do this.
-    bool sendServerBW(RTMP& r);
-
     void handleMetadata(RTMP& r, const boost::uint8_t *payload,
             unsigned int len);
     void handleChangeChunkSize(RTMP& r, const RTMPPacket& packet);
@@ -212,7 +212,7 @@ RTMP::connect(const URL& url)
         try {
             port = boost::lexical_cast<boost::uint16_t>(p);
         }
-        catch (boost::bad_lexical_cast&) {}
+        catch (const boost::bad_lexical_cast&) {}
     }
 
     // Basic connection attempt.
@@ -365,7 +365,7 @@ RTMP::readSocket(boost::uint8_t* buffer, int n)
 
     const std::streamsize bytesRead = _socket.read(buffer, n);
     
-    if (_socket.bad()) {
+    if (_socket.bad() || _socket.eof() || !_socket.connected()) {
         _error = true;
         return 0;
     }
@@ -399,6 +399,24 @@ RTMP::play(const SimpleBuffer& buf, int streamID)
     sendPacket(packet);
 }
 
+/// Send the server bandwidth.
+//
+/// Why would we want to send this?
+bool
+sendServerBW(RTMP& r)
+{
+    RTMPPacket packet(4);
+  
+    packet.header.channel = CHANNEL_CONTROL1;
+    packet.header.packetType = PACKET_TYPE_SERVERBW;
+  
+    SimpleBuffer& buf = *packet.buffer;
+  
+    buf.appendNetworkLong(r.serverBandwidth());
+    return r.sendPacket(packet);
+}
+
+
 /// Fills a pre-existent RTMPPacket with information.
 //
 /// This is either read entirely from incoming data, or copied from a
@@ -569,83 +587,6 @@ RTMP::readPacketPayload(RTMPPacket& packet)
     return true;
 }
 
-bool
-RTMP::handShake()
-{
-
-    /// It is a size type, but our socket functions return int.
-    const int sigSize = 1536;
-
-    boost::uint8_t clientbuf[sigSize + 1];
-    boost::uint8_t* ourSig = clientbuf + 1;
-
-    // Not encrypted
-    clientbuf[0] = 0x03;
-    
-    // TODO: do this properly.
-    boost::uint32_t uptime = htonl(getUptime());
-    std::memcpy(ourSig, &uptime, 4);
-
-    std::fill_n(ourSig + 4, 4, 0);
-
-    // Generate 1536 random bytes.
-    std::generate(ourSig + 8, ourSig + sigSize, RandomByte());
-
-    // Send it to server.
-    if (_socket.write(clientbuf, sigSize + 1) != sigSize + 1) {
-        return false;
-    }
-
-    // Expect the same byte as we sent.
-    boost::uint8_t type;
-    if (readSocket(&type, 1) != 1) {
-        return false;
-    }
-
-    log_debug( "%s: Type Answer   : %02X", __FUNCTION__, (int)type);
-
-    if (type != clientbuf[0]) {
-        log_error( "%s: Type mismatch: client sent %d, server answered %d",
-            __FUNCTION__, clientbuf[0], type);
-    }
-    
-    boost::uint8_t serverSig[sigSize];
-
-    // Read from server.
-    if (readSocket(serverSig, sigSize) != sigSize) {
-        return false;
-    }
-
-    // decode server response
-    boost::uint32_t suptime;
-
-    memcpy(&suptime, serverSig, 4);
-    suptime = ntohl(suptime);
-
-    log_debug("Server Uptime : %d", suptime);
-    log_debug("FMS Version   : %d.%d.%d.%d",
-            +serverSig[4], +serverSig[5], +serverSig[6], +serverSig[7]);
-
-    // Send what we received from server.
-    if (_socket.write(serverSig, sigSize) != sigSize) {
-        return false;
-    }
-
-    // Expect it back again.
-    if (readSocket(serverSig, sigSize) != sigSize) {
-        return false;
-    }
-
-    const bool match = std::equal(serverSig, serverSig + arraySize(serverSig),
-                                  ourSig);
-
-    if (!match) {
-        log_error( "Signatures do not match during handshake!");
-    }
-    return true;
-}
-
-
 bool
 RTMP::sendPacket(RTMPPacket& packet)
 {
@@ -998,7 +939,7 @@ HandShaker::stage3()
    
     if (!got) return false;
     
-    assert (got == sigSize);
+    assert(got == sigSize);
 
     const boost::uint8_t* serverSig = &_recvBuf.front();
     const boost::uint8_t* ourSig = &_sendBuf.front() + 1;
@@ -1054,23 +995,6 @@ sendCtrl(RTMP& r, ControlType t, unsigned int nObject, unsigned int nTime)
 
 namespace {
 
-/// Send the server bandwidth.
-//
-/// Why would we want to send this?
-bool
-sendServerBW(RTMP& r)
-{
-    RTMPPacket packet(4);
-  
-    packet.header.channel = CHANNEL_CONTROL1;
-    packet.header.packetType = PACKET_TYPE_SERVERBW;
-  
-    SimpleBuffer& buf = *packet.buffer;
-  
-    buf.appendNetworkLong(r.serverBandwidth());
-    return r.sendPacket(packet);
-}
-
 
 bool
 sendBytesReceived(RTMP* r)
@@ -1260,8 +1184,15 @@ encodeInt32(boost::uint8_t *output, boost::uint8_t *outend, int nVal)
 boost::uint32_t
 getUptime()
 {
+#if !defined(_WIN32) && !defined(__amigaos4__)
     struct tms t;
     return times(&t) * 1000 / sysconf(_SC_CLK_TCK);
+#elif defined(__amigaos4__)
+    struct tms t;
+    return times(&t) * 1000 / 50;
+#else
+    return std::clock() * 100 / CLOCKS_PER_SEC;   
+#endif
 }
 
 } // anonymous namespace
