@@ -1,6 +1,7 @@
+// rc.h:  "Run Command" configuration file declarations, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,20 +21,22 @@
 #define GNASH_RC_H
 
 #include "dsodefs.h"
-#include "StringPredicates.h"
-
 #include <string>
 #include <vector>
-#include <iostream>
+#include <iosfwd>
 #include <sstream>
 #include <boost/cstdint.hpp>
 #include <boost/tokenizer.hpp>
 
-#ifndef _WIN32
+#include "StringPredicates.h"
+
+#if !defined(_WIN32) && !defined(__HAIKU__) && !defined(ANDROID)
 #include <sys/shm.h>
 #else
-typedef boost::uint32_t key_t;
+#ifdef _WIN32
+  typedef boost::uint32_t key_t;
 #endif // _WIN32
+#endif // _WIN32 and __HAIKU__
 
 namespace gnash {
   
@@ -132,9 +135,6 @@ public:
     bool useLocalDomain() const { return _localdomainOnly; }
     void useLocalDomain(bool value);
 
-    bool useXv() const { return _useXv; }
-    void useXv(bool value);
- 
     /// Whether to restrict access to the local host   
     bool useLocalHost() const { return _localhostOnly; }
 
@@ -240,6 +240,24 @@ public:
     {
         _urlOpenerFormat = value;
     }
+ 
+    // Get the name of the hardware acclerator to use for video
+    const std::string &getHWAccel() const { return _hwaccel; }
+
+    // Set the name of the hardware acclerator to use for video
+    void setHWAccel(const std::string &x) { _hwaccel = x; }
+
+    // Get the name of the renderer to draw the display
+    const std::string& getRenderer() const { return _renderer; }
+
+    // Set the name of the renderer to draw the display
+    void setRenderer(const std::string& x) { _renderer = x; }
+
+    // Get the name of the media handler to use for video/audio
+    const std::string& getMediaHandler() const { return _mediahandler; }
+
+    // Set the name of the media handler to use for video/audio
+    void setMediaHandler(const std::string& x) { _mediahandler = x; }
 
     // Get the location of the sandbox for .sol files
     const std::string &getSOLSafeDir() const { return _solsandbox; }
@@ -319,7 +337,7 @@ public:
         _certdir = value;
     }
 
-    void ignoreShowMenu(bool value) { _ignoreShowMenu= value; }
+    void ignoreShowMenu(bool value) { _ignoreShowMenu=value; }
 
     bool ignoreShowMenu() const { return _ignoreShowMenu; }
 
@@ -374,10 +392,6 @@ protected:
             std::istringstream in(value);
             if (in >> num) return true;
             
-            // If conversion fails, set value to 0 rather than leaving
-            // it as the default.
-            std::cerr << "Conversion overflow in extractNumber: " << 
-                value << std::endl;
             num = 0;
             return true;
         }
@@ -569,6 +583,19 @@ protected:
     /// default value is true
     bool _ignoreShowMenu;
 
+    /// Whether to use HW video decoding support, no value means disabled.
+    /// The only currently supported values are: none, vaapi, or xv.  omap
+    /// support is coming. 
+    std::string _hwaccel;
+
+    /// Which renderer backend to use, no value means use the default.
+    /// The currently supported values are agg, opengl, or cairo. AGG
+    /// being the default.
+    std::string _renderer;
+
+    /// Which media player backend to use, no value means use the default.
+    /// The default is set in the MediaFactory initialization table.
+    std::string _mediahandler;
 };
 
 // End of gnash namespace 
