@@ -20,37 +20,39 @@
 //
 // Much useful info at "UTF-8 and Unicode FAQ" http://www.cl.cam.ac.uk/~mgk25/unicode.html
 
-
 #include "utf8.h"
 
-// This isn't actually an invalid character; it's a valid char that
-// looks like an inverted question mark.
-#define INVALID_CHAR 0x0FFFD
+#include <limits>
+#include <boost/cstdint.hpp>
+#include <string>
+#include <vector>
+#include <cstdlib>
+
+namespace gnash {
+namespace utf8 {
+
+namespace {
+    const boost::uint32_t invalid = std::numeric_limits<boost::uint32_t>::max();
+}
 
 std::wstring
-utf8::decodeCanonicalString(const std::string& str, int version)
+decodeCanonicalString(const std::string& str, int version)
 {
     
     std::wstring wstr;
     
     std::string::const_iterator it = str.begin(), e = str.end();
     
-    if (version > 5)
-    {
-        while (boost::uint32_t code = decodeNextUnicodeCharacter(it, e))
-        {
-            if (code == utf8::invalid)
-            {
-                wstr.push_back(static_cast<wchar_t>(INVALID_CHAR));
+    if (version > 5) {
+        while (boost::uint32_t code = decodeNextUnicodeCharacter(it, e)) {
+            if (code == invalid) {
                 continue;        
             }
             wstr.push_back(static_cast<wchar_t>(code));
         }
     }
-    else
-    {
-        while (it != str.end())
-        {
+    else {
+        while (it != str.end()) {
             // This mangles UTF-8 (UCS4) strings, but is what is
             // wanted for SWF5.
             wstr.push_back(static_cast<unsigned char>(*it++));
@@ -62,7 +64,7 @@ utf8::decodeCanonicalString(const std::string& str, int version)
 }
 
 std::string
-utf8::encodeCanonicalString(const std::wstring& wstr, int version)
+encodeCanonicalString(const std::wstring& wstr, int version)
 {
 
     std::string str;
@@ -79,7 +81,7 @@ utf8::encodeCanonicalString(const std::wstring& wstr, int version)
 }
 
 std::string
-utf8::encodeLatin1Character(boost::uint32_t ucsCharacter)
+encodeLatin1Character(boost::uint32_t ucsCharacter)
 {
     std::string text;
     text.push_back(static_cast<unsigned char>(ucsCharacter));
@@ -88,10 +90,10 @@ utf8::encodeLatin1Character(boost::uint32_t ucsCharacter)
 
 
 boost::uint32_t
-utf8::decodeNextUnicodeCharacter(std::string::const_iterator& it,
-                                 const std::string::const_iterator& e)
+decodeNextUnicodeCharacter(std::string::const_iterator& it,
+                             const std::string::const_iterator& e)
 {
-    boost::uint32_t    uc;
+    boost::uint32_t uc;
 
     // Security considerations:
     //
@@ -114,7 +116,7 @@ utf8::decodeNextUnicodeCharacter(std::string::const_iterator& it,
 #define NEXT_BYTE(shift)                        \
                     \
     if (it == e || *it == 0) return 0; /* end of buffer, do not advance */    \
-    if ((*it & 0xC0) != 0x80) return utf8::invalid; /* standard check */    \
+    if ((*it & 0xC0) != 0x80) return invalid; /* standard check */    \
     /* Post-increment iterator: */        \
     uc |= (*it++ & 0x3F) << shift;
 
@@ -124,70 +126,43 @@ utf8::decodeNextUnicodeCharacter(std::string::const_iterator& it,
     if ((*it & 0x80) == 0) return static_cast<boost::uint32_t>(*it++);
 
     // Multi-byte sequences
-    if ((*it & 0xE0) == 0xC0)
-    {
+    if ((*it & 0xE0) == 0xC0) {
         // Two-byte sequence.
         FIRST_BYTE(0x1F, 6);
         NEXT_BYTE(0);
-        if (uc < 0x80) return utf8::invalid;    // overlong
+        if (uc < 0x80) return invalid;    // overlong
         return uc;
     }
-    else if ((*it & 0xF0) == 0xE0)
-    {
+    else if ((*it & 0xF0) == 0xE0) {
         // Three-byte sequence.
         FIRST_BYTE(0x0F, 12);
         NEXT_BYTE(6);
         NEXT_BYTE(0);
-        if (uc < 0x800) return utf8::invalid;    // overlong
-        if (uc >= 0x0D800 && uc <= 0x0DFFF) return utf8::invalid;    // not valid ISO 10646
-        if (uc == 0x0FFFE || uc == 0x0FFFF) return utf8::invalid;    // not valid ISO 10646
+        if (uc < 0x800) {
+            return invalid;
+        }
         return uc;
     }
-    else if ((*it & 0xF8) == 0xF0)
-    {
+    else if ((*it & 0xF8) == 0xF0) {
         // Four-byte sequence.
         FIRST_BYTE(0x07, 18);
         NEXT_BYTE(12);
         NEXT_BYTE(6);
         NEXT_BYTE(0);
-        if (uc < 0x010000) return utf8::invalid;    // overlong
+        if (uc < 0x010000) return invalid;    // overlong
         return uc;
     }
-    else if ((*it & 0xFC) == 0xF8)
-    {
-        // Five-byte sequence.
-        FIRST_BYTE(0x03, 24);
-        NEXT_BYTE(18);
-        NEXT_BYTE(12);
-        NEXT_BYTE(6);
-        NEXT_BYTE(0);
-        if (uc < 0x0200000) return utf8::invalid;    // overlong
-        return uc;
-    }
-    else if ((*it & 0xFE) == 0xFC)
-    {
-        // Six-byte sequence.
-        FIRST_BYTE(0x01, 30);
-        NEXT_BYTE(24);
-        NEXT_BYTE(18);
-        NEXT_BYTE(12);
-        NEXT_BYTE(6);
-        NEXT_BYTE(0);
-        if (uc < 0x04000000) return utf8::invalid;    // overlong
-        return uc;
-    }
-    else
-    {
+    else {
         // Invalid.
         it++;
-        return utf8::invalid;
+        return invalid;
     }
 }
 
 // TODO: buffer as std::string; index (iterator); 
 
 std::string
-utf8::encodeUnicodeCharacter(boost::uint32_t ucs_character)
+encodeUnicodeCharacter(boost::uint32_t ucs_character)
 {
 
     std::string text;
@@ -203,42 +178,20 @@ utf8::encodeUnicodeCharacter(boost::uint32_t ucs_character)
         text.push_back(0xC0 | (ucs_character >> 6));
         text.push_back(0x80 | ((ucs_character >> 0) & 0x3F));
     }
-    else if (ucs_character <= 0xFFFF)
-    {
+    else if (ucs_character <= 0xFFFF) {
         // Three bytes.
         text.push_back(0xE0 | (ucs_character >> 12));
         text.push_back(0x80 | ((ucs_character >> 6) & 0x3F));
         text.push_back(0x80 | ((ucs_character >> 0) & 0x3F));
     }
-    else if (ucs_character <= 0x1FFFFF)
-    {
+    else if (ucs_character <= 0x1FFFFF) {
         // Four bytes.
         text.push_back(0xF0 | (ucs_character >> 18));
         text.push_back(0x80 | ((ucs_character >> 12) & 0x3F));
         text.push_back(0x80 | ((ucs_character >> 6) & 0x3F));
         text.push_back(0x80 | ((ucs_character >> 0) & 0x3F));
     }
-    else if (ucs_character <= 0x3FFFFFF)
-    {
-        // Five bytes.
-        text.push_back(0xF8 | (ucs_character >> 24));
-        text.push_back(0x80 | ((ucs_character >> 18) & 0x3F));
-        text.push_back(0x80 | ((ucs_character >> 12) & 0x3F));
-        text.push_back(0x80 | ((ucs_character >> 6) & 0x3F));
-        text.push_back(0x80 | ((ucs_character >> 0) & 0x3F));
-    }
-    else if (ucs_character <= 0x7FFFFFFF)
-    {
-        // Six bytes.
-        text.push_back(0xFC | (ucs_character >> 30));
-        text.push_back(0x80 | ((ucs_character >> 24) & 0x3F));
-        text.push_back(0x80 | ((ucs_character >> 18) & 0x3F));
-        text.push_back(0x80 | ((ucs_character >> 12) & 0x3F));
-        text.push_back(0x80 | ((ucs_character >> 6) & 0x3F));
-        text.push_back(0x80 | ((ucs_character >> 0) & 0x3F));
-    }
-    else
-    {
+    else {
         // Invalid char; don't encode anything.
     }
     
@@ -252,7 +205,7 @@ utf8::encodeUnicodeCharacter(boost::uint32_t ucs_character)
 #define ENC_UTF16LE 3
 
 char*
-utf8::stripBOM(char* in, size_t& size, TextEncoding& encoding)
+stripBOM(char* in, size_t& size, TextEncoding& encoding)
 {
     encoding = encUNSPECIFIED;
     if ( size > 2 )
@@ -260,8 +213,7 @@ utf8::stripBOM(char* in, size_t& size, TextEncoding& encoding)
         // need *ptr to be unsigned or cast all 0xNN
         unsigned char* ptr = reinterpret_cast<unsigned char*>(in);
 
-        if ( *ptr == 0xFF && *(ptr+1) == 0xFE )
-        {
+        if (*ptr == 0xFF && *(ptr+1) == 0xFE) {
             // Text is UTF-16 LE
             encoding = encUTF16LE;
             in+=2;
@@ -307,7 +259,7 @@ utf8::stripBOM(char* in, size_t& size, TextEncoding& encoding)
 }
 
 const char*
-utf8::textEncodingName(TextEncoding enc)
+textEncodingName(TextEncoding enc)
 {
     switch (enc)
     {
@@ -325,6 +277,99 @@ utf8::textEncodingName(TextEncoding enc)
     }
 }
 
+EncodingGuess
+guessEncoding(const std::string &str, int &length, std::vector<int>& offsets)
+{
+    int width = 0; // The remaining width, not the total.
+    bool is_sought = true;
+
+    std::string::const_iterator it = str.begin();
+    const std::string::const_iterator e = str.end();
+
+    length = 0;
+    
+    // First, assume it's UTF8 and try to be wrong.
+    while (it != e && is_sought) {
+        ++length;
+
+        offsets.push_back(it - str.begin()); // current position
+
+        // Advances the iterator to point to the next 
+        boost::uint32_t c = utf8::decodeNextUnicodeCharacter(it, e);
+
+        if (c == utf8::invalid) {
+            is_sought = false;
+            break;
+        }
+    }
+
+    offsets.push_back(it - str.begin()); // current position
+
+    if (it == e && is_sought) {
+        // No characters left, so it's almost certainly UTF8.
+        return ENCGUESS_UNICODE;
+    }
+
+    it = str.begin();
+    int index = 0;
+    is_sought = true;
+    width = 0;
+    length = 0;
+    bool was_odd = true;
+    bool was_even = true;
+    // Now, assume it's SHIFT_JIS and try to be wrong.
+    while (it != e && is_sought) {
+        int c = static_cast<int> (*it);
+
+        if (width) {
+            --width;
+            if ((c < 0x40) || ((c < 0x9F) && was_even) ||
+                ((c > 0x9E) && was_odd) || (c == 0x7F)) {
+                is_sought = false;
+            }
+            continue;
+        }
+
+        ++length;
+        offsets.push_back(index); // [length - 1] = index;
+
+        if ((c == 0x80) || (c == 0xA0) || (c >= 0xF0)) {
+            is_sought = false;
+            break;
+        }
+
+        if (((c >= 0x81) && (c <= 0x9F)) || ((c >= 0xE0) && (c <= 0xEF))) {
+            width = 1;
+            was_odd = c & 0x01;
+            was_even = !was_odd;
+        }
+    
+        it++;
+        index++;    
+    }
+    offsets.push_back(index); // [length - 1] = index;
+    
+    if (!width && is_sought) {
+        // No width left, so it's probably SHIFT_JIS.
+        return ENCGUESS_JIS;
+    }
+
+    // It's something else.
+#ifdef ANDROID
+    length = str.size();
+#else
+    length = std::mbstowcs(NULL, str.c_str(), 0);
+#endif
+    if (length == -1)
+    {
+        length = str.length();
+    }
+    return ENCGUESS_OTHER;
+}
+
+
+} // namespace utf8
+} // namespace gnash
 
 // Local Variables:
 // mode: C++
