@@ -1,7 +1,7 @@
 // utility.h --	Various little utility functions, macros & typedefs.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -52,6 +52,10 @@
 #endif // _WIN32
 
 #ifdef __amigaos4__
+#include <stdio.h> //for FILE * in tu_file.h
+#include <fcntl.h> //for fcntl in Socket.cpp
+#include <netdb.h> //for hostent in Socket.cpp
+#include <netinet/tcp.h> //for TCP_NODELAY in Socket.cpp
 #undef UNUSED //to avoid "already defined" messages
 #define SHUT_RDWR 0
 
@@ -149,9 +153,9 @@ inline unsigned long int /* pthread_t */ get_thread_id(void)
 } // namespace gnash
 
 // Handy macro to quiet compiler warnings about unused parameters/variables.
-#define UNUSED(x) (x) = (x)
+#define UNUSED(x) static_cast<void>((x))
 
-#endif // _GNASH_UTILITY_H
+#endif 
 
 
 // Local Variables:
