@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,18 +16,18 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
+#include "ClassHierarchy.h"
+
+#include <boost/bind.hpp>
+
 #include "as_object.h"
 #include "PropFlags.h"
 #include "as_value.h"
 #include "namedStrings.h"
-#include "ClassHierarchy.h"
 #include "as_function.h"
-#include "Class.h"
 #include "Global_as.h"
 #include "extension.h"
 
-#include <boost/bind.hpp>
-
 namespace gnash {
 
 // anonymous namespace
@@ -119,8 +119,7 @@ public:
         // Successfully loaded it, now find it, set its proto, and return.
         as_value us;
         if (mTarget->get_member(_decl.uri, &us)) {
-            Global_as& gl = getGlobal(fn);
-            if (!us.to_object(gl)) {
+            if (!toObject(us, getVM(fn))) {
                 log_error("Native class %s is not an object after "
                         "initialization (%s)",
                         st.value(getName(_decl.uri)), us);
@@ -180,39 +179,6 @@ ClassHierarchy::declareAll(const NativeClasses& classes)
     std::for_each(classes.begin(), classes.end(), boost::bind(nf, this, _1));
 }
 
-void
-ClassHierarchy::markReachableResources() const
-{
-    // TODO
-}
-
-std::ostream&
-operator<<(std::ostream& os, const ClassHierarchy::NativeClass& c)
-{
-    string_table& st = VM::get().getStringTable();
-
-    os << "("
-        << " name:" << st.value(getName(c.uri))
-        << " version:" << c.version
-        << ")";
-
-    return os;
-}
-
-std::ostream&
-operator<<(std::ostream& os, const ClassHierarchy::ExtensionClass& c)
-{
-    string_table& st = VM::get().getStringTable();
-
-    os << "(file:" << c.file_name
-        << " init:" << c.init_name
-        << " name:" << st.value(getName(c.uri))
-        << " version:" << c.version
-        << ")";
-
-    return os;
-}
-
 } // end of namespace gnash
 
 // local Variables:
