@@ -1,7 +1,7 @@
 // FreetypeGlyphsProvider.cpp:  Freetype glyphs manager
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,6 +22,12 @@
 #endif
 
 #include "FreetypeGlyphsProvider.h"
+
+#include <string>
+#include <memory> // for auto_ptr
+#include <boost/cstdint.hpp>
+#include <boost/format.hpp>
+
 #include "smart_ptr.h" // for intrusive_ptr
 #include "GnashException.h"
 #include "ShapeRecord.h"
@@ -56,11 +62,6 @@
 # include <FindDirectory.h>
 #endif
 
-#include <string>
-#include <memory> // for auto_ptr
-#include <boost/cstdint.hpp>
-#include <boost/format.hpp>
-
 // Define the following to make outline decomposition verbose
 //#define DEBUG_OUTLINE_DECOMPOSITION 1
 
@@ -249,21 +250,21 @@ FreetypeGlyphsProvider::init()
 
     if (m_lib) return; 
 
-    int error = FT_Init_FreeType(&m_lib);
+    const int error = FT_Init_FreeType(&m_lib);
     if (error) {
-        std::cerr << boost::format(_("Can't init FreeType! Error "
-                    "= %d")) % error << std::endl;
-        exit(EXIT_FAILURE);
+        boost::format err = boost::format(_("Can't init FreeType! Error "
+                    "= %d")) % error;
+        throw GnashException(err.str());
     }
 }
 
 // static private
-void FreetypeGlyphsProvider::close()
+void
+FreetypeGlyphsProvider::close()
 {
-    int error = FT_Done_FreeType(m_lib);
+    const int error = FT_Done_FreeType(m_lib);
     if (error) {
-        std::cerr << boost::format(_("Can't close FreeType! Error "
-                                     "= %d")) % error << std::endl;
+        log_error(_("Can't close FreeType! Error %d"), error);
     }
 }
 
@@ -375,8 +376,9 @@ FreetypeGlyphsProvider::createFace(const std::string& name, bool bold, bool ital
     std::auto_ptr<FreetypeGlyphsProvider> ret;
 
     try { 
-        ret.reset( new FreetypeGlyphsProvider(name, bold, italic) );
-    } catch (GnashException& ge) {
+        ret.reset(new FreetypeGlyphsProvider(name, bold, italic));
+    }
+    catch (const GnashException& ge) {
         log_error(ge.what());
         assert(! ret.get());
     }
@@ -529,9 +531,12 @@ FreetypeGlyphsProvider::getGlyph(boost::uint16_t code, float& advance)
 
     FT_Outline_Decompose(outline, &walk, &walker);
 #ifdef DEBUG_OUTLINE_DECOMPOSITION 
+#if 0
+    // Don't use VM::get if this is to be re-enabled.
     SWFRect bound; sh->compute_bound(&bound, VM::get().getSWFVersion());
     log_debug("Decomposed glyph for DisplayObject '%c' has bounds %s",
             code, bound.toString());
+#endif
 #endif
 
     walker.finish();
