@@ -1,7 +1,7 @@
 // LineStyle.cpp   Line style types.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,8 +20,10 @@
 // Based on public domain work by Thatcher Ulrich <tu@tulrich.com> 2003
 // styles.cpp   -- Thatcher Ulrich <tu@tulrich.com> 2003
 
-#include "RunResources.h"
 #include "LineStyle.h"
+
+#include "TypesParser.h"
+#include "RunResources.h"
 #include "log.h"
 #include "SWFStream.h"
 #include "smart_ptr.h"
