@@ -1,7 +1,7 @@
 // MorphShape.h: the MorphShape DisplayObject implementation for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 #ifndef GNASH_MORPH_SHAPE_H
 #define GNASH_MORPH_SHAPE_H
 
-#include "smart_ptr.h" // GNASH_USE_GC
+#include "smart_ptr.h" 
 #include "DisplayObject.h"
 #include "swf/DefineMorphShapeTag.h"
 #include <boost/intrusive_ptr.hpp>
@@ -54,7 +54,7 @@ public:
     MorphShape(movie_root& mr, as_object* object,
             const SWF::DefineMorphShapeTag* def, DisplayObject* parent);
 
-	virtual void display(Renderer& renderer);
+    virtual void display(Renderer& renderer, const Transform& xform);
 
     virtual SWFRect getBounds() const;
     
