@@ -1,7 +1,7 @@
 // MovieClip.h:  Stateful live Sprite instance, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,40 +23,41 @@
 #define GNASH_MOVIECLIP_H
 
 #ifdef HAVE_CONFIG_H
-#include "gnashconfig.h" // GNASH_USE_GC, USE_SWFTREE
+#include "gnashconfig.h" 
 #endif
 
+#include <vector>
+#include <list>
+#include <map>
+#include <string>
+#include <boost/ptr_container/ptr_list.hpp>
+#include <boost/intrusive_ptr.hpp>
+
 #include "ControlTag.h"
 #include "movie_definition.h" // for inlines
 #include "DisplayList.h" // DisplayList 
 #include "DisplayObjectContainer.h"
 #include "as_environment.h" // for composition
 #include "DynamicShape.h" // for composition
-#include "Range2d.h"
+#include "snappingrange.h"
 #include "dsodefs.h" // for DSOEXPORT
 
-#include <vector>
-#include <list>
-#include <map>
-#include <string>
-
 // Forward declarations
 namespace gnash {
     class Movie;
     class swf_event;
     class drag_state;
     class LoadVariablesThread;
-    class gradient_record;
+    class GradientRecord;
     class TextField;
     class BitmapData_as;
-    class BitmapInfo;
+    class CachedBitmap;
     namespace SWF {
         class PlaceObject2Tag;
     }
 }
 
-namespace gnash
-{
+namespace gnash {
 
 /// A MovieClip is a container for DisplayObjects.
 //
@@ -83,9 +84,14 @@ namespace gnash
 /// dependent on the _swf. Exports are also sought in this Movie.
 class MovieClip : public DisplayObjectContainer 
 {
-
 public:
 
+    typedef std::vector<TextField*> TextFields;
+
+    /// A container for textfields, indexed by their variable name
+    typedef std::map<ObjectURI, TextFields, ObjectURI::LessThan>
+        TextFieldIndex;
+
     typedef std::map<std::string, std::string> MovieVariables;
 
     typedef std::list<const action_buffer*> ActionList;
@@ -200,11 +206,6 @@ public:
     // delegates to movie_root (possibly wrong)
     void set_background_color(const rgba& color);
 
-    bool has_looped() const
-    {
-        return _hasLooped;
-    }
-
     /// Return true if we have any mouse event handlers.
     //
     /// NOTE: this function currently does not consider
@@ -222,16 +223,17 @@ public:
     const DisplayObject* findDropTarget(boost::int32_t x, boost::int32_t y,
             DisplayObject* dragging) const;
 
-    void setDropTarget(const std::string& tgt)
-    {
+    void setDropTarget(const std::string& tgt) {
         _droptarget = tgt;
     }
 
-    const std::string& getDropTarget() const
-    {
+    const std::string& getDropTarget() const {
         return _droptarget;
     }
     
+    /// Advance to the next frame of the MovieClip.
+    //
+    /// Actions will be executed or pushed to the queue as necessary.
     virtual void advance();
 
     /// Set the sprite state at the specified frame number.
@@ -259,34 +261,18 @@ public:
     /// False if the frame_spec was invalid.
     bool get_frame_number(const as_value& frame_spec, size_t& frameno) const;
 
-    /// MovieClip instances need to handle cxform specially 
-    //
-    /// This is to suppor the Color asobject class
-    cxform  get_world_cxform() const;
-
-    /// Update user-defined color transform
-    //
-    /// This should only be used by the Color AS class
-    void set_user_cxform(const cxform& cx)
-    {
-        set_invalidated();
-        _userCxform = cx;
-    }
-
-    /// Return the user-defined color transform
-    //
-    /// This should only be used by the Color AS class
-    cxform get_user_cxform() const
-    {
-        return _userCxform;
-    }
-
     /// Look up the labeled frame, and jump to it.
     bool goto_labeled_frame(const std::string& label);
         
-    /// Display (render?) this Sprite/MovieClip, unless invisible
-    void display(Renderer& renderer);
+    /// Render this MovieClip.
+    virtual void display(Renderer& renderer, const Transform& xform);
     
+    /// Draw this MovieClip
+    //
+    /// This is effectively the same as display(), but uses only the passed
+    /// transform.
+    void draw(Renderer& renderer, const Transform& xform);
+
     void omit_display();
 
     /// Swap depth of the given DisplayObjects in the DisplayList
@@ -340,27 +326,6 @@ public:
     void remove_display_object(const SWF::PlaceObject2Tag* tag,
             DisplayList& dlist);
 
-    /// Proxy of DisplayList::removeDisplayObject()
-    ///
-    /// @
-    /// new DisplayObject to be used for replacing.
-    ///
-    /// @param depth
-    /// depth at which the old DisplayObject is to be replaced.
-    ///
-    /// @param use_old_cxform
-    /// if true, the cxform of the new DisplayObject will be set to the old one.
-    /// if false, the cxform of the new DisplayObject will be untouched.
-    ///
-    /// @param use_old_matrix
-    /// if true, the transformation SWFMatrix of the new DisplayObject 
-    /// will be set to the old one.
-    /// if false, the transformation SWFMatrix of the new DisplayObject will
-    /// be untouched.
-    void replace_display_object(DisplayObject* ch, int depth,
-        bool use_old_cxform, bool use_old_matrix);
-
-
     /// \brief
     /// Remove the object at the specified depth.
     //
@@ -370,25 +335,15 @@ public:
     /// (2)the id might be used for specifying a DisplayObject
     /// in the depth(think about multiple DisplayObjects within the same
     /// depth, not tested and a rare case)
-    void remove_display_object(int depth, int /* id */)
-    {
-        set_invalidated();
-        _displayList.removeDisplayObject(depth);
-    }
+    void remove_display_object(int depth, int /*id*/);
 
     void unloadMovie();
 
     /// Attach the given DisplayObject instance to current display list
     //
-    /// @param newch
-    /// The DisplayObject instance to attach.
-    ///
-    /// @param depth
-    /// The depth to assign to the instance.
-    ///
-    /// @return true on success, false on failure
-    /// FIXME: currently never returns false !
-    bool attachCharacter(DisplayObject& newch, int depth, as_object* initObject);
+    /// @param newch    The DisplayObject instance to attach.
+    /// @param depth    The depth to assign to the instance.
+    void attachCharacter(DisplayObject& newch, int depth, as_object* initObject);
 
     /// Handle placement event
     //
@@ -402,12 +357,9 @@ public:
     /// The callback will also (known to be bogus):
     //
     /// (1) Construct this instance as an ActionScript object.
-    ///     See constructAsScriptObject() method.
-    virtual void stagePlacementCallback(as_object* initObj = 0);
-
-    /// Unload all contents in the displaylist and this instance
-    /// See DisplayObject::unload for more info
-    bool unload();
+    ///     See constructAsScriptObject() method, including constructing
+    ///     registered class and adding properties.
+    virtual void construct(as_object* initObj = 0);
 
     /// Mark this sprite as destroyed
     //
@@ -429,9 +381,9 @@ public:
     /// Add the given action buffer to the list of action
     /// buffers to be processed at the end of the next
     /// frame advance.
-    void    add_action_buffer(const action_buffer* a)
+    void add_action_buffer(const action_buffer* a)
     {
-        if ( ! _callingFrameActions ) queueAction(*a);
+        if (!_callingFrameActions) queueAction(*a);
         else execute_action(*a);
     }
 
@@ -496,7 +448,7 @@ public:
     // Set TextField variables
     //
     /// TODO: this is also unlikely to be the best way to do it.
-    bool setTextFieldVariables(const ObjectURI& uri, const as_value& val);;
+    bool setTextFieldVariables(const ObjectURI& uri, const as_value& val);
 
     /// Search for a named object on the DisplayList
     //
@@ -504,13 +456,13 @@ public:
     /// MovieClip object. They take priority over DisplayObject magic
     /// properties and inherited properties, but not over own properties.
     //
-    /// @param name     The name of the object. This function handles
+    /// @param name     Object identifier. This function handles
     ///                 case-sensitivity.
     /// @return         The object if found, otherwise 0.
-    DisplayObject* getDisplayListObject(string_table::key name);
+    DisplayObject* getDisplayListObject(const ObjectURI& uri);
 
     /// Overridden to look in DisplayList for a match
-    as_object* pathElement(string_table::key key);
+    as_object* pathElement(const ObjectURI& uri);
 
     /// Execute the actions for the specified frame. 
     //
@@ -545,7 +497,7 @@ public:
     virtual void notifyEvent(const event_id& id);
 
     // inherited from DisplayObject class, see dox in DisplayObject.h
-    as_environment& get_environment() {
+    virtual as_environment& get_environment() {
         return _environment;
     }
 
@@ -554,7 +506,7 @@ public:
     //
     /// A TextField variable is a variable that acts
     /// as a setter/getter for a TextField 'text' member.
-    void set_textfield_variable(const std::string& name, TextField* ch);
+    void set_textfield_variable(const ObjectURI& name, TextField* ch);
 
     void add_invalidated_bounds(InvalidatedRanges& ranges, bool force);
     
@@ -599,83 +551,10 @@ public:
     /// testsuite/misc-ming.all/displaylist_depths_test.swf
     void removeMovieClip();
 
-    /// Render this MovieClip to a GnashImage using the passed transform
-    //
-    /// @return     The GnashImage with the MovieClip drawn onto it.
-    virtual std::auto_ptr<GnashImage> drawToBitmap(
-            const SWFMatrix& mat = SWFMatrix(), 
-            const cxform& cx = cxform(),
-            DisplayObject::BlendMode bm = DisplayObject::BLENDMODE_NORMAL,
-            const SWFRect& clipRect = SWFRect(),
-            bool smooth = false);
-
-    /// @name Drawing API
-    /// @{ 
-    
-    void lineStyle(boost::uint16_t thickness, const rgba& color,
-        bool vScale=true, bool hScale=true,
-        bool pixelHinting=false,
-        bool noClose=false,
-        CapStyle startCapStyle=CAP_ROUND,
-        CapStyle endCapStyle=CAP_ROUND,
-        JoinStyle joinStyle=JOIN_ROUND,
-        float miterLimitFactor=1.0f)
-    {
-        _drawable.lineStyle(thickness, color, vScale, hScale,
-            pixelHinting, noClose,
-            startCapStyle, endCapStyle, joinStyle,
-            miterLimitFactor);
-    }
-
-    void resetLineStyle()
-    {
-        _drawable.resetLineStyle();
-    }
-
-    void beginFill(const rgba& color)
-    {
-        _drawable.beginFill(color);
-    }
-
-    void beginLinearGradientFill(const std::vector<gradient_record>& grad,
-            const SWFMatrix& mat)
-    {
-        _drawable.beginLinearGradientFill(grad, mat);
-    }
-
-    void beginRadialGradientFill(const std::vector<gradient_record>& grad,
-            const SWFMatrix& mat)
-    {
-        _drawable.beginRadialGradientFill(grad, mat);
-    }
-
-    void endFill()
-    {
-        _drawable.endFill();
-    }
-
-    void moveTo(boost::int32_t x, boost::int32_t y)
-    {
-        _drawable.moveTo(x, y);
-    }
-
-    void lineTo(boost::int32_t x, boost::int32_t y)
-    {
+    /// Direct access to the Graphics object for drawing.
+    DynamicShape& graphics() {
         set_invalidated();
-        _drawable.lineTo(x, y, getDefinitionVersion());
-    }
-
-    void curveTo(boost::int32_t cx, boost::int32_t cy, 
-                 boost::int32_t ax, boost::int32_t ay)
-    {
-        set_invalidated();
-        _drawable.curveTo(cx, cy, ax, ay, getDefinitionVersion());
-    }
-
-    void clear()
-    {
-        set_invalidated();
-        _drawable.clear();
+        return _drawable;
     }
 
     /// Set focus to this MovieClip
@@ -690,9 +569,8 @@ public:
 
     /// Enumerate child DisplayObjects
     //
-    /// See as_object::enumerateNonProperties(as_environment&) for more info.
-    ///
-    virtual void enumerateNonProperties(as_environment&) const;
+    /// See DisplayObject::enumerateNonProperties for more info.
+    virtual void visitNonProperties(KeyVisitor& v) const;
 
     /// Delete DisplayObjects removed from the stage
     /// from the display lists
@@ -726,6 +604,11 @@ public:
 
 protected:
 
+    /// Unload all contents in the displaylist and this instance
+    //
+    /// Return true if there was an unloadHandler.
+    virtual bool unloadChildren();
+
     /// Mark sprite-specific reachable resources.
     //
     /// sprite-specific reachable resources are:
@@ -745,11 +628,6 @@ protected:
 
 private:
 
-    typedef std::vector<TextField*> TextFields;
-
-    /// A container for textfields, indexed by their variable name
-    typedef std::map<std::string, TextFields> TextFieldIndex;
-
     /// Process any completed loadVariables request
     void processCompletedLoadVariableRequests();
 
@@ -773,14 +651,6 @@ private:
 
     void stopStreamSound();
 
-    /// Register this sprite as a listener of core broadcasters
-    //
-    /// This is currently only used for key events broadcaster
-    /// and it's supposed to be called only once (or should we
-    /// also call it whenever onKey* user-defined function is defined ?
-    ///
-    void registerAsListener();
-
     /// Return value of the 'enabled' property cast to a boolean value.
     //
     /// This is true if not found (undefined to bool evaluates to false).
@@ -836,38 +706,8 @@ private:
     ///
     void restoreDisplayList(size_t targetFrame);
 
-    /// Queue actions in the action list
-    //
-    /// The list of action will be pushed on the current
-    /// global list (see movie_root).
-    ///
-    void queueActions(ActionList& action_list);
-
-    /// Execute the actions in the action list
-    //
-    /// The list of action will be consumed starting from the first
-    /// element. When the function returns the list should be empty.
-    ///
-    void execute_actions(ActionList& action_list);
-
     /// Increment _currentFrame, and take care of looping.
     void increment_frame_and_check_for_loop();
-    
-    /// \brief
-    /// Returns a vector of TextField associated with the given variable name,
-    /// or NULL if no such variable name is known.
-    //
-    /// A TextField variable is a variable that acts
-    /// as a setter/getter for a TextField 'text' member.
-    ///
-    /// Search is case-sensitive.
-    ///
-    /// @todo find out wheter we should be case sensitive or not
-    ///
-    /// @return a pointer inside a vector, will be invalidated by modifications
-    ///         of the vector (set_textfield_variable)
-    ///
-    TextFields* get_textfield_variable(const std::string& name);
 
     /// Unregister textfield variables bound to unloaded TextFields
     void cleanup_textfield_variables();
@@ -877,7 +717,7 @@ private:
     const boost::intrusive_ptr<const movie_definition> _def;
 
     /// List of loadVariables requests
-    typedef std::list<LoadVariablesThread*> LoadVariablesThreads;
+    typedef boost::ptr_list<LoadVariablesThread> LoadVariablesThreads;
     
     /// List of active loadVariable requests 
     //
@@ -894,8 +734,21 @@ private:
 
     PlayState _playState;
 
+    /// This timeline's variable scope
+    as_environment _environment;
+
+    /// We'll only allocate Textfield variables map if
+    /// we need them (ie: anyone calls set_textfield_variable)
+    ///
+    std::auto_ptr<TextFieldIndex> _text_variables;
+
+    std::string _droptarget;
+
     // 0-based index to current frame
     size_t _currentFrame;
+    
+    /// soundid for current playing stream. If no stream set to -1
+    int m_sound_stream_id;
 
     // true if this sprite reached the last frame and restarted
     bool _hasLooped;
@@ -903,21 +756,6 @@ private:
     // true is we're calling frame actions
     bool _callingFrameActions;
 
-    /// This timeline's variable scope
-    as_environment _environment;
-
-   /// We'll only allocate Textfield variables map if
-    /// we need them (ie: anyone calls set_textfield_variable)
-    ///
-    std::auto_ptr<TextFieldIndex> _text_variables;
-
-    /// soundid for current playing stream. If no stream set to -1
-    int m_sound_stream_id;
-
-    cxform _userCxform;
-
-    std::string _droptarget;
-
     bool _lockroot;
 };
 
