@@ -1,6 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,11 +20,13 @@
 #define GNASH_NATIVE_FUNCTION_H
 
 #include "as_function.h" // for inheritance
-#include "fn_call.h" // for call operator
-#include "namedStrings.h"
 
 #include <cassert>
 
+namespace gnash {
+    class fn_call;
+}
+
 namespace gnash {
 
 /// This class implements functions native to the player.
