@@ -1,7 +1,7 @@
 // PropertyList.cpp:  ActionScript property lists, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,17 +18,24 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
+#ifdef HAVE_CONFIG_H
+#include "gnashconfig.h" // GNASH_STATS_PROPERTY_LOOKUPS
+#endif
+
 #include "PropertyList.h"
+
+#include <utility> 
+#include <boost/bind.hpp> 
+#include <boost/tuple/tuple.hpp>
+
 #include "Property.h" 
 #include "as_environment.h"
 #include "log.h"
 #include "as_function.h"
-#include "as_value.h" // for enumerateValues
-#include "VM.h" // For string_table
+#include "as_value.h" 
+#include "VM.h" 
 #include "string_table.h"
-
-#include <utility> // for std::make_pair
-#include <boost/bind.hpp> 
+#include "GnashAlgorithm.h"
 
 // Define the following to enable printing address of each property added
 //#define DEBUG_PROPERTY_ALLOC
@@ -36,84 +43,64 @@
 // Define this to get verbosity of properties insertion and flags setting
 //#define GNASH_DEBUG_PROPERTY 1
 
+// Define this to get stats of property lookups 
+//#define GNASH_STATS_PROPERTY_LOOKUPS 1
+
+#ifdef GNASH_STATS_PROPERTY_LOOKUPS
+# include "Stats.h"
+# include "namedStrings.h"
+#endif
+
 namespace gnash {
 
 namespace {
 
 inline
-PropertyList::container::iterator
-iterator_find(const PropertyList::container &p, const ObjectURI& uri)
+PropertyList::const_iterator
+iterator_find(const PropertyList::container& p, const ObjectURI& uri, VM& vm)
 {
-    return p.find(uri);
+    const bool caseless = vm.getSWFVersion() < 7;
+
+    if (!caseless) {
+        return p.project<PropertyList::CreationOrder>(
+                p.get<PropertyList::Case>().find(uri));
+    }
+        
+    return p.project<PropertyList::CreationOrder>(
+            p.get<PropertyList::NoCase>().find(uri));
 }
 
 }
-
-typedef PropertyList::container::index<PropertyList::OrderTag>::type::iterator
-    order_iterator;
-
-order_iterator
-iterator_find(PropertyList::container &p, int order)
+    
+PropertyList::PropertyList(as_object& obj)
+    :
+    _props(boost::make_tuple(
+                boost::tuple<>(),
+                boost::make_tuple(
+                    KeyExtractor(),
+                    ObjectURI::LessThan()
+                ),
+                boost::make_tuple(
+                    KeyExtractor(),
+                    ObjectURI::CaseLessThan(getStringTable(obj), true)
+                )
+            )
+        ),
+    _owner(obj)
 {
-	return p.get<1>().find(order);
-}
-
-const Property*
-PropertyList::getPropertyByOrder(int order)
-{
-    order_iterator i = iterator_find(_props, order);
-	if (i == _props.get<1>().end()) return 0;
-
-	return &(*i);
-}
-
-const Property*
-PropertyList::getOrderAfter(int order)
-{
-    order_iterator i = iterator_find(_props, order);
-
-	if (i == _props.get<1>().end()) return 0;
-
-	do {
-		++i;
-		if (i == _props.get<1>().end()) return 0;
-	} while (i->getFlags().get_dont_enum());
-
-	return &(*i);
-}
-
-bool
-PropertyList::reserveSlot(const ObjectURI& uri, boost::uint16_t slotId)
-{
-    order_iterator found = iterator_find(_props, slotId + 1);
-	if (found != _props.get<1>().end()) return false;
-
-	Property a(uri, as_value());
-	a.setOrder(slotId + 1);
-	_props.insert(a);
-
-#ifdef GNASH_DEBUG_PROPERTY
-    ObjectURI::Logger l(getStringTable(_owner));
-	log_debug("Slot for AS property %s inserted with flags %s", l(uri)
-            a.getFlags());
-#endif
-
-	return true;
 }
 
 bool
 PropertyList::setValue(const ObjectURI& uri, const as_value& val,
         const PropFlags& flagsIfMissing)
 {
-	container::iterator found = iterator_find(_props, uri);
+	const_iterator found = iterator_find(_props, uri, getVM(_owner));
 	
-	if (found == _props.end())
-	{
+	if (found == _props.end()) {
 		// create a new member
 		Property a(uri, val, flagsIfMissing);
 		// Non slot properties are negative ordering in insertion order
-		a.setOrder(- ++_defaultOrder - 1);
-		_props.insert(a);
+		_props.push_back(a);
 #ifdef GNASH_DEBUG_PROPERTY
         ObjectURI::Logger l(getStringTable(_owner));
 		log_debug("Simple AS property %s inserted with flags %s",
@@ -123,54 +110,42 @@ PropertyList::setValue(const ObjectURI& uri, const as_value& val,
 	}
 
 	const Property& prop = *found;
-	if (prop.isReadOnly() && ! prop.isDestructive())
-	{
-        ObjectURI::Logger l(getStringTable(_owner));
-		log_error(_("Property %s is read-only %s, not setting it to %s"), 
-			l(uri), prop.getFlags(), val);
-		return false;
-	}
+	return prop.setValue(_owner, val);
 
-	// Property is const because the container uses its members
-	// for indexing. We don't use value (only name and namespace)
-	// so this const_cast is safe
-	const_cast<Property&>(prop).setValue(_owner, val);
-
-	return true;
 }
 
-bool
+void
 PropertyList::setFlags(const ObjectURI& uri, int setFlags, int clearFlags)
 {
-	container::iterator found = iterator_find(_props, uri);
-	if ( found == _props.end() ) return false;
+	iterator found = iterator_find(_props, uri, getVM(_owner));
+	if (found == _props.end()) return;
+    PropFlags f = found->getFlags();
+    f.set_flags(setFlags, clearFlags);
+	found->setFlags(f);
 
-	PropFlags oldFlags = found->getFlags();
-
-	PropFlags& f = const_cast<PropFlags&>(found->getFlags());
-	return f.set_flags(setFlags, clearFlags);
-
-#ifdef GNASH_DEBUG_PROPERTY
-    ObjectURI::Logger l(getStringTable(_owner));
-	log_debug("Flags of property %s changed from %s to  %s",
-		l(uri), oldFlags, found->getFlags());
-#endif
 }
 
 void
 PropertyList::setFlagsAll(int setFlags, int clearFlags)
 {
-    PropertyList::container::iterator it;
-    for (it=_props.begin(); it != _props.end(); ++it) {
-		PropFlags& f = const_cast<PropFlags&>(it->getFlags());
-		f.set_flags(setFlags, clearFlags);
+    for (const_iterator it = _props.begin(); it != _props.end(); ++it) {
+        PropFlags f = it->getFlags();
+        f.set_flags(setFlags, clearFlags);
+        it->setFlags(f);
     }
 }
 
 Property*
 PropertyList::getProperty(const ObjectURI& uri) const
 {
-	container::iterator found = iterator_find(_props, uri);
+#ifdef GNASH_STATS_PROPERTY_LOOKUPS
+    // HINT: can add a final arg to KeyLookup ctor, like NSV::PROP_ON_MOUSE_MOVE
+    //       to have *that* property lookup drive dump triggers
+    static stats::KeyLookup kcl("getProperty",
+        getStringTable(_owner), 10000000, NSV::PROP_uuPROTOuu, 10);
+    kcl.check(uri.name);
+#endif // GNASH_STATS_PROPERTY_LOOKUPS
+	iterator found = iterator_find(_props, uri, getVM(_owner));
 	if (found == _props.end()) return 0;
 	return const_cast<Property*>(&(*found));
 }
@@ -179,13 +154,13 @@ std::pair<bool,bool>
 PropertyList::delProperty(const ObjectURI& uri)
 {
 	//GNASH_REPORT_FUNCTION;
-	container::iterator found = iterator_find(_props, uri);
+	iterator found = iterator_find(_props, uri, getVM(_owner));
 	if (found == _props.end()) {
 		return std::make_pair(false, false);
 	}
 
 	// check if member is protected from deletion
-	if (found->getFlags().get_dont_delete()) {
+	if (found->getFlags().test<PropFlags::dontDelete>()) {
 		return std::make_pair(true, false);
 	}
 
@@ -193,44 +168,20 @@ PropertyList::delProperty(const ObjectURI& uri)
 	return std::make_pair(true, true);
 }
 
-
-/// This does not reflect the normal enumeration order. It is sorted
-/// lexicographically by property.
 void
-PropertyList::dump(std::map<std::string, as_value>& to) 
-{
-    ObjectURI::Logger l(getStringTable(_owner));
-
-	for (container::const_iterator i=_props.begin(), ie=_props.end();
-            i != ie; ++i)
-	{
-		to.insert(std::make_pair(l(i->uri()), i->getValue(_owner)));
-	}
-}
-
-void
-PropertyList::enumerateKeys(as_environment& env, PropertyTracker& donelist)
+PropertyList::visitKeys(KeyVisitor& visitor, PropertyTracker& donelist)
     const
 {
-	string_table& st = getStringTable(_owner);
-
     // We should enumerate in order of creation, not lexicographically.
-    typedef container::nth_index<1>::type ContainerByOrder;
+	for (const_iterator i = _props.begin(),
+            ie = _props.end(); i != ie; ++i) {
 
-	for (ContainerByOrder::const_reverse_iterator i=_props.get<1>().rbegin(),
-            ie=_props.get<1>().rend(); i != ie; ++i) {
-
-		if (i->getFlags().get_dont_enum()) continue;
+		if (i->getFlags().test<PropFlags::dontEnum>()) continue;
 
         const ObjectURI& uri = i->uri();
 
 		if (donelist.insert(uri).second) {
-
-            const std::string& qname = getNamespace(uri) ?
-                st.value(getName(uri)) + "." + st.value(getNamespace(uri)) :
-                st.value(getName(uri));
-
-			env.push(qname);
+			visitor(uri);
 		}
 	}
 }
@@ -239,7 +190,7 @@ void
 PropertyList::dump()
 {
     ObjectURI::Logger l(getStringTable(_owner));
-	for (container::const_iterator it=_props.begin(), itEnd=_props.end();
+	for (const_iterator it=_props.begin(), itEnd=_props.end();
             it != itEnd; ++it) {
 		log_debug("  %s: %s", l(it->uri()), it->getValue(_owner));
 	}
@@ -251,14 +202,11 @@ PropertyList::addGetterSetter(const ObjectURI& uri, as_function& getter,
 	const PropFlags& flagsIfMissing)
 {
 	Property a(uri, &getter, setter, flagsIfMissing);
-	a.setOrder(- ++_defaultOrder - 1);
-
-	container::iterator found = iterator_find(_props, uri);
-	if (found != _props.end())
-	{
+	iterator found = iterator_find(_props, uri, getVM(_owner));
+    
+	if (found != _props.end()) {
 		// copy flags from previous member (even if it's a normal member ?)
-		PropFlags& f = a.getFlags();
-		f = found->getFlags();
+		a.setFlags(found->getFlags());
 		a.setCache(found->getCache());
 		_props.replace(found, a);
 
@@ -269,10 +217,9 @@ PropertyList::addGetterSetter(const ObjectURI& uri, as_function& getter,
 #endif
 
 	}
-	else
-	{
+	else {
 		a.setCache(cacheVal);
-		_props.insert(a);
+		_props.push_back(a);
 #ifdef GNASH_DEBUG_PROPERTY
         ObjectURI::Logger l(getStringTable(_owner));
 		log_debug("AS GetterSetter %s inserted with flags %s", l(uri),
@@ -288,14 +235,12 @@ PropertyList::addGetterSetter(const ObjectURI& uri, as_c_function_ptr getter,
 	as_c_function_ptr setter, const PropFlags& flagsIfMissing)
 {
 	Property a(uri, getter, setter, flagsIfMissing);
-	a.setOrder(- ++_defaultOrder - 1);
 
-	container::iterator found = iterator_find(_props, uri);
+	const_iterator found = iterator_find(_props, uri, getVM(_owner));
 	if (found != _props.end())
 	{
 		// copy flags from previous member (even if it's a normal member ?)
-		PropFlags& f = a.getFlags();
-		f = found->getFlags();
+		a.setFlags(found->getFlags());
 		_props.replace(found, a);
 
 #ifdef GNASH_DEBUG_PROPERTY
@@ -307,7 +252,7 @@ PropertyList::addGetterSetter(const ObjectURI& uri, as_c_function_ptr getter,
 	}
 	else
 	{
-		_props.insert(a);
+		_props.push_back(a);
 #ifdef GNASH_DEBUG_PROPERTY
 		string_table& st = getStringTable(_owner);
 		log_debug("Native GetterSetter %s in namespace %s inserted with "
@@ -322,7 +267,7 @@ bool
 PropertyList::addDestructiveGetter(const ObjectURI& uri, as_function& getter, 
 	const PropFlags& flagsIfMissing)
 {
-	container::iterator found = iterator_find(_props, uri);
+	const_iterator found = iterator_find(_props, uri, getVM(_owner));
 	if (found != _props.end())
 	{
         ObjectURI::Logger l(getStringTable(_owner));
@@ -331,10 +276,10 @@ PropertyList::addDestructiveGetter(const ObjectURI& uri, as_function& getter,
 		return false; // Already exists.
 	}
 
-	// destructive getter don't need a setter
-	Property a(uri, &getter, (as_function*)0, flagsIfMissing, true);
-	a.setOrder(- ++_defaultOrder - 1);
-	_props.insert(a);
+	// destructive getter doesn't need a setter
+	Property a(uri, &getter, 0, flagsIfMissing, true);
+
+	_props.push_back(a);
 
 #ifdef GNASH_DEBUG_PROPERTY
     ObjectURI::Logger l(getStringTable(_owner));
@@ -349,13 +294,12 @@ bool
 PropertyList::addDestructiveGetter(const ObjectURI& uri,
 	as_c_function_ptr getter, const PropFlags& flagsIfMissing)
 {
-	container::iterator found = iterator_find(_props, uri);
+	iterator found = iterator_find(_props, uri, getVM(_owner));
 	if (found != _props.end()) return false; 
 
-	// destructive getter don't need a setter
-	Property a(uri, getter, (as_c_function_ptr)0, flagsIfMissing, true);
-	a.setOrder(- ++_defaultOrder - 1);
-	_props.insert(a);
+	// destructive getter doesn't need a setter
+	Property a(uri, getter, 0, flagsIfMissing, true);
+	_props.push_back(a);
 
 #ifdef GNASH_DEBUG_PROPERTY
     ObjectURI::Logger l(getStringTable(_owner));
@@ -371,12 +315,5 @@ PropertyList::clear()
 	_props.clear();
 }
 
-void
-PropertyList::setReachable() const
-{
-    std::for_each(_props.begin(), _props.end(),
-            boost::mem_fn(&Property::setReachable));
-}
-
 } // namespace gnash
 
