@@ -1,7 +1,7 @@
 // StaticText.h:  StaticText DisplayObject implementation for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 #ifndef GNASH_STATIC_TEXT_H
 #define GNASH_STATIC_TEXT_H
 
-#include "smart_ptr.h" // GNASH_USE_GC
+#include "smart_ptr.h" 
 #include "DisplayObject.h" // for inheritance
 #include "swf/DefineTextTag.h"
 
@@ -68,7 +68,7 @@ public:
     virtual StaticText* getStaticText(std::vector<const SWF::TextRecord*>& to,
             size_t& numChars);
 
-	virtual void display(Renderer& renderer);
+	virtual void display(Renderer& renderer, const Transform& xform);
 
     void setSelected(size_t pos, bool selected) {
         _selectedText.set(pos, selected);
