@@ -1,6 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,18 +16,19 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-
 #ifndef GNASH_VIDEO_H
 #define GNASH_VIDEO_H
 
 #include <boost/intrusive_ptr.hpp>
-#include "DisplayObject.h" // for inheritance
+#include "DisplayObject.h"
 
 // Forward declarations
 namespace gnash {
 	class NetStream_as;
-    class GnashImage;
+    class Global_as;
+    namespace image {
+        class GnashImage;
+    }
     struct ObjectURI;
     namespace SWF {
         class DefineVideoStreamTag;
@@ -47,7 +48,6 @@ namespace gnash {
 ///
 class Video : public DisplayObject
 {
-
 public:
 	
 	Video(as_object* object, const SWF::DefineVideoStreamTag* def,
@@ -66,7 +66,7 @@ public:
 	/// Register this video instance as a live DisplayObject
 	virtual void construct(as_object* init = 0);
 
-	void display(Renderer& renderer);
+	virtual void display(Renderer& renderer, const Transform& xform);
 
 	void add_invalidated_bounds(InvalidatedRanges& ranges, bool force);
 
@@ -109,7 +109,7 @@ protected:
 private:
 
 	/// Get video frame to be displayed
-	GnashImage* getVideoFrame();
+    image::GnashImage* getVideoFrame();
 
 	const boost::intrusive_ptr<const SWF::DefineVideoStreamTag> m_def;
 
@@ -123,10 +123,14 @@ private:
 	boost::int32_t _lastDecodedVideoFrameNum;
 
 	/// Last decoded frame 
-	std::auto_ptr<GnashImage> _lastDecodedVideoFrame;
+	std::auto_ptr<image::GnashImage> _lastDecodedVideoFrame;
 
-	/// The decoder used to decode the video frames
-	std::auto_ptr<media::VideoDecoder> _decoder;
+    /// The decoder used to decode the video frames for embedded streams
+    //
+    /// For dynamically loaded videos NetStream takes care of decoding
+    /// see the _ns property
+    ///
+    std::auto_ptr<media::VideoDecoder> _decoder;
 
     /// Whether to request smoothing when the video is scaled
     bool _smoothing;
