@@ -1,6 +1,6 @@
-// Accessibility_as.h:  ActionScript "Accessibility" class, for Gnash.
+// Accessibility_as.h:  ActionScript 3 "Accessibility" class, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2009, 2010, 2011 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,26 +17,27 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#ifndef GNASH_ASOBJ_ACCESSIBILITY_H
-#define GNASH_ASOBJ_ACCESSIBILITY_H
-
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
-//#include <smart_ptr.h> // for intrusive_ptr and GC stuff
+#ifndef GNASH_ASOBJ3_ACCESSIBILITY_H
+#define GNASH_ASOBJ3_ACCESSIBILITY_H
 
 namespace gnash {
 
+// Forward declarations
 class as_object;
+struct ObjectURI;
 
 /// Initialize the global Accessibility class
-void Accessibility_class_init(as_object& global);
+void accessibility_class_init(as_object& where, const ObjectURI& uri);
 
-/// Return a Accessibility instance (in case the core lib needs it)
-//boost::intrusive_ptr<as_object> init_Accessibility_instance();
+void registerAccessibilityNative(as_object& global);
 
-} // end of gnash namespace
+} // gnash namespace
 
-// GNASH_ASOBJ_ACCESSIBILITY_H
+// GNASH_ASOBJ3_ACCESSIBILITY_H
 #endif
+
+// local Variables:
+// mode: C++
+// indent-tabs-mode: t
+// End:
+
