@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,6 +24,7 @@ namespace gnash {
 	class as_value;
 	class as_object;
 	class fn_call;
+    struct ObjectURI;
 }
 
 namespace gnash {
@@ -46,9 +48,6 @@ public:
 	///
 	static void initialize(as_object& obj);
 
-	/// AsBroadcaster.initialize() AS method
-	static as_value initialize_method(const fn_call& fn);
-
 	/// Return the global AsBroadcaster
 	/// (the native one, immune to any override)
 	///
@@ -62,6 +61,5 @@ public:
 
 } // end of gnash namespace
 
-// __ASBROADCASTER_H__
 #endif
 
