@@ -1,6 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,12 +20,11 @@
 #ifndef GNASH_ASOBJ_BOOLEAN_H
 #define GNASH_ASOBJ_BOOLEAN_H
 
-#include "smart_ptr.h"
 
 namespace gnash {
 
 class as_object;
-class ObjectURI;
+struct ObjectURI;
 class Global_as;
 
 /// Initialize the global Boolean class
