@@ -1,7 +1,7 @@
 // System.cpp:  ActionScript "System" class, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,19 +18,23 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
+#include "System_as.h"
+
+#include <sstream>
+#include <string>
+#include <iomanip>
+#include <boost/tuple/tuple.hpp>
+
 #include "movie_root.h" // interface callback
 #include "log.h"
-#include "System_as.h"
 #include "fn_call.h"
 #include "smart_ptr.h" // for boost intrusive_ptr
 #include "Global_as.h"
-#include "builtin_function.h"
 #include "NativeFunction.h" 
 #include "VM.h" // for getPlayerVersion() 
 #include "GnashAlgorithm.h"
 #include "RunResources.h"
-
-#include <sstream>
+#include "HostInterface.h"
 
 namespace gnash {
 
@@ -38,12 +42,9 @@ namespace gnash {
 // Forward declarations.
 namespace {
 
-    inline std::string trueFalse(bool x) { return x ? "t" : "f"; }
+    inline char trueFalse(bool x) { return x ? 't' : 'f'; }
 
-    template<typename T> inline void convertValue(const std::string& in,
-                                                  T& val);
-
-    const std::string& systemLanguage(as_object& proto);
+    std::string systemLanguage(as_object& proto);
 
     as_value system_security_allowdomain(const fn_call& fn);
     as_value system_security_allowinsecuredomain(const fn_call& fn);
@@ -55,12 +56,6 @@ namespace {
     void attachSystemSecurityInterface(as_object& o);
     void attachSystemCapabilitiesInterface(as_object& o);
     void attachSystemInterface(as_object& proto);
-
-    // AS3 functions.
-    as_value system_gc(const fn_call& fn);
-    as_value system_pause(const fn_call& fn);
-    as_value system_resume(const fn_call& fn);
-
 }
 
 void
@@ -144,25 +139,30 @@ attachSystemCapabilitiesInterface(as_object& o)
 
     const movie_root& m = vm.getRoot();
 
-    int screenResolutionX;
-    convertValue(m.callInterface("System.capabilities.screenResolutionX"),
-            screenResolutionX);
-    int screenResolutionY;
-    convertValue(m.callInterface("System.capabilities.screenResolutionY"),
-            screenResolutionY);
-    int screenDPI;
-    convertValue(m.callInterface("System.capabilities.screenDPI"), screenDPI);
-        
-    // Documented to be a number, but is in fact a string.
-    const std::string pixelAspectRatio = 
-        m.callInterface("System.capabilities.pixelAspectRatio");
+    int resX;
+    int resY;
+
+    boost::tie(resX, resY) = m.callInterface<std::pair<int, int> >(
+            HostMessage(HostMessage::SCREEN_RESOLUTION));
+
+    const double screenDPI = m.callInterface<double>(HostMessage(
+                HostMessage::SCREEN_DPI));
+
+    const double aspectRatio = m.callInterface<double>(HostMessage(
+                HostMessage::PIXEL_ASPECT_RATIO));
+
+    // Note that the pp uses the current locale to display the
+    // ratio (for the decimal separator).
+    std::ostringstream s;
+    s << std::setprecision(7) << aspectRatio;
+    const std::string pixelAspectRatio = s.str();
 
     // "StandAlone", "External", "PlugIn", "ActiveX" (get from GUI)
-    const std::string playerType =
-        m.callInterface("System.capabilities.playerType");
+    const std::string playerType = m.callInterface<std::string>(HostMessage(
+                HostMessage::PLAYER_TYPE));
 
-    const std::string screenColor =
-        m.callInterface("System.capabilities.screenColor");
+    const std::string screenColor = m.callInterface<std::string>(HostMessage(
+                HostMessage::SCREEN_COLOR));
 
     //
     // Media
@@ -228,77 +228,66 @@ attachSystemCapabilitiesInterface(as_object& o)
                  << "&SP="	<< trueFalse(hasScreenPlayback) 
                  << "&SB="	<< trueFalse(hasScreenBroadcast) 
                  << "&DEB="	<< trueFalse(isDebugger)
-                 << "&V="       << URL::encode(version)
-                 << "&M="       << URL::encode(manufacturer)
-                 << "&R="       << screenResolutionX << "x" << screenResolutionY
+                 << "&V="   << URL::encode(version)
+                 << "&M="   << URL::encode(manufacturer)
+                 << "&R="   << resX << "x" << resY
                  << "&DP="	<< screenDPI
                  << "&COL="	<< screenColor					
-                 << "&AR="      << pixelAspectRatio
-                 << "&OS="      << URL::encode(os)
-                 << "&L="       << language			
-                 << "&PT="      << playerType
+                 << "&AR="  << pixelAspectRatio
+                 << "&OS="  << URL::encode(os)
+                 << "&L="   << language			
+                 << "&PT="  << playerType
                  << "&AVD="	<< trueFalse(avHardwareDisable) 
                  << "&LFD="	<< trueFalse(localFileReadDisable)
-                 << "&WD="      << trueFalse(windowlessDisable)
+                 << "&WD="  << trueFalse(windowlessDisable)
                  << "&TLS="	<< trueFalse(hasTLS);
     
     const int flags = PropFlags::dontDelete
         | PropFlags::dontEnum | PropFlags::readOnly;
 
+    o.init_member("hasAccessibility", hasAccessibility, flags);
+    o.init_member("pixelAspectRatio", pixelAspectRatio, flags);
+    o.init_member("screenColor", screenColor, flags);
+    o.init_member("screenDPI", screenDPI, flags);
+    o.init_member("screenResolutionY", resY, flags);
+    o.init_member("screenResolutionX", resX, flags);
+    o.init_member("hasTLS", hasTLS, flags);
+    o.init_member("hasVideoEncoder", hasVideoEncoder, flags);
+    o.init_member("hasAudioEncoder", hasAudioEncoder, flags);
+    o.init_member("hasMP3", hasMP3, flags);
+    o.init_member("hasAudio", hasAudio, flags);
+    o.init_member("serverString", serverString.str(), flags);
     o.init_member("version", version, flags);
+    o.init_member("hasStreamingAudio", hasStreamingAudio, flags);
+    o.init_member("hasStreamingVideo", hasStreamingVideo, flags);
+    o.init_member("hasEmbeddedVideo", hasEmbeddedVideo, flags);
+    o.init_member("hasPrinting", hasPrinting, flags);
+    o.init_member("hasScreenPlayback", hasScreenPlayback, flags);
+    o.init_member("hasScreenBroadcast", hasScreenBroadcast, flags);
+    o.init_member("isDebugger", isDebugger, flags);
     o.init_member("playerType", playerType, flags);
+    o.init_member("avHardwareDisable", avHardwareDisable, flags);
+    o.init_member("localFileReadDisable", localFileReadDisable, flags);
+    o.init_member("windowlessDisable", windowlessDisable, flags);
     o.init_member("os", os, flags);
     o.init_member("manufacturer", manufacturer, flags);
     o.init_member("language", language, flags);
-    o.init_member("hasAudio", hasAudio, flags);
-    o.init_member("screenResolutionX", screenResolutionX, flags);
-    o.init_member("screenResolutionY", screenResolutionY, flags);
-    o.init_member("screenColor", screenColor, flags);
-    o.init_member("screenDPI", screenDPI, flags);
-    o.init_member("pixelAspectRatio", pixelAspectRatio, flags);
-    o.init_member("serverString", serverString.str(), flags);
-    o.init_member("avHardwareDisable", avHardwareDisable, flags);
-    o.init_member("hasAudioEncoder", hasAudioEncoder, flags);
-    o.init_member("hasEmbeddedVideo", hasEmbeddedVideo, flags);
-    o.init_member("hasMP3", hasMP3, flags);
-    o.init_member("hasPrinting", hasPrinting, flags);
-    o.init_member("hasScreenBroadcast", hasScreenBroadcast, flags);
-    o.init_member("hasScreenPlayback", hasScreenPlayback, flags);
-    o.init_member("hasStreamingAudio", hasStreamingAudio, flags);
-    o.init_member("hasStreamingVideo", hasStreamingVideo, flags);
-    o.init_member("hasVideoEncoder", hasVideoEncoder, flags);
-    o.init_member("hasAccessibility", hasAccessibility, flags);
-    o.init_member("isDebugger", isDebugger, flags);
-    o.init_member("localFileReadDisable", localFileReadDisable, flags);
-    o.init_member("hasTLS", hasTLS, flags);
-    o.init_member("windowlessDisable", windowlessDisable, flags);
 }
 
-/// Convert a string to the type passed in, making sure the target variable
-/// is initialized.
-template<typename T>
-inline void
-convertValue(const std::string& in, T& val)
-{
-    std::istringstream is(in);
-    if (!(is >> val)) val = T();
-} 
-
 void
 attachSystemInterface(as_object& proto)
 {
     Global_as& gl = getGlobal(proto);
     
-    string_table& st = getStringTable(proto);
+    VM& vm = getVM(proto);
     registerBuiltinObject(proto, attachSystemSecurityInterface,
-                          st.find("security"));
+                          getURI(vm, "security"));
     registerBuiltinObject(proto, attachSystemCapabilitiesInterface,
-                          st.find("capabilities"));
+                          getURI(vm, "capabilities"));
     
     proto.init_member("setClipboard", 
                       gl.createFunction(system_setClipboard));
     
-    VM& vm = getVM(proto);
     proto.init_member("showSettings", vm.getNative(2107, 0));
     proto.init_property("useCodepage", &system_usecodepage,
                         &system_usecodepage);
@@ -320,7 +309,7 @@ as_value
 system_security_allowdomain(const fn_call& fn)
 {
     LOG_ONCE(log_unimpl("System.security.allowDomain"));
-    if (fn.nargs < 1) {
+    if (!fn.nargs) {
         return as_value(false);
     }
     return as_value(true);
@@ -330,7 +319,7 @@ system_security_allowdomain(const fn_call& fn)
 as_value
 system_security_allowinsecuredomain(const fn_call& /*fn*/)
 {
-    LOG_ONCE(log_unimpl ("System.security.allowInsecureDomain") );
+    LOG_ONCE(log_unimpl("System.security.allowInsecureDomain"));
     return as_value();
 }
 
@@ -338,46 +327,31 @@ system_security_allowinsecuredomain(const fn_call& /*fn*/)
 as_value
 system_security_loadpolicyfile(const fn_call& /*fn*/)
 {
-    LOG_ONCE(log_unimpl ("System.security.loadPolicyFile") );
+    LOG_ONCE(log_unimpl("System.security.loadPolicyFile"));
     return as_value();
 }
 
 as_value
-system_setClipboard(const fn_call& /*fn*/)
+system_setClipboard(const fn_call& fn)
 {
-    LOG_ONCE(log_unimpl ("System.setClipboard") );
+    if (!fn.nargs) {
+        return as_value();
+    }
+
+    const std::string& s = fn.arg(0).to_string();
+    movie_root& m = getRoot(fn);
+    m.callInterface(HostMessage(HostMessage::SET_CLIPBOARD, s));
+
     return as_value();
 }
 
 as_value
 system_showsettings(const fn_call& /*fn*/)
 {
-    LOG_ONCE(log_unimpl ("System.showSettings") );
+    LOG_ONCE(log_unimpl("System.showSettings"));
     return as_value();
 }
 
-as_value
-system_gc(const fn_call& /*fn*/)
-{
-    log_unimpl (__FUNCTION__);
-    return as_value();
-}
-
-as_value
-system_pause(const fn_call& /*fn*/)
-{
-    log_unimpl (__FUNCTION__);
-    return as_value();
-}
-
-as_value
-system_resume(const fn_call& /*fn*/)
-{
-    log_unimpl (__FUNCTION__);
-    return as_value();
-}
-
-
 // FIXME: should return true if shared object files
 // are stored under an exact domain name (www.gnashdev.org or
 // gnashdev.org); false if both are stored under gnashdev.org.
@@ -385,16 +359,13 @@ system_resume(const fn_call& /*fn*/)
 as_value
 system_exactsettings(const fn_call& fn)
 {
-
     // Getter
     if (fn.nargs == 0) {
         // Is always true until we implement it.
         return as_value(true);   
     }
-    
-    // Setter
     else {
-        LOG_ONCE(log_unimpl ("System.exactSettings") );
+        LOG_ONCE(log_unimpl("System.exactSettings"));
         return as_value();
     }
 }
@@ -407,23 +378,17 @@ as_value
 system_usecodepage(const fn_call& fn)
 {
     // Getter
-    if (fn.nargs == 0)
-    {
+    if (!fn.nargs) {
         // Is always false until we implement it.
         return as_value(false);   
     }
-    
-    // Setter
-    else 
-    {
+    else {
         LOG_ONCE(log_unimpl ("System.useCodepage") );
         return as_value();
     }
 }
 
-
-
-const std::string&
+std::string
 systemLanguage(as_object& proto)
 {
 	// Two-letter language code ('en', 'de') corresponding to ISO 639-1
@@ -436,7 +401,7 @@ systemLanguage(as_object& proto)
 	// some scripts rely on there being only 20 possible languages. It could
 	// be a run time option if it's important enough to care.
 
-	static std::string lang = getVM(proto).getSystemLanguage();
+	std::string lang = getVM(proto).getSystemLanguage();
 	
 	const char* languages[] = {"en", "fr", "ko", "ja", "sv",
 				"de", "es", "it", "zh", "pt",
@@ -446,32 +411,27 @@ systemLanguage(as_object& proto)
 	const size_t size = arraySize(languages);
 	
 	if (std::find(languages, languages + size, lang.substr(0, 2)) !=
-            languages + size)
-	{
-		if (lang.substr(0,2) == "zh")
-		{
+            languages + size) {
+		if (lang.substr(0, 2) == "zh") {
 			// Chinese is the only language since the pp version 7
 			// to need an additional qualifier.
 			if (lang.substr(2, 3) == "_TW") lang = "zh-TW";
 			else if (lang.substr(2, 3) == "_CN") lang = "zh-CN";
 			else lang = "xu";
 		}
-		else
-		{
+		else {
 			// All other matching cases: retain just the first
 			// two DisplayObjects.
 			lang.erase(2);
 		}
 	}
-	else
-	{
+	else {
 		// Unknown language. We also return this if
 		// getSystemLanguage() returns something unexpected. 
 		lang = "xu";
 	}
 
 	return lang;
-
 }
 
 } // anonymous namespace
