@@ -1,6 +1,6 @@
 // TextField_as.cpp:  ActionScript "TextField" class, for Gnash.
 //
-//   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
+//   Copyright (C) 2009, 2010, 2011 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,15 +17,14 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
+#include "TextField_as.h"
 
 #include "namedStrings.h"
-#include "TextField_as.h"
 #include "TextField.h"
 #include "log.h"
 #include "fn_call.h"
 #include "Global_as.h"
 #include "smart_ptr.h" // for boost intrusive_ptr
-#include "builtin_function.h" // need builtin_function
 #include "AsBroadcaster.h" // for initializing self as a broadcaster
 #include "TextFormat_as.h"
 #include "MovieClip.h"
@@ -90,7 +89,7 @@ namespace {
 as_object*
 createTextFieldObject(Global_as& gl)
 {
-    as_value tf(gl.getMember(NSV::CLASS_TEXT_FIELD));
+    as_value tf(getMember(gl, NSV::CLASS_TEXT_FIELD));
     as_function* ctor = tf.to_function();
     if (!ctor) return 0;
     fn_call::Args args;
@@ -108,7 +107,7 @@ textfield_class_init(as_object& where, const ObjectURI& uri)
 {
 
     Global_as& gl = getGlobal(where);
-    as_object* proto = gl.createObject();
+    as_object* proto = createObject(gl);
     as_object* cl = gl.createClass(&textfield_ctor, proto);
 
     attachTextFieldInterface(*proto);
@@ -147,8 +146,6 @@ attachPrototypeProperties(as_object& o)
     // SWF6 or higher
     const int swf6Flags = as_object::DefaultFlags | PropFlags::onlySWF6Up;
 
-    boost::intrusive_ptr<builtin_function> getset;
-
     // The following properties should only be attached to the prototype
     // on first textfield creation.
     o.init_property(NSV::PROP_TEXT_WIDTH,
@@ -158,7 +155,7 @@ attachPrototypeProperties(as_object& o)
 
     Global_as& gl = getGlobal(o);
 
-    getset = gl.createFunction(textfield_variable);
+    as_function* getset = gl.createFunction(textfield_variable);
     o.init_property("variable", *getset, *getset, swf6Flags);
     getset = gl.createFunction(textfield_background);
     o.init_property("background", *getset, *getset, swf6Flags);
@@ -226,11 +223,11 @@ textfield_createTextField(const fn_call& fn)
     }
 
     const std::string& name = fn.arg(0).to_string();
-    const int depth = toInt(fn.arg(1));
-    const int x = toInt(fn.arg(2));
-    const int y = toInt(fn.arg(3));
+    const int depth = toInt(fn.arg(1), getVM(fn));
+    const int x = toInt(fn.arg(2), getVM(fn));
+    const int y = toInt(fn.arg(3), getVM(fn));
     
-    int width = toInt(fn.arg(4));
+    int width = toInt(fn.arg(4), getVM(fn));
     if (width < 0) {
         IF_VERBOSE_ASCODING_ERRORS(
         log_aserror(_("createTextField: negative width (%d)"
@@ -239,7 +236,7 @@ textfield_createTextField(const fn_call& fn)
         width = -width;
     }
 
-    int height = toInt(fn.arg(5));
+    int height = toInt(fn.arg(5), getVM(fn));
     if ( height < 0 )
     {
         IF_VERBOSE_ASCODING_ERRORS(
@@ -257,12 +254,14 @@ textfield_createTextField(const fn_call& fn)
     //  2. Make that object into a TextField and put it on the display list.
     as_object* obj = createTextFieldObject(getGlobal(fn));
 
+    if (!obj) return as_value();
+
     DisplayObject* tf = new TextField(obj, ptr, bounds);
 
-    string_table& st = getStringTable(fn);
+    VM& vm = getVM(fn);
 
     // Give name and mark as dynamic
-    tf->set_name(st.find(name));
+    tf->set_name(getURI(vm, name));
     tf->setDynamic();
 
     // Set _x and _y
@@ -286,7 +285,7 @@ textfield_background(const fn_call& fn)
         return as_value(ptr->getDrawBackground());
     }
     else {
-        ptr->setDrawBackground(fn.arg(0).to_bool());
+        ptr->setDrawBackground(toBool(fn.arg(0), getVM(fn)));
     }
 
     return as_value();
@@ -301,7 +300,7 @@ textfield_border(const fn_call& fn)
         return as_value(ptr->getDrawBorder());
     }
     else {
-        ptr->setDrawBorder(fn.arg(0).to_bool());
+        ptr->setDrawBorder(toBool(fn.arg(0), getVM(fn)));
     }
 
     return as_value();
@@ -317,7 +316,7 @@ textfield_backgroundColor(const fn_call& fn)
     }
     else {
         rgba newColor;
-        newColor.parseRGB(static_cast<boost::uint32_t>(toInt(fn.arg(0))));
+        newColor.parseRGB(static_cast<boost::uint32_t>(toInt(fn.arg(0), getVM(fn))));
         ptr->setBackgroundColor(newColor);
     }
 
@@ -334,7 +333,7 @@ textfield_borderColor(const fn_call& fn)
     }
     else {
         rgba newColor;
-        newColor.parseRGB(static_cast<boost::uint32_t>(fn.arg(0).to_number()));
+        newColor.parseRGB(static_cast<boost::uint32_t>(toNumber(fn.arg(0), getVM(fn))));
         ptr->setBorderColor(newColor);
     }
 
@@ -354,7 +353,7 @@ textfield_textColor(const fn_call& fn)
 
     // Setter
     rgba newColor;
-    newColor.parseRGB(static_cast<boost::uint32_t>(fn.arg(0).to_number()));
+    newColor.parseRGB(static_cast<boost::uint32_t>(toNumber(fn.arg(0), getVM(fn))));
     ptr->setTextColor(newColor);
 
     return as_value();
@@ -371,7 +370,7 @@ textfield_embedFonts(const fn_call& fn)
     }
 
     // Setter
-    ptr->setEmbedFonts(fn.arg(0).to_bool());
+    ptr->setEmbedFonts(toBool(fn.arg(0), getVM(fn)));
     return as_value();
 }
 
@@ -384,7 +383,7 @@ textfield_wordWrap(const fn_call& fn)
         return as_value(ptr->doWordWrap());
     }
     else {
-        ptr->setWordWrap(fn.arg(0).to_bool());
+        ptr->setWordWrap(toBool(fn.arg(0), getVM(fn)));
     }
 
     return as_value();
@@ -399,7 +398,7 @@ textfield_html(const fn_call& fn)
         return as_value(ptr->doHtml());
     }
     else {
-        ptr->setHtml( fn.arg(0).to_bool() );
+        ptr->setHtml( toBool(fn.arg(0), getVM(fn)) );
     }
 
     return as_value();
@@ -416,7 +415,7 @@ textfield_selectable(const fn_call& fn)
     }
     else // setter
     {
-        ptr->setSelectable( fn.arg(0).to_bool() );
+        ptr->setSelectable( toBool(fn.arg(0), getVM(fn)) );
     }
 
     return as_value();
@@ -508,7 +507,7 @@ textfield_autoSize(const fn_call& fn)
     {
         const as_value& arg = fn.arg(0);
         if (arg.is_bool()) {
-            if (arg.to_bool()) {
+            if (toBool(arg, getVM(fn))) {
                 // True equates to left, every other bool to none.
                 ptr->setAutoSize(TextField::AUTOSIZE_LEFT);
             }
@@ -624,7 +623,7 @@ textfield_getTextFormat(const fn_call& fn)
     TextField* text = ensure<IsDisplayObject<TextField> >(fn);
 
     Global_as& gl = getGlobal(fn);
-    as_function* ctor = gl.getMember(NSV::CLASS_TEXT_FORMAT).to_function();
+    as_function* ctor = getMember(gl, NSV::CLASS_TEXT_FORMAT).to_function();
 
     if (!ctor) return as_value();
 
@@ -687,7 +686,7 @@ textfield_setTextFormat(const fn_call& fn)
     }
 
     TextFormat_as* tf;
-    if (!isNativeType(fn.arg(0).to_object(getGlobal(fn)), tf)) {
+    if (!isNativeType(toObject(fn.arg(0), getVM(fn)), tf)) {
 
         IF_VERBOSE_ASCODING_ERRORS(
             std::stringstream ss; fn.dump_args(ss);
@@ -744,7 +743,7 @@ textfield_password(const fn_call& fn)
         return as_value(text->password());
     }
     // Setter
-    text->password(fn.arg(0).to_bool());
+    text->password(toBool(fn.arg(0), getVM(fn)));
     return as_value();
 }
 
@@ -758,7 +757,7 @@ textfield_multiline(const fn_call& fn)
         return as_value(text->multiline());
     }
     // Setter
-    text->multiline(fn.arg(0).to_bool());
+    text->multiline(toBool(fn.arg(0), getVM(fn)));
     return as_value();
 }
 
@@ -797,7 +796,7 @@ textfield_bottomScroll(const fn_call& fn)
         return as_value(1 + text->getBottomScroll());
     }
     // Setter
-    //text->setBottomScroll(int(fn.arg(0).to_number())); READ-ONLY
+    //text->setBottomScroll(int(toNumber(fn.arg(0), getVM(fn)))); READ-ONLY
 
     return as_value();
 }
@@ -817,7 +816,7 @@ textfield_maxhscroll(const fn_call& fn)
         return as_value(text->getMaxHScroll());
     }
     // Setter
-    //text->setMaxHScroll(int(fn.arg(0).to_number())); READ-ONLY
+    //text->setMaxHScroll(int(toNumber(fn.arg(0), getVM(fn)))); READ-ONLY
 
     return as_value();
 }
@@ -845,7 +844,7 @@ textfield_maxChars(const fn_call& fn)
         return as_value(maxChars);
     }
     // Setter
-    text->maxChars(toInt(fn.arg(0)));
+    text->maxChars(toInt(fn.arg(0), getVM(fn)));
     return as_value();
 }
 
@@ -872,11 +871,9 @@ textfield_text(const fn_call& fn)
 as_value
 textfield_htmlText(const fn_call& fn)
 {
-    GNASH_REPORT_FUNCTION;
 
     TextField* ptr = ensure<IsDisplayObject<TextField> >(fn);
-    if (!fn.nargs)
-    {
+    if (!fn.nargs) {
         // Getter
         return as_value(ptr->get_htmltext_value());
     }
@@ -884,8 +881,8 @@ textfield_htmlText(const fn_call& fn)
     // Setter
     const int version = getSWFVersion(fn);
     
-    ptr->setHtmlTextValue(
-            utf8::decodeCanonicalString(fn.arg(0).to_string(), version));
+    ptr->setTextValue(utf8::decodeCanonicalString(fn.arg(0).to_string(),
+                version));
 
     return as_value();
 }
@@ -935,7 +932,7 @@ textfield_scroll(const fn_call& fn)
         return as_value(1 + text->getScroll());
     }
     // Setter
-    text->setScroll(int(fn.arg(0).to_number()) - 1); 
+    text->setScroll(int(toNumber(fn.arg(0), getVM(fn))) - 1); 
 
     return as_value();
 }
@@ -953,7 +950,7 @@ textfield_hscroll(const fn_call& fn)
         return as_value(text->getHScroll());
     }
     // Setter
-    text->setHScroll(int(fn.arg(0).to_number()));
+    text->setHScroll(int(toNumber(fn.arg(0), getVM(fn))));
 
     return as_value();
 }
@@ -989,7 +986,7 @@ textfield_replaceText(const fn_call& fn)
         return as_value();
     }
 
-    int userEnd = toInt(fn.arg(1));
+    int userEnd = toInt(fn.arg(1), getVM(fn));
     if ( userEnd < 0 )
     {
         IF_VERBOSE_ASCODING_ERRORS(
@@ -1000,7 +997,7 @@ textfield_replaceText(const fn_call& fn)
         return as_value();
     }
 
-    wstring::size_type start = toInt(fn.arg(0));
+    wstring::size_type start = toInt(fn.arg(0), getVM(fn));
     wstring::size_type end = userEnd;
 
     int version = getSWFVersion(fn);
