@@ -1,7 +1,7 @@
 // ColorTransform_as.cpp:  ActionScript "ColorTransform" class, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 #include "fn_call.h"
 #include "Global_as.h"
 #include "smart_ptr.h" // for boost intrusive_ptr
-#include "builtin_function.h" 
 #include "NativeFunction.h"
 #include "VM.h"
 
@@ -54,7 +53,6 @@ namespace {
 }
 
 
-
 ColorTransform_as::ColorTransform_as(double rm, double gm,
                                      double bm, double am,
                                      double ro, double go,
@@ -71,6 +69,20 @@ ColorTransform_as::ColorTransform_as(double rm, double gm,
 {
 }
 
+void
+ColorTransform_as::concat(const ColorTransform_as& other)
+{
+    _redOffset += _redMultiplier * other.getRedOffset();
+    _greenOffset += _greenMultiplier * other.getGreenOffset();
+    _blueOffset += _blueMultiplier * other.getBlueOffset();
+    _alphaOffset += _alphaMultiplier * other.getAlphaOffset();
+
+    _redMultiplier *= other.getRedMultiplier();
+    _greenMultiplier *= other.getGreenMultiplier();
+    _blueMultiplier *= other.getBlueMultiplier();
+    _alphaMultiplier *= other.getAlphaMultiplier();
+}
+
 // extern 
 void
 colortransform_class_init(as_object& where, const ObjectURI& uri)
@@ -98,17 +110,18 @@ registerColorTransformNative(as_object& global)
     vm.registerNative(colortransform_rgb, 1105, 109);
 }
 
+
 namespace {
 
 void
 attachColorTransformInterface(as_object& o)
 {
-    const int flags = 0;
     
     /// These have no flags:
+    const int flags = 0;
     VM& vm = getVM(o);
-    o.init_member("concat", vm.getNative(1105, 1), flags);
     Global_as& gl = getGlobal(o);
+    o.init_member("concat", vm.getNative(1105, 1), flags);
     o.init_member("toString", gl.createFunction(colortransform_toString),
             flags);
 
@@ -144,7 +157,7 @@ colortransform_alphaMultiplier(const fn_call& fn)
         return as_value(relay->getAlphaMultiplier());
     }
     
-    relay->setAlphaMultiplier(fn.arg(0).to_number());
+    relay->setAlphaMultiplier(toNumber(fn.arg(0), getVM(fn)));
 	return as_value();
 }
 
@@ -156,7 +169,7 @@ colortransform_alphaOffset(const fn_call& fn)
         return as_value(relay->getAlphaOffset());
     }
     
-    relay->setAlphaOffset(fn.arg(0).to_number());
+    relay->setAlphaOffset(toNumber(fn.arg(0), getVM(fn)));
 	return as_value();
 }
 
@@ -168,7 +181,7 @@ colortransform_blueMultiplier(const fn_call& fn)
         return as_value(relay->getBlueMultiplier());
     }
     
-    relay->setBlueMultiplier(fn.arg(0).to_number());
+    relay->setBlueMultiplier(toNumber(fn.arg(0), getVM(fn)));
 	return as_value();
 }
 
@@ -180,7 +193,7 @@ colortransform_blueOffset(const fn_call& fn)
         return as_value(relay->getBlueOffset());
     }
     
-    relay->setBlueOffset(fn.arg(0).to_number());
+    relay->setBlueOffset(toNumber(fn.arg(0), getVM(fn)));
 	return as_value();
 }
 
@@ -192,7 +205,7 @@ colortransform_greenMultiplier(const fn_call& fn)
         return as_value(relay->getGreenMultiplier());
     }
     
-    relay->setGreenMultiplier(fn.arg(0).to_number());
+    relay->setGreenMultiplier(toNumber(fn.arg(0), getVM(fn)));
 	return as_value();
 }
 
@@ -205,7 +218,7 @@ colortransform_greenOffset(const fn_call& fn)
         return as_value(relay->getGreenOffset());
     }
     
-    relay->setGreenOffset(fn.arg(0).to_number());
+    relay->setGreenOffset(toNumber(fn.arg(0), getVM(fn)));
 	return as_value();
 }
 
@@ -218,7 +231,7 @@ colortransform_redMultiplier(const fn_call& fn)
         return as_value(relay->getRedMultiplier());
     }
     
-    relay->setRedMultiplier(fn.arg(0).to_number());
+    relay->setRedMultiplier(toNumber(fn.arg(0), getVM(fn)));
 	return as_value();
 }
 
@@ -233,7 +246,7 @@ colortransform_redOffset(const fn_call& fn)
     }
     
     // Setter
-    relay->setRedOffset(fn.arg(0).to_number());
+    relay->setRedOffset(toNumber(fn.arg(0), getVM(fn)));
 	return as_value();
 }
 
@@ -242,8 +255,19 @@ as_value
 colortransform_concat(const fn_call& fn)
 {
 	ColorTransform_as* relay = ensure<ThisIsNative<ColorTransform_as> >(fn);
-	UNUSED(relay);
-	LOG_ONCE( log_unimpl (__FUNCTION__) );
+
+    if (!fn.nargs) {
+        // Log error
+        return as_value();
+    }
+    as_object* o = toObject(fn.arg(0), getVM(fn));
+    ColorTransform_as* tr;
+    if (!isNativeType(o, tr)) {
+        return as_value();
+    }
+    
+    relay->concat(*tr);
+
 	return as_value();
 }
 
@@ -253,31 +277,36 @@ colortransform_toString(const fn_call& fn)
 
     as_object* ptr = ensure<ValidThis>(fn);
 
-    string_table& st = getStringTable(fn);
+    VM& vm = getVM(fn);
 
-    const as_value& am = ptr->getMember(st.find("alphaMultiplier"));
-    const as_value& ao = ptr->getMember(st.find("alphaOffset"));
-    const as_value& bm = ptr->getMember(st.find("blueMultiplier"));
-    const as_value& bo = ptr->getMember(st.find("blueOffset"));
-    const as_value& gm = ptr->getMember(st.find("greenMultiplier"));
-    const as_value& go = ptr->getMember(st.find("greenOffset"));
-    const as_value& rm = ptr->getMember(st.find("redMultiplier"));
-    const as_value& ro = ptr->getMember(st.find("redOffset"));
+    const as_value& am = getMember(*ptr, getURI(vm, "alphaMultiplier"));
+    const as_value& ao = getMember(*ptr, getURI(vm, "alphaOffset"));
+    const as_value& bm = getMember(*ptr, getURI(vm, "blueMultiplier"));
+    const as_value& bo = getMember(*ptr, getURI(vm, "blueOffset"));
+    const as_value& gm = getMember(*ptr, getURI(vm, "greenMultiplier"));
+    const as_value& go = getMember(*ptr, getURI(vm, "greenOffset"));
+    const as_value& rm = getMember(*ptr, getURI(vm, "redMultiplier"));
+    const as_value& ro = getMember(*ptr, getURI(vm, "redOffset"));
    
-    std::ostringstream ss;
-    
-    const int version = getSWFVersion(fn);
+    as_value ret("(redMultiplier=");
+    newAdd(ret, rm, vm);
+    newAdd(ret, ", greenMultiplier=", vm);
+    newAdd(ret, gm, vm);
+    newAdd(ret, ", blueMultiplier=", vm);
+    newAdd(ret, bm, vm);
+    newAdd(ret, ", alphaMultiplier=", vm);
+    newAdd(ret, am, vm);
+    newAdd(ret, ", redOffset=", vm);
+    newAdd(ret, ro, vm);
+    newAdd(ret, ", greenOffset=", vm);
+    newAdd(ret, go, vm);
+    newAdd(ret, ", blueOffset=", vm);
+    newAdd(ret, bo, vm);
+    newAdd(ret, ", alphaOffset=", vm);
+    newAdd(ret, ao, vm);
+    newAdd(ret, ")", vm);
 
-    ss << "(redMultiplier=" << rm.to_string(version) << ", "
-       << "greenMultiplier=" << gm.to_string(version) << ", "
-       << "blueMultiplier=" << bm.to_string(version) << ", "
-       << "alphaMultiplier=" << am.to_string(version) << ", "
-       << "redOffset=" << ro.to_string(version) << ", "
-       << "greenOffset=" << go.to_string(version) << ", "
-       << "blueOffset=" << bo.to_string(version) << ", "
-       << "alphaOffset=" << ao.to_string(version) << ")";
-       
-    return as_value(ss.str());
+    return ret;
 
 }
 
@@ -307,7 +336,7 @@ colortransform_rgb(const fn_call& fn)
 
     // Setter
 
-    boost::uint32_t rgb = toInt(fn.arg(0));
+    boost::uint32_t rgb = toInt(fn.arg(0), getVM(fn));
     relay->setRedOffset((rgb & 0xFF0000) >> 16);
     relay->setGreenOffset((rgb & 0x00FF00) >> 8);
     relay->setBlueOffset(rgb & 0x0000FF);
@@ -354,14 +383,14 @@ colortransform_ctor(const fn_call& fn)
         );
     }
 
-	obj->setRelay(new ColorTransform_as(fn.arg(0).to_number(),
-                                        fn.arg(1).to_number(),
-                                        fn.arg(2).to_number(),
-                                        fn.arg(3).to_number(),
-                                        fn.arg(4).to_number(),
-                                        fn.arg(5).to_number(),
-                                        fn.arg(6).to_number(),
-                                        fn.arg(7).to_number()));
+	obj->setRelay(new ColorTransform_as(toNumber(fn.arg(0), getVM(fn)),
+                                        toNumber(fn.arg(1), getVM(fn)),
+                                        toNumber(fn.arg(2), getVM(fn)),
+                                        toNumber(fn.arg(3), getVM(fn)),
+                                        toNumber(fn.arg(4), getVM(fn)),
+                                        toNumber(fn.arg(5), getVM(fn)),
+                                        toNumber(fn.arg(6), getVM(fn)),
+                                        toNumber(fn.arg(7), getVM(fn))));
 
     return as_value();
 }
@@ -372,7 +401,7 @@ get_flash_geom_color_transform_constructor(const fn_call& fn)
 {
     log_debug("Loading flash.geom.ColorTransform class");
     Global_as& gl = getGlobal(fn);
-    as_object* proto = gl.createObject();
+    as_object* proto = createObject(gl);
     as_object* cl = gl.createClass(&colortransform_ctor, proto);
     attachColorTransformInterface(*proto);
     return cl;
