@@ -1,6 +1,6 @@
 // BitmapMovieDefinition.cpp:  Bitmap movie definition, for Gnash.
 // 
-//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#include "smart_ptr.h" // GNASH_USE_GC
+#include "smart_ptr.h" 
 #include "BitmapMovie.h"
 #include "BitmapMovieDefinition.h"
 #include "Geometry.h" // for class path and class edge
@@ -38,7 +38,8 @@ BitmapMovieDefinition::createMovie(Global_as& gl, DisplayObject* parent)
     return new BitmapMovie(o, this, parent);
 }
 
-BitmapMovieDefinition::BitmapMovieDefinition(std::auto_ptr<GnashImage> image,
+BitmapMovieDefinition::BitmapMovieDefinition(
+        std::auto_ptr<image::GnashImage> image,
 		Renderer* renderer, const std::string& url)
 	:
 	_version(6),
