@@ -1,6 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -27,8 +27,7 @@
 #include "gnashconfig.h" // for USE_SWFTREE
 #endif
 
-#include "smart_ptr.h" // GNASH_USE_GC
-#include "GnashImageJpeg.h"
+#include "smart_ptr.h" 
 #include "movie_definition.h" // for inheritance
 #include "DefinitionTag.h" // for boost::intrusive_ptr visibility of dtor
 #include "StringPredicates.h" 
@@ -38,10 +37,10 @@
 
 #include <boost/intrusive_ptr.hpp>
 #include <vector>
-#include <map> // for CharacterDictionary
-#include <set> // for _importSources
+#include <map>
+#include <set> 
 #include <string>
-#include <memory> // for auto_ptr
+#include <memory> 
 #include <boost/thread/thread.hpp>
 #include <boost/thread/condition.hpp>
 #include <boost/thread/barrier.hpp>
@@ -49,6 +48,10 @@
 
 // Forward declarations
 namespace gnash {
+    namespace image {
+        class JpegInput;
+    }
+    class IOChannel;
     class SWFMovieDefinition;
     class SWFStream;
     class movie_root;
@@ -249,7 +252,7 @@ public:
     bool get_labeled_frame(const std::string& label, size_t& frame_number)
         const;
 
-    void add_font(int font_id, Font* f);
+    void add_font(int font_id, boost::intrusive_ptr<Font> f);
 
     Font* get_font(int font_id) const;
 
@@ -278,7 +281,7 @@ public:
     }
 
     // See dox in movie_definition.h
-    void addControlTag(SWF::ControlTag* tag) {
+    void addControlTag(boost::intrusive_ptr<SWF::ControlTag> tag) {
         assert(tag);
         boost::mutex::scoped_lock lock(_frames_loaded_mutex);
         m_playlist[_frames_loaded].push_back(tag);
@@ -292,23 +295,10 @@ public:
 
     /// Set an input object for later loading DefineBits
     /// images (JPEG images without the table info).
-    void set_jpeg_loader(std::auto_ptr<JpegImageInput> j_in) {
-        if (m_jpeg_in.get()) {
-            /// There should be only one JPEGTABLES tag in an SWF (see: 
-            /// http://www.m2osw.com/en/swf_alexref.html#tag_jpegtables)
-            /// Discard any subsequent attempts to set the jpeg loader
-            /// to avoid crashing on very malformed SWFs. (No conclusive tests
-            /// for pp behaviour, though one version also crashes out on the
-            /// malformed SWF that triggers this assert in Gnash).
-            log_swferror(_("More than one JPEGTABLES tag found: not "
-                        "resetting JPEG loader"));
-            return;
-        }
-        m_jpeg_in = j_in;
-    }
+    void set_jpeg_loader(std::auto_ptr<image::JpegInput> j_in);
 
     // See dox in movie_definition.h
-    JpegImageInput* get_jpeg_loader() const {
+    image::JpegInput* get_jpeg_loader() const {
         return m_jpeg_in.get();
     }
 
@@ -496,7 +486,7 @@ private:
 
     boost::uint32_t m_file_length;
 
-    std::auto_ptr<JpegImageInput> m_jpeg_in;
+    std::auto_ptr<image::JpegInput> m_jpeg_in;
 
     std::string _url;
 
@@ -529,7 +519,7 @@ private:
         _bytes_loaded=bytes;
     }
 
-    /// A flag set to true when load cancelation is requested
+    /// A flag set to true when load cancellation is requested
     bool _loadingCanceled;
 
     /// Movies we import resources from
