@@ -1,6 +1,7 @@
 // action_buffer.cpp:  holds actions for later execution, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,32 +18,25 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "action_buffer.h"
-#include "log.h"
-#include "SWFStream.h"
-#include "swf.h"
-#include "ASHandlers.h"
-#include "as_environment.h"
-#include "movie_definition.h"
-
-#include <typeinfo> 
 
 #include <string>
+#include <cstring> // for memcpy
 #include <boost/static_assert.hpp>
 
-using std::string;
-using std::endl;
+#include "log.h"
+#include "SWFStream.h"
+#include "SWF.h"
+#include "ASHandlers.h"
+#include "movie_definition.h"
 
 namespace gnash {
 
 // Forward declarations
-static float convert_float_little(const void *p);
-static double convert_double_wacky(const void *p);
+namespace {
+    float convert_float_little(const void *p);
+    double convert_double_wacky(const void *p);
+}
 
 action_buffer::action_buffer(const movie_definition& md)
     :
@@ -58,10 +52,10 @@ action_buffer::read(SWFStream& in, unsigned long endPos)
     assert(endPos <= in.get_tag_end_position());
     unsigned size = endPos-startPos;
 
-    if ( ! size )
-    {
+    if (!size) {
         IF_VERBOSE_MALFORMED_SWF(
-        log_swferror(_("Empty action buffer starting at offset %lu"), startPos);
+            log_swferror(_("Empty action buffer starting at offset %lu"),
+                startPos);
         );
         return;
     }
@@ -91,36 +85,31 @@ action_buffer::read(SWFStream& in, unsigned long endPos)
     // NOTE: it is common to find such movies, swfmill is known to write
     //       DoAction w/out the terminating END tag
     //
-    if ( m_buffer.back() != SWF::ACTION_END )
-    {
+    if (m_buffer.back() != SWF::ACTION_END) {
         // Add a null terminator so read_string won't read off
         // the end of the buffer.
         m_buffer.push_back(0x00);
 
         IF_VERBOSE_MALFORMED_SWF(
-            log_swferror(_("Action buffer starting at offset %lu doesn't end with an END tag"),
-                startPos);
+            log_swferror(_("Action buffer starting at offset %lu doesn't "
+                    "end with an END tag"), startPos);
         );
     }
     
 }
 
-/*public*/
 void
 action_buffer::process_decl_dict(size_t start_pc, size_t stop_pc) const
 {
     assert(stop_pc <= m_buffer.size());
-    
-
     // Skip if we've already processed this decl_dict, but make sure
     // the size is the same.
     if (static_cast<size_t>(m_decl_dict_processed_at) == start_pc) {
         const int dictSize = read_int16(start_pc + 3);
-        if (static_cast<int>(m_dictionary.size()) != dictSize)
-        {
+        if (static_cast<int>(m_dictionary.size()) != dictSize) {
             /// TODO: is it possible to continue?
             throw ActionParserException(_("Constant pool size "
-                        "mismatch. This is probably a very malformed SWF"));
+                "mismatch. This is probably a very malformed SWF"));
         }
         return;
     }
@@ -129,8 +118,8 @@ action_buffer::process_decl_dict(size_t start_pc, size_t stop_pc) const
     
     // Actual processing.
     size_t i = start_pc;
-    boost::uint16_t length = boost::uint16_t(read_int16(i+1));
-    boost::uint16_t count = boost::uint16_t(read_int16(i+3)); 
+    const boost::uint16_t length = read_uint16(i + 1);
+    const boost::uint16_t count = read_uint16(i + 3); 
     i += 2;
     
     assert(start_pc + 3 + length == stop_pc);
@@ -140,7 +129,7 @@ action_buffer::process_decl_dict(size_t start_pc, size_t stop_pc) const
     // Index the strings.
     for (int ct = 0; ct < count; ct++) {
         // Point into the current action buffer.
-        m_dictionary[ct] = (const char*) &m_buffer[3 + i];
+        m_dictionary[ct] = reinterpret_cast<const char*>(&m_buffer[3 + i]);
 
         while (m_buffer[3 + i]) {
             // safety check.
@@ -165,7 +154,8 @@ action_buffer::process_decl_dict(size_t start_pc, size_t stop_pc) const
 // and prevents malformed instructions causing a read past the
 // end of the buffer.
 static std::string
-disasm_instruction(const unsigned char* instruction_data, size_t maxBufferLength)
+disasm_instruction(const unsigned char* instruction_data,
+        size_t maxBufferLength)
 {
 
     using namespace SWF;
@@ -174,60 +164,60 @@ disasm_instruction(const unsigned char* instruction_data, size_t maxBufferLength
 
     assert (maxBufferLength > 0);
 
-    as_arg_t fmt = ARG_HEX;
+    ArgumentType fmt = ARG_HEX;
     ActionType action_id = static_cast<ActionType>(instruction_data[0]);
 
     std::stringstream ss;
 
     // Show instruction.
     if (action_id > ash.lastType()) {
-        ss << "<unknown>[0x]" <<  action_id << endl;
+        ss << "<unknown>[0x]" <<  action_id << "\n";
     }
     else {
-        ss << ash[action_id].getName();
+        ss << ash[action_id].getType();
     }
     
     // Show instruction argument(s).
-    if (action_id & 0x80)
-    {
-        assert (maxBufferLength >= 3);
+    if (action_id & 0x80) {
+
+        assert(maxBufferLength >= 3);
         ss << " (";
         fmt = ash[action_id].getArgFormat();
-        assert(fmt != ARG_NONE);
+        
         size_t length = (instruction_data[1] | (instruction_data[2] << 8));
         
         // Assert that length without the three initial bytes
         // is always within the buffer.
-        assert (length <= maxBufferLength - 3);
+        assert(length <= maxBufferLength - 3);
+
+        switch (fmt) {
 
-        switch (fmt)
-        {
             case ARG_NONE:
                 break;
 
             case ARG_HEX:
-            {
                 ss << hexify(&instruction_data[3], length, false) << " ";
                 break;
-            }
 
             case ARG_STR:
             {
-                std::string str = hexify(&instruction_data[3], length, true);
+                const std::string str =
+                    hexify(&instruction_data[3], length, true);
                 ss << "\"" << str.c_str() << "\"";
                 break;
             }
 
             case ARG_U8:
             {
-                int val = instruction_data[3];
+                const int val = instruction_data[3];
                 ss << " " << val;
                 break;
             }
 
             case ARG_U16:
             {
-                int val = instruction_data[3] | (instruction_data[4] << 8);
+                const int val =
+                    instruction_data[3] | (instruction_data[4] << 8);
                 ss << " " << val;
                 break;
             }
@@ -243,16 +233,15 @@ disasm_instruction(const unsigned char* instruction_data, size_t maxBufferLength
             case ARG_PUSH_DATA:
             {
                 size_t i = 0;
-                while (i < length)
-                {
+                while (i < length) {
                     int type = instruction_data[3 + i];
                     
-                    // This should be safe, as the buffer is always 0-terminated.
-                    if ( i++ ) ss << ", ";
+                    // This should be safe, as the buffer is always
+                    // 0-terminated.
+                    if (i++) ss << ", ";
 
                     switch (type)
                     {
-
                         case 0:
                         {
                             // string
@@ -397,20 +386,21 @@ disasm_instruction(const unsigned char* instruction_data, size_t maxBufferLength
                        << " arg count = " << argCount
                        << " register count = " << static_cast<int>(registerCount);
                 
-                boost::uint16_t flags = (instruction_data[3 + i]) | (instruction_data[3 + i + 1] << 8);
+                const boost::uint16_t flags =
+                    (instruction_data[3 + i]) |
+                    (instruction_data[3 + i + 1] << 8);
+
                 i += 2;
                 
-                // @@ What is the difference between "super" and "_parent"?
-                
-                bool preload_global = (flags & 0x100) != 0;
-                bool preload_parent = (flags & 0x80) != 0;
-                bool preload_root   = (flags & 0x40) != 0;
-                bool suppress_super = (flags & 0x20) != 0;
-                bool preload_super  = (flags & 0x10) != 0;
-                bool suppress_args  = (flags & 0x08) != 0;
-                bool preload_args   = (flags & 0x04) != 0;
-                bool suppress_this  = (flags & 0x02) != 0;
-                bool preload_this   = (flags & 0x01) != 0;
+                const bool preload_global = (flags & 0x100);
+                const bool preload_parent = (flags & 0x80);
+                const bool preload_root   = (flags & 0x40);
+                const bool suppress_super = (flags & 0x20);
+                const bool preload_super  = (flags & 0x10);
+                const bool suppress_args  = (flags & 0x08);
+                const bool preload_args   = (flags & 0x04);
+                const bool suppress_this  = (flags & 0x02);
+                const bool preload_this   = (flags & 0x01);
                 
                 ss << " pg=" << preload_global
                 << " pp=" << preload_parent
@@ -422,8 +412,8 @@ disasm_instruction(const unsigned char* instruction_data, size_t maxBufferLength
                 << " st=" << suppress_this
                 << " pt=" << preload_this;
 
-                for (size_t argi = 0; argi < argCount; ++argi)
-                {
+                for (size_t argi = 0; argi < argCount; ++argi) {
+
                     // Make sure not to read past the end of the
                     // instruction.
                     if (i >= length) break;
@@ -433,8 +423,7 @@ disasm_instruction(const unsigned char* instruction_data, size_t maxBufferLength
 
                     std::string argName;
                     // Signature info for a function2 opcode.
-                    while (instruction_data[3 + i] && i <= length)
-                    {
+                    while (instruction_data[3 + i] && i <= length) {
                         argName.push_back(instruction_data[3 + i]);
                         i++;
                     }
@@ -472,6 +461,32 @@ action_buffer::disasm(size_t pc) const
     return disasm_instruction(&m_buffer[pc], maxBufferLength);
 }
 
+float
+action_buffer::read_float_little(size_t pc) const
+{
+    return convert_float_little(&m_buffer[pc]);
+}
+
+double
+action_buffer::read_double_wacky(size_t pc) const
+{
+    return convert_double_wacky(&m_buffer[pc]);
+}
+
+const std::string&
+action_buffer::getDefinitionURL() const
+{
+    return _src.get_url();
+}
+
+int
+action_buffer::getDefinitionVersion() const
+{
+    return _src.get_version();
+}
+
+namespace {
+
 // Endian conversion routines.
 //
 // Flash format stores integers as little-endian,
@@ -487,12 +502,12 @@ action_buffer::disasm(size_t pc) const
 
 // Read a little-endian 32-bit float from m_buffer[pc]
 // and return it as a host-endian float.
-static float
+float
 convert_float_little(const void *p)
 {
     // Hairy union for endian detection and munging
     union {
-        float    f;
+        float f;
         boost::uint32_t i;
         struct {    // for endian detection
             boost::uint16_t s0;
@@ -508,36 +523,38 @@ convert_float_little(const void *p)
 
     u.f = 1.0;
     switch (u.s.s0) {
-    case 0x0000:    // little-endian host
-        memcpy(&u.i, p, 4);
-        break;
-    case 0x3f80:    // big-endian host
+
+        case 0x0000:    // little-endian host
+            std::memcpy(&u.i, p, 4); // TODO: use std::copy instead ..
+            break;
+        case 0x3f80:    // big-endian host
         {
-        const boost::uint8_t *cp = (const boost::uint8_t *) p;
-        u.c.c0 = cp[3];
-        u.c.c1 = cp[2];
-        u.c.c2 = cp[1];
-        u.c.c3 = cp[0];
+            const boost::uint8_t *cp = static_cast<const boost::uint8_t*>(p);
+            u.c.c0 = cp[3];
+            u.c.c1 = cp[2];
+            u.c.c2 = cp[1];
+            u.c.c3 = cp[0];
+            break;
         }
-        break;
-    default:
-        log_error(_("Native floating point format not recognised"));
-        abort();
+        default:
+            log_error(_("Native floating point format not recognised"));
+            std::abort();
     }
     
     return u.f;
 }
 
+
 // Read a 64-bit double from memory, stored in word-swapped little-endian
 // format and return it as a host-endian double.
 // "Wacky format" is 45670123.
-static double
+double
 convert_double_wacky(const void *p)
 {
-    const boost::uint8_t *cp = (const boost::uint8_t *)p;    // Handy uchar version
+    const boost::uint8_t *cp = static_cast<const boost::uint8_t*>(p);
     union {
-        double    d;
-        boost::uint64_t    i;
+        double d;
+        boost::uint64_t i;
         struct {
             boost::uint32_t l0;
             boost::uint32_t l1;
@@ -566,11 +583,11 @@ convert_double_wacky(const void *p)
     // exactly representable and that has different values in the
     // four 16-bit words.
     // 0x11223344 is represented as 0x41b1 2233 4400 0000 (bigendian)
-    u.d = (double) 0x11223344;
+    u.d = static_cast<double>(0x11223344);
     switch (u.s.s0) {
     case 0x0000:    // pure little-endian host: swap words only.
-        memcpy(&u.l.l1, cp, 4);
-        memcpy(&u.l.l0, cp + 4, 4);
+        std::memcpy(&u.l.l1, cp, 4);
+        std::memcpy(&u.l.l0, cp + 4, 4);
         break;
     case 0x41b1:    // pure big-endian host: swap contents of 32-bit words
         u.c.c0 = cp[3];
@@ -584,7 +601,7 @@ convert_double_wacky(const void *p)
         break;
     case 0x2233:    // word-swapped little-endian host (PDP / ARM FPA)
             // is the same as wacky format.
-        memcpy(&u.i, cp, 8);
+        std::memcpy(&u.i, cp, 8);
         break;
     case 0x4400:    // word-swapped big-endian host: does this exist?
         u.c.c0 = cp[7];
@@ -604,31 +621,8 @@ convert_double_wacky(const void *p)
     return u.d;
 }
 
-float
-action_buffer::read_float_little(size_t pc) const
-{
-    return(convert_float_little(&m_buffer[pc]));
-}
-
-double
-action_buffer::read_double_wacky(size_t pc) const
-{
-    return(convert_double_wacky(&m_buffer[pc]));
-}
-
-const std::string&
-action_buffer::getDefinitionURL() const
-{
-    return _src.get_url();
-}
-
-int
-action_buffer::getDefinitionVersion() const
-{
-    return _src.get_version();
-}
-
-}
+} // unnamed namespace
+} // namespace gnash
 
 // Local Variables:
 // mode: C++
