@@ -1,6 +1,6 @@
 // DefineFontAlignZonesTag.cpp:  for Gnash.
 //
-//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,17 +17,17 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
+#include "DefineFontAlignZonesTag.h"
+
+#include <boost/cstdint.hpp>
 
 #include "RunResources.h"
 #include "Font.h"
 #include "log.h"
 #include "SWFStream.h"
 #include "movie_definition.h"
-#include "DefineFontAlignZonesTag.h"
 #include "smart_ptr.h"
 
-#include <iostream>
-
 namespace gnash {
 namespace SWF {
 
@@ -107,7 +107,7 @@ DefineFontAlignZonesTag::loader(SWFStream& in, TagType tag,
 			
     }
 	in.skip_to_tag_end();
-	LOG_ONCE(log_unimpl(_("*** DefineFontAlignZoneTag")));
+	LOG_ONCE(log_unimpl(_("DefineFontAlignZoneTag")));
 
 }
 
