@@ -1,6 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,9 +16,11 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+#include "DefineVideoStreamTag.h"
+
+#include <algorithm>
 
 #include "RunResources.h"
-#include "DefineVideoStreamTag.h"
 #include "VideoFrameTag.h"
 #include "VideoDecoder.h"
 #include "SWFStream.h" // for read()
@@ -26,8 +28,6 @@
 #include "utility.h"
 #include "smart_ptr.h"
 
-#include <algorithm>
-
 namespace gnash {
 namespace SWF {
 
@@ -61,20 +61,30 @@ VideoFrameTag::loader(SWFStream& in, SWF::TagType tag, movie_definition& m,
         return;
     }
 
-	// TODO: skip if there's no MediaHandler registered ?
+    // TODO: skip if there's no MediaHandler registered ?
 
     const unsigned short padding = 8;
 
-	in.ensureBytes(2);
-	unsigned int frameNum = in.read_u16(); 
+    in.ensureBytes(3);
+    unsigned int frameNum = in.read_u16();
+
+    const media::VideoInfo* info = vs->getVideoInfo();
+
+    if (info && info->codec == media::VIDEO_CODEC_SCREENVIDEO) {
+        // According to swfdec, every SV frame comes with keyframe
+        // and format identifiers (4 bits each), but these are not
+        // part of the codec bitstream and break the decoder.
+        (void) in.read_u8();
+    }
+
 	
-	const unsigned int dataLength = in.get_tag_end_position() - in.tell();
+    const unsigned int dataLength = in.get_tag_end_position() - in.tell();
 
     // FIXME: catch bad_alloc
-	boost::uint8_t* buffer = new boost::uint8_t[dataLength + padding]; 
+    boost::uint8_t* buffer = new boost::uint8_t[dataLength + padding]; 
 
-	const size_t bytesRead = in.read(reinterpret_cast<char*>(buffer),
-            dataLength);
+    const size_t bytesRead = in.read(reinterpret_cast<char*>(buffer),
+                                     dataLength);
 
     if (bytesRead < dataLength)
     {
@@ -85,12 +95,12 @@ VideoFrameTag::loader(SWFStream& in, SWF::TagType tag, movie_definition& m,
 	
     std::fill_n(buffer + bytesRead, padding, 0);
 
-	using namespace media;
+    using namespace media;
 
     std::auto_ptr<EncodedVideoFrame> frame(
             new EncodedVideoFrame(buffer, dataLength, frameNum));
 
-	vs->addVideoFrameTag(frame);
+    vs->addVideoFrameTag(frame);
 }
 
 } // namespace SWF
