@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,8 +22,7 @@
 #include <vector>
 #include <boost/noncopyable.hpp>
 
-#include "smart_ptr.h" // GNASH_USE_GC
-#include "as_function.h"
+#include "smart_ptr.h" 
 #include "ActionExec.h"
 #include "Global_as.h"
 #include "fn_call.h"
@@ -33,24 +32,21 @@ namespace gnash {
 /// Any executable code 
 class ExecutableCode : boost::noncopyable
 {
-
 public:
 
     ExecutableCode(DisplayObject* t) : _target(t) {}
 
-    virtual void execute()=0;
+    virtual void execute() = 0;
 
     virtual ~ExecutableCode() {}
 
     virtual void setReachable() const {}
 
-#ifdef GNASH_USE_GC
     /// Mark reachable resources (for the GC)
     void markReachableResources() const {
         setReachable();
         if (_target) _target->setReachable();
     }
-#endif 
 
     DisplayObject* target() const { 
         return _target;
@@ -62,8 +58,8 @@ private:
 };
 
 /// Global code (out of any function)
-class GlobalCode : public ExecutableCode {
-
+class GlobalCode : public ExecutableCode
+{
 public:
 
     GlobalCode(const action_buffer& nBuffer, DisplayObject* nTarget)
@@ -72,8 +68,7 @@ public:
         buffer(nBuffer)
     {}
 
-    virtual void execute()
-    {
+    virtual void execute() {
         if (!target()->unloaded()) {
             ActionExec exec(buffer, target()->get_environment());
             exec();
@@ -81,14 +76,12 @@ public:
     }
 
 private:
-
     const action_buffer& buffer;
-
 };
 
 /// Event code 
-class EventCode : public ExecutableCode {
-
+class EventCode : public ExecutableCode
+{
 public:
 
     typedef std::vector<const action_buffer*> BufferList;
@@ -111,8 +104,7 @@ public:
     /// and not copied, so make sure it's kept
     /// alive for the whole EventCode lifetime.
     ///
-    void addAction(const action_buffer& buffer)
-    {
+    void addAction(const action_buffer& buffer) {
         // don't push actions for destroyed DisplayObjects, 
         // our opcode guard is bogus at the moment.
         if (!target()->isDestroyed()) {
@@ -120,14 +112,13 @@ public:
         }
     }
 
-    virtual void execute()
-    {
-        for (BufferList::iterator it=_buffers.begin(), itEnd=_buffers.end();
-                it != itEnd; ++it)
-        {
+    virtual void execute() {
+        for (BufferList::iterator it = _buffers.begin(),
+            itEnd = _buffers.end(); it != itEnd; ++it) {
+
             // onClipEvents code are guarded by isDestroyed(),
             // still might be also guarded by unloaded()
-            if (target()->isDestroyed())  break;
+            if (target()->isDestroyed()) break;
 
             ActionExec exec(*(*it), target()->get_environment(), false);
             exec();
@@ -135,14 +126,12 @@ public:
     }
 
 private:
-
     BufferList _buffers;
-
 };
 
 /// Generic event  (constructed by id, invoked using notifyEvent
-class QueuedEvent: public ExecutableCode {
-
+class QueuedEvent : public ExecutableCode
+{
 public:
 
     QueuedEvent(DisplayObject* nTarget, const event_id& id)
@@ -151,19 +140,15 @@ public:
         _eventId(id)
     {}
 
-    virtual void execute()
-    {
+    virtual void execute() {
         // don't execute any events for destroyed DisplayObject.
-        if (!target()->isDestroyed() )
-        {
+        if (!target()->isDestroyed()) {
             target()->notifyEvent(_eventId);
         }
     }
 
 private:
-
     const event_id _eventId;
-
 };
 
 /// This class is used to queue a function call action
@@ -179,11 +164,10 @@ private:
 ///
 class DelayedFunctionCall : public ExecutableCode
 {
-
 public:
 
     DelayedFunctionCall(DisplayObject* target,
-            as_object* obj, string_table::key name,
+            as_object* obj, const ObjectURI& name,
             const as_value& arg1, const as_value& arg2)
         :
         ExecutableCode(target),
@@ -193,31 +177,21 @@ public:
         _arg2(arg2)
     {}
 
-    virtual void execute()
-    {
+    virtual void execute() {
         callMethod(_obj, _name, _arg1, _arg2);
     }
 
-#ifdef GNASH_USE_GC
     /// Mark reachable resources (for the GC)
-    //
-    /// Reachable resources are:
-    ///  - the action target (_target)
-    ///
-    virtual void setReachable() const
-    {
+    virtual void setReachable() const {
         _obj->setReachable();
         _arg1.setReachable();
         _arg2.setReachable();
     }
-#endif // GNASH_USE_GC
 
 private:
-
     as_object* _obj;
-    string_table::key _name;
+    ObjectURI _name;
     as_value _arg1, _arg2;
-
 };
 
 
