@@ -2,7 +2,7 @@
 // asked to do so, so that values outside of the stack are guaranteed good
 // in an appropriate scope.
 //
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,13 +20,12 @@
 #ifndef GNASH_SAFESTACK_H
 #define GNASH_SAFESTACK_H
 
-#include "log.h"
 
 #include <vector>
 
 namespace gnash {
 
-class StackException {/**/};
+class StackException {};
 
 /// A stack in which all references given remain valid while the stack lives.
 ///
@@ -48,92 +47,135 @@ public:
     // May be useful for callers to know.
     typedef typename StackType::size_type StackSize;
 
-	/// From the top of the stack, get the i'th value down. 0 is the topmost
-	/// value.
+	/// From the top of the stack, get the i'th value down.
+    //
+    /// 0 is the topmost value.
+	const T& top(StackSize i) const
+	{
+
+		if (i >= size()) throw StackException();
+		const StackSize offset = _end - i;
+		return _data[offset >> _chunkShift][offset & _chunkMod];
+	}
+
+	/// From the top of the stack, get the i'th value down. 
+    //
+    /// This is a non-const version of top().
+    /// 0 is the topmost value value.
 	T& top(StackSize i)
 	{
 
-		if (i >= size()) 
-			throw StackException();
-		StackSize offset = mEnd - i;
-		//log_debug("top(%d): mEnd:%d, mDownstop:%d, offset:%d", i, mEnd, mDownstop, offset);
-		return mData[offset >> mChunkShift][offset & mChunkMod];
+		if (i >= size()) throw StackException();
+		const StackSize offset = _end - i;
+		return _data[offset >> _chunkShift][offset & _chunkMod];
+	}
+    
+    /// From the top of the stack, get the i'th value down. 
+    //
+    /// 0 is the topmost value value.
+	const T& at(StackSize i) const
+	{
+
+		if (i >= totalSize()) throw StackException();
+		const StackSize offset = _end - i;
+		return _data[offset >> _chunkShift][offset & _chunkMod];
+	}
+
+	
+    /// From the top of the stack, get the i'th value down. 
+    //
+    /// This is a non-const version of at().
+    /// 0 is the topmost value value.
+	T& at(StackSize i)
+	{
+
+		if (i >= totalSize()) throw StackException();
+		const StackSize offset = _end - i;
+		return _data[offset >> _chunkShift][offset & _chunkMod];
 	}
 
 	/// From the bottom of the stack, get the i'th value up. 0 is the
 	/// bottommost value.
 	T& value(StackSize i)
 	{
-		if (i >= size())
-			throw StackException();
+		if (i >= size()) throw StackException();
 
-		StackSize offset = mDownstop + i + 2;
-		//log_debug("value(%d): mEnd:%d, mDownstop:%d, offset:%d", i, mEnd, mDownstop, offset);
-		return mData[offset >> mChunkShift][offset & mChunkMod];
+		StackSize offset = _downstop + i + 2;
+		return _data[offset >> _chunkShift][offset & _chunkMod];
 	}
 
 	const T& value(StackSize i) const
 	{
-		if (i >= size())
-			throw StackException();
+		if (i >= size()) throw StackException();
 
-		StackSize offset = mDownstop + i + 2;
-		//log_debug("value(%d): mEnd:%d, mDownstop:%d, offset:%d", i, mEnd, mDownstop, offset);
-
-		return mData[offset >> mChunkShift][offset & mChunkMod];
+		StackSize offset = _downstop + i + 2;
+		return _data[offset >> _chunkShift][offset & _chunkMod];
 	}
 
 	/// Assign a value to given index counting from bottom.
 	void assign(StackSize i, T val)
 	{ 
-		if (i >= size())
-			throw StackException();
+		if (i >= size()) throw StackException();
 
-		StackSize offset = mDownstop + i + 2;
-		//log_debug("value(%d): mEnd:%d, mDownstop:%d, offset:%d", i, mEnd, mDownstop, offset);
-		mData[offset >> mChunkShift][offset & mChunkMod] = val;
+		StackSize offset = _downstop + i + 2;
+		_data[offset >> _chunkShift][offset & _chunkMod] = val;
 	}
 
 	/// Shrink the stack by i entries. Does not invalidate any entries
 	/// previously given, it just sets the top for pop, push, and top
 	/// operations.
 	void drop(StackSize i)
-	{ if (i > size()) throw StackException(); mEnd -= i; }
+    {
+        if (i > size()) throw StackException();
+        _end -= i;
+    }
 
 	/// Drop all stack elements reguardless of the "downstop"
 	void clear()
-	{ mDownstop=0; mEnd=1; }
+	{
+        _downstop = 0;
+        _end = 1;
+    }
 
 	/// Put a new value onto the top of the stack.  The value will be
 	/// copied.
 	void push(const T t)
-	{ grow(1); top(0) = t; }
+	{
+        grow(1);
+        top(0) = t;
+    }
 
 	/// Pop the top of the stack.
 	T& pop()
-	{	T& ret = top(0); drop(1); return ret; }
+	{
+        T& ret = top(0);
+        drop(1);
+        return ret;
+    }
 
 	/// Grow by i entries. Normally this is 1, but there might be sometime
 	/// when you need more than that.
 	void grow(StackSize i)
 	{
-		StackSize available = (1 << mChunkShift) * mData.size() - mEnd + 1;
+		StackSize available = (1 << _chunkShift) * _data.size() - _end + 1;
 		StackSize n = size()+i;
 		while (available < n)
 		{
-//			log_debug("Increasing size of the real stack: %d.",mData.size());
-			mData.push_back(new T[1 << mChunkShift]);
-			available += 1 << mChunkShift;
+            //log_debug("Increasing size of the real stack: %d.",_data.size());
+			_data.push_back(new T[1 << _chunkShift]);
+			available += 1 << _chunkShift;
 		}
-		mEnd += i;
+		_end += i;
 	}
 
 	/// Gives the size of the stack which is currently accessible.
 	StackSize getDownstop() const 
-	{ return mDownstop; }
+	{
+        return _downstop;
+    }
 
 	/// Alias for getDownstop()
-	StackSize size() const { return mEnd - mDownstop - 1; /*mEnd is one past end*/ }
+	StackSize size() const { return _end - _downstop - 1; }
 
 	/// Is the stack empty to us? (Check totalSize() != for actually empty)
 	bool empty() const { return size() == 0; }
@@ -142,46 +184,52 @@ public:
 	/// to simulate multiple stacks with a single stack, as in function
 	/// calling. Returns the old downstop for restoring it using setDownstop.
 	StackSize fixDownstop() 
-	{ StackSize ret = mDownstop; mDownstop = mEnd-1; return ret; }
+	{
+        StackSize ret = _downstop;
+        _downstop = _end - 1;
+        return ret;
+    }
 
 	/// Makes the stack read to a depth of 'i'. This cannot be more than
 	/// totalSize()
 	void setDownstop(StackSize i)
-	{ if (i > mEnd) throw StackException(); mDownstop = i; }
+	{
+        if (i > _end) throw StackException();
+        _downstop = i;
+    }
 
-	/// The total size of the stack. This is not what can be read. That
-	/// value is given by size()
-	///
-	/// This function is probably not what you need for anything except for
-	/// setting downstops that weren't returned by either fixDownstop() or
-	/// getDownstop()
-	StackSize totalSize() const { return mEnd - 1; }
+    /// Return the complete stack size, including non-accessible elements
+    //
+    /// This is required because AVM2 scope stacks are usable even when they
+    /// appear inaccessible
+	StackSize totalSize() const { return _end - 1; }
 
 	/// Set the total size and local size of the stack, for restoring a
 	/// stack through unknown changes.
 	void setAllSizes(StackSize total, StackSize downstop)
-	{ mEnd = total + 1; mDownstop = downstop; }
+	{
+        _end = total + 1;
+        _downstop = downstop;
+    }
 
 	/// Default constructor.
-	SafeStack() : mData(), mDownstop(0), mEnd(1)
-	{ /**/ }
+	SafeStack() : _data(), _downstop(0), _end(1) {}
 
 	/// Delete the allocated data. 
 	~SafeStack()
 	{
-		for (StackSize i = 0; i < mData.size(); ++i)
-			delete [] mData[i];
+		for (StackSize i = 0; i < _data.size(); ++i) delete [] _data[i];
 	}
 
 private:
-	StackType mData;
-	StackSize mDownstop;
-	StackSize mEnd;
+	StackType _data;
+	StackSize _downstop;
+	StackSize _end;
 
-	// If mChunkMod is not a power of 2 less 1, it will not work properly.
-	static const StackSize mChunkShift = 6;
-	static const StackSize mChunkMod = (1 << mChunkShift) - 1;
+	// If _chunkMod is not a power of 2 less 1, it will not work properly.
+	static const StackSize _chunkShift = 6;
+	static const StackSize _chunkMod = (1 << _chunkShift) - 1;
 };
 
 } // namespace gnash
-#endif /* GNASH_SAFESTACK_H */
+#endif 
