@@ -1,6 +1,6 @@
 // FLVParser.cpp:  Flash Video file parser, for Gnash.
 //
-//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -188,10 +188,10 @@ FLVParser::parseAudioTag(const FLVTag& flvtag, const FLVAudioTag& audiotag, boos
 
 	// If this is the first audioframe no info about the
 	// audio format has been noted, so we do that now
-	if ( !_audioInfo.get() )
-	{
+	if (!_audioInfo.get()) {
 		_audioInfo.reset(new AudioInfo(audiotag.codec, audiotag.samplerate,
-                    audiotag.samplesize, audiotag.stereo, 0, FLASH) );
+                    audiotag.samplesize, audiotag.stereo, 0,
+                    CODEC_TYPE_FLASH));
 
         if (header) {
 
@@ -266,7 +266,8 @@ FLVParser::parseVideoTag(const FLVTag& flvtag, const FLVVideoTag& videotag, boos
 	// If this is the first videoframe no info about the
 	// video format has been noted, so we do that now
 	if ( ! _videoInfo.get() ) {
-		_videoInfo.reset(new VideoInfo(videotag.codec, 0, 0, 0, 0, FLASH));
+		_videoInfo.reset(new VideoInfo(videotag.codec, 0, 0, 0, 0,
+                    CODEC_TYPE_FLASH));
 
 		if (header) {
             // The frame is 0-padded up to the end. It may be larger than
@@ -306,7 +307,7 @@ FLVParser::parseNextTag(bool index_only)
 		_seekRequest = false;
 	}
 
-	uint64_t& position = index_only ? _nextPosToIndex : _lastParsedPosition;
+	boost::uint64_t& position = index_only ? _nextPosToIndex : _lastParsedPosition;
 	bool& completed = index_only ? _indexingCompleted : _parsingComplete;
 
 	//log_debug("parseNextTag: _lastParsedPosition:%d, _nextPosToIndex:%d, index_only:%d", _lastParsedPosition, _nextPosToIndex, index_only);
