@@ -1,6 +1,6 @@
 // MediaParser.cpp:  Media file parser, for Gnash.
 //
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -64,6 +64,17 @@ MediaParser::getBufferLength() const
 	return getBufferLengthNoLock();
 }
 
+/* public */
+bool
+MediaParser::isBufferEmpty() const
+{
+#ifdef LOAD_MEDIA_IN_A_SEPARATE_THREAD
+	boost::mutex::scoped_lock lock(_qMutex);
+#endif
+	return _videoFrames.empty() && _audioFrames.empty();
+}
+
+
 boost::uint64_t
 MediaParser::getBufferLengthNoLock() const
 {
@@ -107,11 +118,12 @@ MediaParser::audioBufferLength() const
 	return _audioFrames.back()->timestamp - _audioFrames.front()->timestamp; 
 }
 
+/*private*/
 const EncodedVideoFrame*
 MediaParser::peekNextVideoFrame() const
 {
 #ifdef LOAD_MEDIA_IN_A_SEPARATE_THREAD
-	boost::mutex::scoped_lock lock(_qMutex);
+    // TODO: assert _qMutex is locked by this thread
 #else // ndef LOAD_MEDIA_IN_A_SEPARATE_THREAD
 	while (!parsingCompleted() && _videoInfo.get() && _videoFrames.empty())
 	{
@@ -123,9 +135,52 @@ MediaParser::peekNextVideoFrame() const
 	return _videoFrames.front();
 }
 
+bool
+MediaParser::nextFrameTimestamp(boost::uint64_t& ts) const
+{
+#ifdef LOAD_MEDIA_IN_A_SEPARATE_THREAD
+    boost::mutex::scoped_lock lock(_qMutex);
+#else // ndef LOAD_MEDIA_IN_A_SEPARATE_THREAD
+    while (!parsingCompleted() && _videoInfo.get() && _videoFrames.empty())
+    {
+        const_cast<MediaParser*>(this)->parseNextChunk();
+    }
+#endif
+
+    if (_videoFrames.empty())
+    {
+        if (_audioFrames.empty())
+        {
+            return false;
+        }
+        else
+        {
+            ts = _audioFrames.front()->timestamp;
+            return true;
+        }
+    }
+    else
+    {
+        if (_audioFrames.empty())
+        {
+            ts = _videoFrames.front()->timestamp();
+            return true;
+        }
+        else
+        {
+            ts = std::min(_videoFrames.front()->timestamp(),
+                          _audioFrames.front()->timestamp);
+            return true;
+        }
+    }
+}
+
 bool
 MediaParser::nextVideoFrameTimestamp(boost::uint64_t& ts) const
 {
+#ifdef LOAD_MEDIA_IN_A_SEPARATE_THREAD
+	boost::mutex::scoped_lock lock(_qMutex);
+#endif // def LOAD_MEDIA_IN_A_SEPARATE_THREAD
 	const EncodedVideoFrame* ef = peekNextVideoFrame();
 	if ( ! ef ) return false;
 	ts = ef->timestamp();
@@ -181,17 +236,21 @@ MediaParser::nextAudioFrame()
 bool
 MediaParser::nextAudioFrameTimestamp(boost::uint64_t& ts) const
 {
+#ifdef LOAD_MEDIA_IN_A_SEPARATE_THREAD
+	boost::mutex::scoped_lock lock(_qMutex);
+#endif // def LOAD_MEDIA_IN_A_SEPARATE_THREAD
 	const EncodedAudioFrame* ef = peekNextAudioFrame();
 	if ( ! ef ) return false;
 	ts = ef->timestamp;
 	return true;
 }
 
+/*private*/
 const EncodedAudioFrame*
 MediaParser::peekNextAudioFrame() const
 {
 #ifdef LOAD_MEDIA_IN_A_SEPARATE_THREAD
-	boost::mutex::scoped_lock lock(_qMutex);
+    // TODO: assert _qMutex is locked by this thread
 #else // ndef LOAD_MEDIA_IN_A_SEPARATE_THREAD
 	while (!parsingCompleted() && _audioInfo.get() && _audioFrames.empty())
 	{
@@ -371,7 +430,14 @@ MediaParser::parserLoop()
 	while (!parserThreadKillRequested())
 	{
 		parseNextChunk();
-        	gnashSleep(100); // no rush....
+		gnashSleep(100); // thread switch 
+
+		// check for parsing complete
+		// TODO: have a setParsingComplete() function
+		//       exposed in base class for taking care
+		//       of this on appropriate time.
+		boost::mutex::scoped_lock lock(_qMutex);
+		waitIfNeeded(lock);
 	}
 }
 
