@@ -1,6 +1,6 @@
 // MediaHandlerFfmpeg.h: FFMPEG media handler, for Gnash
 // 
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,16 +16,12 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-
 #ifndef GNASH_MEDIAHANDLERFFMPEG_H
 #define GNASH_MEDIAHANDLERFFMPEG_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "MediaHandler.h" // for inheritance
 
+#include <vector>
 #include <memory>
 
 namespace gnash {
@@ -44,10 +40,12 @@ namespace media {
 namespace ffmpeg {
 
 /// FFMPEG based MediaHandler
-class DSOEXPORT MediaHandlerFfmpeg : public MediaHandler
+class MediaHandlerFfmpeg : public MediaHandler
 {
 public:
 
+    std::string description() const;
+
 	virtual std::auto_ptr<MediaParser>
         createMediaParser(std::auto_ptr<IOChannel> stream);
 
@@ -64,6 +62,8 @@ public:
     virtual size_t getInputPaddingSize() const;
     
     virtual VideoInput* getVideoInput(size_t index);
+    
+    virtual AudioInput* getAudioInput(size_t index);
 
     virtual void cameraNames(std::vector<std::string>& names) const;
 
