@@ -1,6 +1,6 @@
-// MediaParserFfmpeg.cpp: FFMPG media parsers, for Gnash
+// MediaParserFfmpeg.cpp: FFMPEG media parsers, for Gnash
 //
-//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -200,8 +200,9 @@ MediaParserFfmpeg::parseAudioFrame(AVPacket& packet)
         //   mdb:93, lastbuf:0 skiping granule 0
         // When playing: http://downloads.bbc.co.uk/news/nol/shared/spl/hi/audio_slideshow/kenadamptw/slideshow_629.swf
         //
-        log_error("FIXME: FFMPEG packet decompression timestamp has no value, taking as zero");
-        dts=0;
+        LOG_ONCE(log_error("FIXME: FFmpeg packet decompression "
+                    "timestamp has no value, taking as zero"));
+        dts = 0;
     }
 	boost::uint64_t timestamp = static_cast<boost::uint64_t>(dts * as_double(_audioStream->time_base) * 1000.0); 
     //log_debug("On getting audio frame with timestamp %d, duration is %d", timestamp, _audioStream->duration);
@@ -334,171 +335,170 @@ MediaParserFfmpeg::MediaParserFfmpeg(std::auto_ptr<IOChannel> stream)
 void
 MediaParserFfmpeg::initializeParser()
 {
-	av_register_all(); // TODO: needs to be invoked only once ?
+    av_register_all(); // TODO: needs to be invoked only once ?
 
-	_byteIOCxt.buffer = NULL;
-
-	_inputFmt = probeStream();
+    _byteIOCxt.buffer = NULL;
+    
+    _inputFmt = probeStream();
 #ifdef GNASH_ALLOW_VCODEC_ENV	
-	if ( ! _inputFmt )
-	{
-	  char* defcodec = getenv("GNASH_DEFAULT_VCODEC");
-	  if (defcodec && strlen(defcodec))
-      _inputFmt = av_find_input_format(defcodec);	
-
-  }
+    if ( ! _inputFmt ) {
+	char* defcodec = getenv("GNASH_DEFAULT_VCODEC");
+	if (defcodec && strlen(defcodec))
+	    _inputFmt = av_find_input_format(defcodec);	
+	
+    }
 #endif	
-	if ( ! _inputFmt )
-	{
-		throw MediaException("MediaParserFfmpeg couldn't figure out input "
-                "format");
-	}
-
-	_formatCtx = av_alloc_format_context();
-	assert(_formatCtx);
-
-	// Setup the filereader/seeker mechanism. 7th argument (NULL) is the writer function,
-	// which isn't needed.
-	_byteIOBuffer.reset( new unsigned char[byteIOBufferSize] );
-	init_put_byte(&_byteIOCxt,
-		_byteIOBuffer.get(), // buffer
-		byteIOBufferSize, // buffer size
-		0, // write flags
-		this, // opaque pointer to pass to the callbacks
-		MediaParserFfmpeg::readPacketWrapper, // packet reader callback
-		NULL, // packet writer callback
-		MediaParserFfmpeg::seekMediaWrapper // seeker callback
-		);
-
-	_byteIOCxt.is_streamed = 1;
-
-	// Open the stream. the 4th argument is the filename, which we ignore.
-	if(av_open_input_stream(&_formatCtx, &_byteIOCxt, "", _inputFmt, NULL) < 0)
-	{
-		throw IOException("MediaParserFfmpeg couldn't open input stream");
-	}
-
-	log_debug("Parsing FFMPEG media file: format:%s; nstreams:%d", _inputFmt->name, _formatCtx->nb_streams);
-
-	if ( _formatCtx->title[0] )     log_debug(_("  Title:'%s'"), _formatCtx->title);
-	if ( _formatCtx->author[0] )    log_debug(_("  Author:'%s'"), _formatCtx->author);
-	if ( _formatCtx->copyright[0] ) log_debug(_("  Copyright:'%s'"), _formatCtx->copyright);
-	if ( _formatCtx->comment[0] )   log_debug(_("  Comment:'%s'"), _formatCtx->comment);
-	if ( _formatCtx->album[0] )     log_debug(_("  Album:'%s'"), _formatCtx->album);
-
-	// Find first audio and video stream
-	for (unsigned int i = 0; i < static_cast<unsigned int>(_formatCtx->nb_streams); i++)
-	{
-		AVStream* stream = _formatCtx->streams[i];
-		if ( ! stream ) 
-		{
-			log_debug("Stream %d of FFMPEG media file is null ?", i);
-			continue;
-		}
-
-		AVCodecContext* enc = stream->codec; 
-		if ( ! enc ) 
-		{
-			log_debug("Stream %d of FFMPEG media file has no codec info", i);
-			continue;
-		}
-
-		switch (enc->codec_type)
-		{
-			case CODEC_TYPE_AUDIO:
-				if (_audioStreamIndex < 0)
-				{
-					_audioStreamIndex = i;
-					_audioStream = _formatCtx->streams[i];
-					log_debug(_("  Using stream %d for audio: codec id %d"),
-						i, _audioStream->codec->codec_id);
-					// codec_name will only be filled by avcodec_find_decoder (later);
-				}
-				break;
-
-			case CODEC_TYPE_VIDEO:
-				if (_videoStreamIndex < 0)
-				{
-					_videoStreamIndex = i;
-					_videoStream = _formatCtx->streams[i];
-					log_debug(_("  Using stream %d for video: codec id %d"),
-						i, _videoStream->codec->codec_id);
-					// codec_name will only be filled by avcodec_find_decoder (later);
-				}
-				break;
-			default:
-				break;
-		}
-	}
-
-	// Create VideoInfo
-	if ( _videoStream)
-	{
-		int codec = static_cast<int>(_videoStream->codec->codec_id); // originally an enum CodecID 
-		boost::uint16_t width = _videoStream->codec->width;
-		boost::uint16_t height = _videoStream->codec->height;
-		boost::uint16_t frameRate = static_cast<boost::uint16_t>(as_double(_videoStream->r_frame_rate));
-#if !defined(HAVE_LIBAVFORMAT_AVFORMAT_H) && !defined(HAVE_FFMPEG_AVCODEC_H)
-		boost::uint64_t duration = _videoStream->codec_info_duration;
+    if ( ! _inputFmt ) {
+	throw MediaException("MediaParserFfmpeg couldn't figure out input "
+			     "format");
+    }
+    
+// av_alloc_format_context was deprecated on
+// 2009-02-08 (r17047) in favor of avformat_alloc_context() 
+#if !defined (LIBAVCODEC_VERSION_MAJOR) || LIBAVCODEC_VERSION_MAJOR < 52
+    _formatCtx = av_alloc_format_context();
 #else
-		boost::uint64_t duration = _videoStream->duration;
+    _formatCtx = avformat_alloc_context();
 #endif
-        if ( duration == AV_NOPTS_VALUE )
-        {
+
+    assert(_formatCtx);
+    
+    // Setup the filereader/seeker mechanism.
+    // 7th argument (NULL) is the writer function,
+    // which isn't needed.
+    _byteIOBuffer.reset( new unsigned char[byteIOBufferSize] );
+    init_put_byte(&_byteIOCxt,
+		  _byteIOBuffer.get(), // buffer
+		  byteIOBufferSize, // buffer size
+		  0, // write flags
+		  this, // opaque pointer to pass to the callbacks
+		  MediaParserFfmpeg::readPacketWrapper, // packet reader callback
+		  NULL, // packet writer callback
+		  MediaParserFfmpeg::seekMediaWrapper // seeker callback
+		  );
+    
+    _byteIOCxt.is_streamed = 1;
+    
+    // Open the stream. the 4th argument is the filename, which we ignore.
+    if(av_open_input_stream(&_formatCtx, &_byteIOCxt, "", _inputFmt, NULL) < 0)
+    {
+        throw IOException("MediaParserFfmpeg couldn't open input stream");
+    }
+    
+    log_debug("Parsing FFMPEG media file: format:%s; nstreams:%d",
+        _inputFmt->name, _formatCtx->nb_streams);
+    
+    if ( _formatCtx->title[0] )
+        log_debug(_("  Title:'%s'"), _formatCtx->title);
+    if ( _formatCtx->author[0] )
+        log_debug(_("  Author:'%s'"), _formatCtx->author);
+    if ( _formatCtx->copyright[0] )
+        log_debug(_("  Copyright:'%s'"), _formatCtx->copyright);
+    if ( _formatCtx->comment[0] )
+        log_debug(_("  Comment:'%s'"), _formatCtx->comment);
+    if ( _formatCtx->album[0] )
+        log_debug(_("  Album:'%s'"), _formatCtx->album);
+    
+    // Find first audio and video stream
+    for (unsigned int i = 0; i < static_cast<unsigned int>(_formatCtx->nb_streams); i++)
+	{
+	    AVStream* stream = _formatCtx->streams[i];
+	    if ( ! stream ) {
+		log_debug("Stream %d of FFMPEG media file is null ?", i);
+		continue;
+	    }
+	    
+	    AVCodecContext* enc = stream->codec; 
+	    if ( ! enc ) {
+		log_debug("Stream %d of FFMPEG media file has no codec info", i);
+		continue;
+	    }
+	    
+	    switch (enc->codec_type) {
+	    case CODEC_TYPE_AUDIO:
+		if (_audioStreamIndex < 0) {
+		    _audioStreamIndex = i;
+		    _audioStream = _formatCtx->streams[i];
+		    log_debug(_("  Using stream %d for audio: codec id %d"),
+			      i, _audioStream->codec->codec_id);
+		    // codec_name will only be filled by avcodec_find_decoder (later);
+		}
+		break;
+		
+	    case CODEC_TYPE_VIDEO:
+		if (_videoStreamIndex < 0) {
+		    _videoStreamIndex = i;
+		    _videoStream = _formatCtx->streams[i];
+		    log_debug(_("  Using stream %d for video: codec id %d"),
+			      i, _videoStream->codec->codec_id);
+		    // codec_name will only be filled by avcodec_find_decoder (later);
+		}
+		break;
+	    default:
+		break;
+	    }
+	}
+    
+    // Create VideoInfo
+    if ( _videoStream) {
+        const int codec = static_cast<int>(_videoStream->codec->codec_id); 
+        boost::uint16_t width = _videoStream->codec->width;
+        boost::uint16_t height = _videoStream->codec->height;
+        boost::uint16_t frameRate = static_cast<boost::uint16_t>(as_double(_videoStream->r_frame_rate));
+#if !defined(HAVE_LIBAVFORMAT_AVFORMAT_H) && !defined(HAVE_FFMPEG_AVCODEC_H)
+        boost::uint64_t duration = _videoStream->codec_info_duration;
+#else
+        boost::uint64_t duration = _videoStream->duration;
+#endif
+        if (duration == AV_NOPTS_VALUE) {
             log_error("Duration of video stream unknown");
             duration=0; // TODO: guess!
-        }
-        else
-        {
+        } else {
             duration = duration / as_double(_videoStream->time_base); // TODO: check this
         }
-
-		_videoInfo.reset( new VideoInfo(codec, width, height, frameRate, duration, CUSTOM /*codec type*/) );
-		
-		_videoInfo->extra.reset(new ExtraVideoInfoFfmpeg(
-			// NOTE: AVCodecContext.extradata : void* for 51.11.0, uint8_t* for 51.38.0
-			(uint8_t*)_videoStream->codec->extradata,
-			_videoStream->codec->extradata_size));
-
-		//log_debug("EXTRA: %d bytes of video extra data", _videoStream->codec->extradata_size);
-	}
-
-	// Create AudioInfo
-	if ( _audioStream)
-	{
-		int codec = static_cast<int>(_audioStream->codec->codec_id); // originally an enum CodecID 
-		boost::uint16_t sampleRate = _audioStream->codec->sample_rate;
-		boost::uint16_t sampleSize = SampleFormatToSampleSize(_audioStream->codec->sample_fmt);
-		bool stereo = (_audioStream->codec->channels == 2);
+	
+        _videoInfo.reset(new VideoInfo(codec, width, height, frameRate,
+                    duration, CODEC_TYPE_CUSTOM /*codec type*/));
+	
+        // NOTE: AVCodecContext.extradata : void* for 51.11.0, uint8_t* for 51.38.0
+        _videoInfo->extra.reset(new ExtraVideoInfoFfmpeg(
+                     (uint8_t*)_videoStream->codec->extradata,
+                     _videoStream->codec->extradata_size));
+	
+    }
+    
+    // Create AudioInfo
+    if ( _audioStream) {
+        const int codec = static_cast<int>(_audioStream->codec->codec_id); 
+        boost::uint16_t sampleRate = _audioStream->codec->sample_rate;
+        boost::uint16_t sampleSize = SampleFormatToSampleSize(_audioStream->codec->sample_fmt);
+        bool stereo = (_audioStream->codec->channels == 2);
 #if !defined(HAVE_LIBAVFORMAT_AVFORMAT_H) && !defined(HAVE_FFMPEG_AVCODEC_H)
-		boost::uint64_t duration = _audioStream->codec_info_duration;
+        boost::uint64_t duration = _audioStream->codec_info_duration;
 #else
-		boost::uint64_t duration = _audioStream->duration;
+        boost::uint64_t duration = _audioStream->duration;
 #endif
-        if ( duration == AV_NOPTS_VALUE )
-        {
+        if (duration == AV_NOPTS_VALUE) {
             log_error("Duration of audio stream unknown to ffmpeg");
             duration=0; // TODO: guess!
-        }
-        else
-        {
+        } 
+        else {
             duration = duration / as_double(_audioStream->time_base); // TODO: check this
         }
-
-		_audioInfo.reset( new AudioInfo(codec, sampleRate, sampleSize, stereo, duration, CUSTOM /*codec type*/) );
-
-		_audioInfo->extra.reset(new ExtraAudioInfoFfmpeg(
-			// NOTE: AVCodecContext.extradata : void* for 51.11.0, uint8_t* for 51.38.0
-			(uint8_t*)_audioStream->codec->extradata,
-			_audioStream->codec->extradata_size));
-
-		//log_debug("EXTRA: %d bytes of audio extra data", _videoStream->codec->extradata_size);
-	}
-
-
+	
+        _audioInfo.reset(new AudioInfo(codec, sampleRate, sampleSize, stereo,
+                    duration, CODEC_TYPE_CUSTOM /*codec type*/));
+        
+        // NOTE: AVCodecContext.extradata : void* for 51.11.0, uint8_t* for 51.38.0
+        _audioInfo->extra.reset(new ExtraAudioInfoFfmpeg(
+                     (uint8_t*)_audioStream->codec->extradata,
+                     _audioStream->codec->extradata_size));
+	
+    }
+    
+    
 }
-
-
+    
 MediaParserFfmpeg::~MediaParserFfmpeg()
 {
 	stopParserThread();
@@ -506,7 +506,7 @@ MediaParserFfmpeg::~MediaParserFfmpeg()
 	if ( _formatCtx )
 	{
 		// TODO: check if this is correct (should we create RIIA classes for ffmpeg stuff?)
-		//av_close_input_file(_formatCtx); // NOTE: this one triggers a mismatched free/delete on _byteIOBuffer !
+		//av_close_input_file(_formatCtx); // NOTE: this one triggers a mismatched free/delete on _byteIOBuffer with libavformat.so.52 !
 		av_free(_formatCtx);
 	}
 
@@ -518,6 +518,11 @@ MediaParserFfmpeg::~MediaParserFfmpeg()
 
 }
 
+// NOTE: as this function is used as a callback from FFMPEG, it should not
+// throw any exceptions, because:
+// a) The behaviour of C++ exceptions passed into C code is undefined.
+// b) Even if we don't crash and burn, the FFMPEG parser is left in an
+//    undefined state.
 int 
 MediaParserFfmpeg::readPacket(boost::uint8_t* buf, int buf_size)
 {
@@ -530,31 +535,55 @@ MediaParserFfmpeg::readPacket(boost::uint8_t* buf, int buf_size)
 
 }
 
+// NOTE: as this function is used as a callback from FFMPEG, it should not
+// throw any exceptions, because:
+// a) The behaviour of C++ exceptions passed into C code is undefined.
+// b) Even if we don't crash and burn, the FFMPEG parser is left in an
+//    undefined state.
 boost::int64_t 
 MediaParserFfmpeg::seekMedia(boost::int64_t offset, int whence)
 {
-	GNASH_REPORT_FUNCTION;
+	//GNASH_REPORT_FUNCTION;
+	//log_debug("::seekMedia(%1%, %2%)", offset, whence);
 
 	assert(_stream.get());
 
-	// Offset is absolute new position in the file
 	if (whence == SEEK_SET)
 	{	
-		_stream->seek(offset);
-		// New position is offset + old position
+		// Offset is absolute new position in the file
+		if ( offset < 0 ) {
+			boost::format fmt = boost::format(
+				_("MediaParserFfmpeg couldn't parse input format: "
+				"tried to seek at negative offset %1%."))
+				% offset;
+	   		log_error(fmt);
+			return -1;
+		} else {
+			_stream->seek(offset);
+		}
 	}
 	else if (whence == SEEK_CUR)
 	{
+		// New position is offset + old position
 		_stream->seek(_stream->tell() + static_cast<std::streamoff>(offset));
-		// New position is offset + end of file
 	}
 	else if (whence == SEEK_END)
 	{
+		// New position is offset + end of file
+		log_unimpl("MediaParserFfmpeg seek from end of file");
 		// This is (most likely) a streamed file, so we can't seek to the end!
 		// Instead we seek to byteIOBufferSize bytes... seems to work fine...
 		_stream->seek(byteIOBufferSize);
 
 	}
+	else
+	{
+		// ffmpeg uses whence=AVSEEK_SIZE and offset=0 to request
+		// stream size !
+		log_unimpl("MediaParserFfmpeg: unsupported whence value %d", whence);
+		return -1;
+	}
+
 
 	return _stream->tell(); 
 }
