@@ -1,6 +1,6 @@
-// MediaParserFfmpeg.h: FFMPG media parsers, for Gnash
+// MediaParserFfmpeg.h: FFMEPG media parsers, for Gnash
 // 
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,18 +16,12 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-
 #ifndef GNASH_MEDIAPARSER_FFMPEG_H
 #define GNASH_MEDIAPARSER_FFMPEG_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "MediaParser.h" // for inheritance
 #include "ffmpegHeaders.h"
 
-#include <vector>
 #include <boost/scoped_array.hpp>
 #include <memory>
 
