@@ -1,6 +1,7 @@
 // VideoDecoderGst.cpp: Video decoding using Gstreamer.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,11 +17,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "VideoDecoderGst.h"
 #include "MediaParserGst.h"
 #include "GstUtil.h"
@@ -97,7 +93,8 @@ VideoDecoderGst::VideoDecoderGst(videoCodecType codec_type,
       throw MediaException(_("Video codec is zero.  Streaming video expected later."));
       break;
     default:
-      boost::format msg = boost::format(_("No support for video codec %d.")) % (int)codec_type;
+      boost::format msg = boost::format(_("No support for video codec %s.")) %
+          codec_type;
       throw MediaException(msg.str());
 
       return;
@@ -151,7 +148,12 @@ VideoDecoderGst::setup(GstCaps* srccaps)
     bool rv = swfdec_gst_decoder_init (&_decoder, srccaps, sinkcaps,
             "ffmpegcolorspace", NULL);
     if (!rv) {
-        throw MediaException(_("VideoDecoderGst: initialisation failed."));      
+        GstStructure* sct = gst_caps_get_structure(srccaps, 0);
+        std::string type(gst_structure_get_name(sct));
+        std::string msg = (boost::format(
+            _("VideoDecoderGst: initialisation failed for video type %s!"))
+            % type).str();
+        throw MediaException(msg);
     }
 
     gst_caps_unref (srccaps);
@@ -185,13 +187,13 @@ VideoDecoderGst::push(const EncodedVideoFrame& frame)
 }
   
 
-std::auto_ptr<GnashImage>
+std::auto_ptr<image::GnashImage>
 VideoDecoderGst::pop()
 {
     GstBuffer * buffer = swfdec_gst_decoder_pull (&_decoder);
 
     if (!buffer) {
-        return std::auto_ptr<GnashImage>();
+        return std::auto_ptr<image::GnashImage>();
     }
   
     GstCaps* caps = gst_buffer_get_caps(buffer);
@@ -205,7 +207,7 @@ VideoDecoderGst::pop()
 
     gst_caps_unref(caps);
   
-    std::auto_ptr<GnashImage> ret(new gnashGstBuffer(buffer, _width, _height));
+    std::auto_ptr<image::GnashImage> ret(new gnashGstBuffer(buffer, _width, _height));
   
     return ret;
 }
