@@ -1,7 +1,7 @@
 // EmbedSound.h - embedded sound definition, for gnash
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "SoundInfo.h" // for composition
 #include "SoundEnvelope.h" // for SoundEnvelopes define
 
+#include <vector>
 #include <map> // for composition (m_frame_size)
 #include <memory> // for auto_ptr (composition)
 #include <set> // for composition (_soundInstances)
@@ -57,14 +58,17 @@ public:
 
     /// Construct a sound with given data, info and volume.
     //
-    /// @param data The encoded sound data. May be the NULL pointer for streaming sounds,
+    /// @param data The encoded sound data.
+    ///     May be the NULL pointer for streaming sounds,
     ///     in which case data will be appended later using ::append()
     ///
     /// @param info encoding info
     ///
     /// @param nVolume initial volume (0..100). Optional, defaults to 100.
     ///
-    EmbedSound(std::auto_ptr<SimpleBuffer> data, std::auto_ptr<media::SoundInfo> info, int nVolume=100);
+    EmbedSound(std::auto_ptr<SimpleBuffer> data,
+            std::auto_ptr<media::SoundInfo> info, int nVolume,
+            size_t paddingBytes);
 
     ~EmbedSound();
 
@@ -244,6 +248,8 @@ public:
     /// @todo make private and mark EmbedSoundInst as friend ?
     ///
     void eraseActiveSound(EmbedSoundInst* inst);
+
+    const size_t _paddingBytes;
 };
 
 } // gnash.sound namespace 
