@@ -1,7 +1,7 @@
 // sound_handler_ahi.cpp: Sound handling using standard AHI
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 #include "SoundInfo.h"
 #include "EmbedSound.h"
 #include "AuxStream.h" // for use..
-#include "../../libcore/vm/VM.h"
 
 #include "log.h" // will import boost::format too
 #include "GnashException.h" // for SoundException
@@ -69,60 +68,12 @@ audioTaskWrapper()
 	return obj->audioTask();
 }
 
-namespace { // anonymous
-
-// Header of a wave file
-// http://ftp.iptel.org/pub/sems/doc/full/current/wav__hdr_8c-source.html
-typedef struct{
-     char rID[4];            // 'RIFF'
-     long int rLen;
-     char wID[4];            // 'WAVE'
-     char fId[4];            // 'fmt '
-     long int pcm_header_len;   // varies...
-     short int wFormatTag;
-     short int nChannels;      // 1,2 for stereo data is (l,r) pairs
-     long int nSamplesPerSec;
-     long int nAvgBytesPerSec;
-     short int nBlockAlign;
-     short int nBitsPerSample;
-} WAV_HDR;
-
-// Chunk of wave file
-// http://ftp.iptel.org/pub/sems/doc/full/current/wav__hdr_8c-source.html
-typedef struct{
-    char dId[4];            // 'data' or 'fact'
-    long int dLen;
-} CHUNK_HDR;
-
-} // end of anonymous namespace
-
 namespace gnash {
 namespace sound {
 
-AOS4_sound_handler::AOS4_sound_handler(const std::string& wavefile)
-    :
-    _audioOpened(false),
-	_closing(false)
-{
-
-    initAudio();
-
-    if (! wavefile.empty() ) {
-        file_stream.open(wavefile.c_str());
-        if (file_stream.fail()) {
-            std::cerr << "Unable to write file '" << wavefile << std::endl;
-            exit(EXIT_FAILURE);
-        } else {
-                write_wave_header(file_stream);
-                std::cout << "# Created 44100 16Mhz stereo wave file:" << std::endl <<
-                    "AUDIOFILE=" << wavefile << std::endl;
-        }
-    }
-
-}
-
-AOS4_sound_handler::AOS4_sound_handler()
+AOS4_sound_handler::AOS4_sound_handler(media::MediaHandler* m)
     :
+    sound_handler(m),
     _audioOpened(false),
 	_closing(false)
 {
@@ -311,54 +262,10 @@ AOS4_sound_handler::tell(int soundHandle)
 }
 
 sound_handler*
-create_sound_handler_aos4()
+create_sound_handler_aos4(media::MediaHandler* m)
 // Factory.
 {
-    return new AOS4_sound_handler;
-}
-
-sound_handler*
-create_sound_handler_aos4(const std::string& wave_file)
-// Factory.
-{
-    return new AOS4_sound_handler(wave_file);
-}
-
-void
-AOS4_sound_handler::write_wave_header(std::ofstream& outfile)
-{
-
-  // allocate wav header
-  WAV_HDR wav;
-  CHUNK_HDR chk;
-
-  // setup wav header
-  std::strncpy(wav.rID, "RIFF", 4);
-  std::strncpy(wav.wID, "WAVE", 4);
-  std::strncpy(wav.fId, "fmt ", 4);
-
-  wav.nBitsPerSample = 16;
-  wav.nSamplesPerSec = 44100;
-  wav.nAvgBytesPerSec = 44100;
-  wav.nAvgBytesPerSec *= wav.nBitsPerSample / 8;
-  wav.nAvgBytesPerSec *= 2;
-  wav.nChannels = 2;
-
-  wav.pcm_header_len = 16;
-  wav.wFormatTag = 1;
-  wav.rLen = sizeof(WAV_HDR) + sizeof(CHUNK_HDR);
-  wav.nBlockAlign = 2 * wav.nBitsPerSample / 8;
-
-  // setup chunk header
-  std::strncpy(chk.dId, "data", 4);
-  chk.dLen = 0;
-
-  /* write riff/wav header */
-  outfile.write((char *)&wav, sizeof(WAV_HDR));
-
-  /* write chunk header */
-  outfile.write((char *)&chk, sizeof(CHUNK_HDR));
-
+    return new AOS4_sound_handler(m);
 }
 
 void
@@ -377,19 +284,6 @@ AOS4_sound_handler::fetchSamples(boost::int16_t* to, unsigned int nSamples)
 		else		
 			return;
 		
-	    // TODO: move this to base class !
-    	if (file_stream)
-	    {
-    	    // NOTE: if muted, the samples will be silent already
-        	boost::uint8_t* stream = reinterpret_cast<boost::uint8_t*>(to);
-	        unsigned int len = nSamples*2;
-
-    	    file_stream.write((char*) stream, len);
-
-        	// now, mute all audio
-	        std::fill(to, to+nSamples, 0);
-    	}
-
 		//memcpy(BufferPointer, inSamples, nBytes);
 		memcpy(BufferPointer, to, nSamples*2);
 
@@ -495,7 +389,7 @@ AOS4_sound_handler::mix(boost::int16_t* outSamples, boost::int16_t* inSamples, u
 	    boost::uint8_t *out = reinterpret_cast<boost::uint8_t*>(outSamples);
     	boost::uint8_t* in = reinterpret_cast<boost::uint8_t*>(inSamples);
 
-	    MixAudio(out, in, nBytes, MIX_MAXVOLUME*volume);
+	    MixAudio(out, in, nBytes, static_cast<int>(MIX_MAXVOLUME*volume));
 	}
 }
 
