@@ -1,6 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,12 +16,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-
-/// \page sound_handler_intro Sound handler introduction
-///
-/// The implementation of this class *must* be thread safe!
-///
-
 #ifndef SOUND_HANDLER_H
 #define SOUND_HANDLER_H
 
@@ -33,13 +27,16 @@
 #include "MediaHandler.h" // for inlined ctor
 #include "SoundEnvelope.h" // for SoundEnvelopes typedef
 #include "AuxStream.h" // for aux_stramer_ptr typedef
+#include "WAVWriter.h" // for dtor visibility 
 
+#include <string>
 #include <vector>
 #include <memory>
 #include <cassert>
 #include <cstring>
 #include <limits>
 #include <set> // for composition
+#include <boost/scoped_ptr.hpp>
 
 namespace gnash {
     namespace media {
@@ -298,7 +295,7 @@ public:
     /// gnash calls this to unpause audio
     virtual void unpause() { _paused=false; }
 
-    /// gnash calls this to unpause audio
+    /// return true if audio is paused
     bool isPaused() const { return _paused; }
 
     /// Plug an external InputStream into the mixer
@@ -388,7 +385,7 @@ public:
 
     /// Fetch mixed samples
     //
-    /// We run trough all the plugged InputStreams fetching decoded
+    /// We run through all the plugged InputStreams fetching decoded
     /// audio blocks and mixing them into the given output stream.
     ///
     /// @param to
@@ -408,37 +405,6 @@ public:
     ///
     virtual void fetchSamples(boost::int16_t* to, unsigned int nSamples);
 
-protected:
-
-
-    sound_handler()
-        :
-        _soundsStarted(0),
-        _soundsStopped(0),
-        _paused(false),
-        _muted(false),
-        _mediaHandler(media::MediaHandler::get()),
-        _volume(100),
-        _sounds(),
-        _inputStreams()
-    {
-        // for now, we rely on this being always available
-        assert(_mediaHandler);
-    }
-
-    /// Plug an InputStream to the mixer
-    //
-    /// @param in
-    ///     The InputStream to plug, ownership transferred
-    ///
-    virtual void plugInputStream(std::auto_ptr<InputStream> in);
-
-    /// Unplug all input streams
-    virtual void unplugAllInputStreams();
-
-    /// Does the mixer have input streams ?
-    bool hasInputStreams() const;
-
     /// Mix nSamples from inSamples to outSamples, with given volume
     //
     /// @param outSamples
@@ -465,7 +431,44 @@ protected:
     ///       with number of channels, at each fetching.
     ///
     virtual void mix(boost::int16_t* outSamples, boost::int16_t* inSamples,
-                unsigned int nSamples, float volume);
+                unsigned int nSamples, float volume) = 0;
+
+
+    /// Request to dump audio to the given filename
+    //
+    /// Every call to this function starts recording
+    /// to a new file, closing any existing other dump.
+    ///
+    void setAudioDump(const std::string& wavefile);
+
+protected:
+
+
+    sound_handler(media::MediaHandler* m)
+        :
+        _soundsStarted(0),
+        _soundsStopped(0),
+        _paused(false),
+        _muted(false),
+        _volume(100),
+        _sounds(),
+        _inputStreams(),
+        _mediaHandler(m)
+    {
+    }
+
+    /// Plug an InputStream to the mixer
+    //
+    /// @param in
+    ///     The InputStream to plug, ownership transferred
+    ///
+    virtual void plugInputStream(std::auto_ptr<InputStream> in);
+
+    /// Unplug all input streams
+    virtual void unplugAllInputStreams();
+
+    /// Does the mixer have input streams ?
+    bool hasInputStreams() const;
 
 private:
 
@@ -481,9 +484,6 @@ private:
     /// True if sound is muted
     bool _muted;
 
-    /// The registered MediaHandler at construction time
-    media::MediaHandler* _mediaHandler;
-
     /// Final output volume
     int _volume;
 
@@ -506,6 +506,8 @@ private:
     ///
     InputStreams _inputStreams;
 
+    media::MediaHandler* _mediaHandler;
+
     /// Unplug any completed input stream
     void unplugCompletedInputStreams();
 
@@ -568,22 +570,21 @@ private:
     unsigned int swfToOutSamples(const media::SoundInfo& sinfo,
                                           unsigned int swfSamples);
 
+    boost::scoped_ptr<WAVWriter> _wavWriter;
+
 };
 
 // TODO: move to appropriate specific sound handlers
 
 #ifdef SOUND_SDL
 /// @throw a SoundException if fails to initialize audio card.
-DSOEXPORT sound_handler* create_sound_handler_sdl();
-
-/// @throw a SoundException if fails to initialize audio card.
-DSOEXPORT sound_handler* create_sound_handler_sdl(const std::string& wave_file);
+DSOEXPORT sound_handler* create_sound_handler_sdl(media::MediaHandler* m);
 #elif defined(SOUND_AHI)
 /// @throw a SoundException if fails to initialize audio card.
-DSOEXPORT sound_handler* create_sound_handler_aos4();
-
-/// @throw a SoundException if fails to initialize audio card.
-DSOEXPORT sound_handler* create_sound_handler_aos4(const std::string& wave_file);
+DSOEXPORT sound_handler* create_sound_handler_aos4(media::MediaHandler* m);
+#elif defined(SOUND_MKIT)
+/// @throw a SoundException if fails to create node.
+DSOEXPORT sound_handler* create_sound_handler_mkit(media::MediaHandler* m);
 #endif
 
 } // gnash.sound namespace 
