@@ -1,6 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-//   Foundation, Inc
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -101,7 +101,7 @@ KDE_NO_CDTOR_EXPORT KlashPart::KlashPart (QWidget * wparent,
    m_started_emited (false) {
     //kdDebug () << "KlashPart(" << this << ")::KlashPart ()" << endl;
     bool show_fullscreen = false;
-    // FIXME what is the right replacement for this? setInstance (KlashFactory::instance (), true);
+    setComponentData (*KlashFactory::instance ());
     KAction *playact = new KAction(this);
     playact->setText(i18n("P&lay"));
     connect(playact, SIGNAL(triggered()), this, SLOT(play()));
@@ -468,7 +468,7 @@ KDE_NO_EXPORT bool KlashLiveConnectExtension::call
     if (!entry)
         return false;
     kdDebug () << "[01;35mentry[00m " << entry->name << endl;
-    for (unsigned int i = 0; i < args.size (); ++i) {
+    for (int i = 0; i < args.size (); ++i) {
         kdDebug () << "      " << args[i] << endl;
     }
     //Klash::View * view = static_cast <Klash::View*> (player->widget ());
