@@ -1,6 +1,7 @@
 // plugin.cpp:  Windows "win32" flash player Mozilla plugin, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+//   2011 Free Software Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -393,7 +394,6 @@ nsPluginInstance::threadMain(void)
 	PR_Lock(playerLock);
 
     // Initialize Gnash core library.
-    gnash::gnashInit();
     DBG("Gnash core initialized.\n");
  
     // Init logfile.
@@ -433,7 +433,7 @@ nsPluginInstance::threadMain(void)
 
     gnash::movie_definition* md = NULL;
     try {
-        md = gnash::create_library_movie(url, _url.c_str(), false);
+        md = gnash::createMovie(url, _url.c_str(), false);
     } catch (const gnash::GnashException& err) {
         md = NULL;
     }
@@ -480,12 +480,12 @@ nsPluginInstance::threadMain(void)
     md->completeLoad();
     DBG("Movie loaded.\n");
 
-    std::auto_ptr<gnash::movie_instance> mr(md->create_movie_instance());
+    std::auto_ptr<gnash::Movie> mr(md->createMovie());
     mr->setVariables(_flashVars);
     root.setRootMovie(mr.release());
     root.set_display_viewport(0, 0, _width, _height);
     root.set_background_alpha(1.0f);
-    gnash::movie_instance* mi = root.getRootMovie();
+    gnash::Movie* mi = root.getRootMovie();
     DBG("Movie instance created.\n");
 
     ShowWindow(_window, SW_SHOW);
