@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,24 +16,19 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
-
 #ifndef GNASH_DUMMYMOVIEDEFINITION_H
 #define GNASH_DUMMYMOVIEDEFINITION_H
 
-#include "movie_definition.h" // for inheritance
-#include "rect.h" // for composition
-#include "movie_instance.h" // for create_movie_instance
+#include "SWFMovieDefinition.h" // for inheritance
+#include "SWFRect.h" // for composition
+#include "SWFMovie.h" // for createMovie
+#include "Global_as.h"
+#include "namedStrings.h"
 
+#include <vector>
 #include <string>
 #include <memory> // for auto_ptr
 
-// Forward declarations
-namespace gnash {
-	class bitmap_character_def;
-}
 
 namespace gnash
 {
@@ -43,14 +39,14 @@ namespace gnash
 /// methods of movie_definition by returning user-defined
 /// values for version/size/frame rate etc..
 ///
-/// The create_instance function will return the same
+/// The createMovie function will return the same
 /// object created by createEmptyMovieClip() calls
 /// (an empty movieclip... still to be designed)
 ///
-class DummyMovieDefinition : public movie_definition
+class DummyMovieDefinition : public SWFMovieDefinition
 {
 	int _version;
-	rect _framesize;
+	SWFRect _framesize;
 	size_t _framecount;
 	std::vector<PlayList> _playlist;
 	float _framerate;
@@ -70,14 +66,15 @@ public:
 	///  - 0 bytes (for get_bytes_loaded()/get_bytes_total())
 	///  - empty url
 	///
-	DummyMovieDefinition()
+	DummyMovieDefinition(const RunResources& ri)
 		:
+        SWFMovieDefinition(ri),
 		_version(6),
 		_framesize(0, 0, 640*20, 480*20),
 		_framecount(1),
 		_playlist(_framecount),
 		_framerate(12),
-		_url()
+		_url("http://www.gnu.org/software/gnash")
 	{
 	}
 
@@ -87,26 +84,31 @@ public:
 	/// All but the target version will be initialized
 	/// exactly as with the default constructor.
 	///
-	DummyMovieDefinition(int version)
+	DummyMovieDefinition(const RunResources& ri, int version)
 		:
+        SWFMovieDefinition(ri),
 		_version(version),
 		_framesize(0, 0, 640*20, 480*20),
 		_framecount(1),
 		_playlist(_framecount),
 		_framerate(12),
-		_url()
+		_url("http://www.gnu.org/software/gnash")
 	{
 	}
 
+    virtual bool ensure_frame_loaded(size_t) const {
+        return true;
+    }
+
 	virtual int	get_version() const {
 		return _version;
 	}
 
-	virtual float	get_width_pixels() const {
+	virtual size_t get_width_pixels() const {
 		return _framesize.width()/20;
 	}
 
-	virtual float	get_height_pixels() const {
+	virtual size_t get_height_pixels() const {
 		return _framesize.height()/20;
 	}
 
@@ -118,11 +120,7 @@ public:
 		return _framerate;
 	}
 
-	virtual const rect& get_frame_size() const {
-		return _framesize;
-	}
-
-	virtual const rect& get_bound() const {
+	virtual const SWFRect& get_frame_size() const {
 		return _framesize;
 	}
 
@@ -134,16 +132,11 @@ public:
 		return 0;
 	}
 	
-	/// Create a playable sprite instance from a def.
-	virtual sprite_instance* create_instance()
-	{
-		return create_movie_instance();
-	}
-
 	/// Create a playable movie instance from a def.
-	virtual movie_instance* create_movie_instance()
+	virtual Movie* createMovie(Global_as& gl, DisplayObject* parent=NULL)
 	{
-		return new movie_instance(this, NULL);
+        as_object* o = getObjectWithPrototype(gl, NSV::CLASS_MOVIE_CLIP);
+		return new SWFMovie(o, this, parent);
 	}
 	
 	virtual const PlayList& get_playlist(size_t frame_number) const
