@@ -4,9 +4,10 @@ use Time::HiRes;
 
 $SIG{PIPE}='IGNORE';
 
-$m=new IO::Socket::INET(Listen=>1,LocalPort=>2229);
-$O=new IO::Select($m);
+my $m=new IO::Socket::INET(Listen=>1,LocalPort=>2229,Reuse=>1,Proto=>'tcp');
+my $O=new IO::Select($m);
 
+$verbose = ( $ARGV[0] eq '-v' ) ? 1 : 0;
 
 $/ = "\0";
 
@@ -20,7 +21,7 @@ while (@S = $O->can_read) {
             my $R=sysread($_, $i, 16000);
             
             # Log message received:
-            print "XmlSocketServer: received \"$i\"\n";
+            print "XmlSocketServer: received \"$i\"\n" if $verbose;
             
             if ($R==0) {
                 $T=syswrite($_, "\n", 16000);
@@ -32,17 +33,23 @@ while (@S = $O->can_read) {
             
                 # Sleep a bit before sending a reply to mimic web traffic
                 # (well, sort of).
-                Time::HiRes::sleep(0.5);
-                print "XmlSocketServer: sending \"$i\" \n";
+                Time::HiRes::sleep(0.1);
+                print "XmlSocketServer: sending \"$i\" \n" if $verbose;
               
                 $i =~ s/\*NEWLINE\*/\n/g;
                 $i =~ s/\*NULL\*/\0/g;
 
-                foreach $C($O->handles) {
-                    $T=syswrite($C, $i, 16000);
-                }
+                $T=syswrite($_, $i, 16000);
+            }
+
+            if ($i =~ m/closeNow/) {
+                print("XmlSocketServer: closing...\n") if $verbose;
+                close($C) || die "Could not close reading Socket";
             }
         }
     }
 }
 
+close($m) || die "Could not close server Socket";
+print("XmlSocketServer: salut!\n") if $verbose;
+
