@@ -1,3 +1,21 @@
+/*
+ *   Copyright (C) 2005, 2006, 2007, 2009, 2010,
+ *   2011 Free Software Foundation, Inc.
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ */ 
 #ifndef _CHECK_H_
 #define _CHECK_H_
 
@@ -9,8 +27,7 @@
 #include <iostream>
 #include <string>
 
-// TODO: ensure dejagnu.h defines xpass/xfail too (seems a new feature :()
-#undef HAVE_DEJAGNU_H // missing xcheck/xfail !
+#define HAVE_DEJAGNU_H 1 // we ship our own now...
 #ifdef HAVE_DEJAGNU_H
 #include "dejagnu.h"
 
