@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include "movie_definition.h"
 #include "ManualClock.h"
 #include "RunResources.h"
+#include "StreamProvider.h"
 
 #include <iostream>
 #include <sstream>
@@ -48,9 +49,11 @@ main(int /*argc*/, char** /*argv*/)
     dbglogfile.setVerbosity();
     
     // Initialize gnash lib
-    gnashInit();
     
-    RunResources ri("");
+    RunResources ri;
+    const URL url("");
+    ri.setStreamProvider(
+            boost::shared_ptr<StreamProvider>(new StreamProvider(url, url)));
     
     // Initialize a VM
     boost::intrusive_ptr<movie_definition> md5(new DummyMovieDefinition(ri, 5));
@@ -60,9 +63,7 @@ main(int /*argc*/, char** /*argv*/)
     movie_root stage(*md5, clock, ri);
     
     MovieClip::MovieVariables v;
-    // We pass 'v' twice, as the second one is for scriptable
-    // Variables, which isn't fully implemented yet.
-    stage.init(md5.get(), v, v);
+    stage.init(md5.get(), v);
     
     DisplayList dlist1;
     
@@ -75,25 +76,20 @@ main(int /*argc*/, char** /*argv*/)
     MovieClip* root = const_cast<Movie*>(&stage.getRootMovie());
     
     // just a couple of DisplayObjects
-    as_object* ob1 = getGlobal(*getObject(root)).createObject();
-    as_object* ob2 = getGlobal(*getObject(root)).createObject();
+    as_object* ob1 = createObject(getGlobal(*getObject(root)));
+    as_object* ob2 = createObject(getGlobal(*getObject(root)));
     
-    boost::intrusive_ptr<DisplayObject> ch1 ( new DummyCharacter(ob1, root) );
-    boost::intrusive_ptr<DisplayObject> ch2 ( new DummyCharacter(ob2, root) );
+    DisplayObject* ch1 ( new DummyCharacter(ob1, root) );
+    DisplayObject* ch2 ( new DummyCharacter(ob2, root) );
     
-    dlist1.placeDisplayObject( ch1.get(), 1);
-    dlist1.placeDisplayObject( ch2.get(), 2);
+    dlist1.placeDisplayObject(ch1, 1);
+    dlist1.placeDisplayObject(ch2, 2);
     
     check(dlist1 != dlist2);
     
-    dlist2.placeDisplayObject( ch2.get(), 1);
-    dlist2.placeDisplayObject( ch1.get(), 2);
+    dlist2.placeDisplayObject(ch2, 1);
+    dlist2.placeDisplayObject(ch1, 2);
     
-    // Resort dlist1 as depth of it's chars has been changed
-    // by placeDisplayObject calls above :/
-    dlist1.sort();
-    
-    check_equals(dlist1, dlist2);
     
 }
 
