@@ -1,5 +1,6 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+ *   2011 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -79,26 +80,28 @@ main(int /*argc*/, char** /*argv*/)
 	// check that the pixel under the mouse is white
 	check_pixel(100, 30, 2, rgba(255,255,255,255), 2);
 
-	string_table& st = VM::get().getStringTable();
+	VM& vm = tester.vm();
+	const ObjectURI& mousedown = getURI(vm, "mousedown");
+	const ObjectURI& mouseup = getURI(vm, "mouseup");
 
-	getObject(root)->get_member(st.find("mousedown"), &tmp);
+	getObject(root)->get_member(mousedown, &tmp);
 	check(tmp.is_undefined());
-	getObject(root)->get_member(st.find("mouseup"), &tmp);
+	getObject(root)->get_member(mouseup, &tmp);
 	check(tmp.is_undefined());
 
 	// Note that we are *not* on an active entity !
 	tester.pressMouseButton();
 
-	getObject(root)->get_member(st.find("mousedown"), &tmp);
-	check_equals(tmp.to_number(), 1);
-	check ( ! getObject(root)->get_member(st.find("mouseup"), &tmp) );
+	getObject(root)->get_member(mousedown, &tmp);
+	check_equals(toNumber(tmp, vm), 1);
+	check ( ! getObject(root)->get_member(mouseup, &tmp) );
 
 	tester.depressMouseButton();
 
-	getObject(root)->get_member(st.find("mousedown"), &tmp);
-	check_equals(tmp.to_number(), 1);
-	getObject(root)->get_member(st.find("mouseup"), &tmp);
-	check_equals(tmp.to_number(), 1);
+	getObject(root)->get_member(mousedown, &tmp);
+	check_equals(toNumber(tmp, vm), 1);
+	getObject(root)->get_member(mouseup, &tmp);
+	check_equals(toNumber(tmp, vm), 1);
 
 	tester.advance();
 
@@ -154,8 +157,8 @@ main(int /*argc*/, char** /*argv*/)
 	// Note that we are *not* on an active entity !
 	tester.pressMouseButton();
 
-	getObject(root)->get_member(st.find("mousedown"), &tmp);
-	check_equals(tmp.to_number(), 5);
+	getObject(root)->get_member(mousedown, &tmp);
+	check_equals(toNumber(tmp, vm), 5);
 
 }
 
