@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,8 +21,8 @@
 #define INPUT_FILENAME "masks_test2.swf"
 
 #include "MovieTester.h"
-#include "sprite_instance.h"
-#include "character.h"
+#include "MovieClip.h"
+#include "DisplayObject.h"
 #include "DisplayList.h"
 #include "log.h"
 
@@ -47,7 +47,7 @@ main(int /*argc*/, char** /*argv*/)
   rgba red(255,0,0,255);  
   rgba white(255, 255, 255, 255);
 
-  sprite_instance* root = tester.getRootMovie();
+  MovieClip* root = tester.getRootMovie();
   assert(root);
   check_equals(root->get_frame_count(), 3);
 
