@@ -1,5 +1,6 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010,
+ *   2011 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -64,7 +65,7 @@ main(int /*argc*/, char** /*argv*/)
 	check_equals(root->getPlayState(), MovieClip::PLAYSTATE_STOP);
 	check_equals(root->get_current_frame(), 1); // 0-based
 	check_equals(root->getDisplayList().size(), 9);
-	root->getDisplayList().dump();
+    std::cout << root->getDisplayList() << "\n";
 	check( tester.findDisplayItemByName(*root, "staticmc2") );
 	check( tester.findDisplayItemByName(*root, "staticmc3") );
 	check( tester.findDisplayItemByName(*root, "staticmc4") );
@@ -237,7 +238,7 @@ main(int /*argc*/, char** /*argv*/)
 	check_pixel(48,232, 2, yellow, 2);
 	// 80,232 = white (yellow not covered by its red mask)
 	check( invalidated.contains(80, 232) );
-	check_pixel(80,232, 2, white, 2);
+	check_pixel(80,232, 2, white, 3);
 
 	// 214,232 = white (green not covered by its cyan mask)
 	check( invalidated.contains(214, 232) );
@@ -287,7 +288,7 @@ main(int /*argc*/, char** /*argv*/)
 	check_pixel(48,232, 2, yellow, 2);
 	// 80,232 = white (yellow not covered by its red mask)
 	check( invalidated.contains(80, 232) );
-	check_pixel(80,232, 2, white, 2);
+	check_pixel(80,232, 2, white, 3);
 
 	tester.movePointerTo(14,232); // on the red mask
 	check( tester.isMouseOverMouseEntity() ) // the mask is still sensible to mouse
