@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2007, 2009, 2010 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2009, 2010, 2011 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -69,7 +69,7 @@ main(int argc, char** argv)
   SWFMovie mo;
   SWFMovieClip  mc1, mc2, mc3, dejagnuclip;
   SWFShape  sh1, sh2, sh3;
-  SWFDisplayItem it1, it2, it3, it4;
+  SWFDisplayItem it3, it4;
   const char *srcdir=".";
 
   if ( argc>1 ) srcdir=argv[1];
@@ -133,7 +133,7 @@ main(int argc, char** argv)
 #if OUTPUT_VERSION > 6
   // succeed in swf7 and swf8
   check_equals(mo, "clip1.constructor", "MovieClip");
-#else if OUTPUT_VERSION == 6
+#elif OUTPUT_VERSION == 6
   check_equals(mo, "clip1.constructor", "theClass1");
 #endif
   check_equals(mo, "clip1.__constructor__", "theClass1");
